//
//  TPAManager.h
//  ThePerfectAppLib
//
//  Created by Peder Toftegaard Olsen on 05/06/12.
//  Copyright (c) 2012 Trifork GmbH. All rights reserved.
//

#import <Foundation/Foundation.h>

#ifndef HAS_THE_PERFECT_APP_LIBRARY
#define HAS_THE_PERFECT_APP_LIBRARY 1
#endif


typedef enum TPALoggingDestination
{
    TPALoggingDestinationNone = 0x00,
    TPALoggingDestinationConsole = 1 << 0,
    TPALoggingDestinationRemote = 1 << 1,
} TPALoggingDestination;

/**
 * The TPAManager is responsible for handling update notifications and crash reporting to a TPA server.
 * To enable either a baseURL and projectUuid must be specified.
 */
@interface TPAManager : NSObject

/** Get the one and only TPAManager instance. */
+(TPAManager *)sharedManager;


#pragma mark - TPA server configuration

/**
 * @param baseUrl The base URL of the TPA server. Usually of the form https://server/.
 * @param projectUuid Project UUID as it is known to the TPA server.
 */
- (void)setBaseUrl:(NSString *)baseUrl projectUuid:(NSString *)projectUuid;


#pragma mark - Update notifications

/** Enable update notifications by polling the TPAW server when the app becomes active. Cannot be enabled for apps distributed via the App Store. Disabled by default. */
@property (nonatomic, readwrite, getter = isUpdateNotificationsEnabled) BOOL updateNotificationsEnabled;


#pragma mark - Crash reporting

/** Enable crash reporting by uploading crash reports to the TPAW server on next restart of the app after a crash. Disabled by default. */
@property (nonatomic, readwrite, getter = isCrashReportingEnabled) BOOL crashReportingEnabled;


#pragma mark - Session Recording

/** If enabled, session start/end events will be sent to the TPA server containing basic information about the device. Disabled by default. */
@property (nonatomic, getter = isSessionRecordingEnabled) BOOL sessionRecordingEnabled;


#pragma mark - Remote Logging

/**
 * Bitmap from TPALoggingDestination enum.
 * Defaults to TPALoggingDestinationConsole, except for apps distributed via the App Store where it is TPALoggingDestinationNone.
 * Value cannot be changed for apps distrbuted via the App Store.
 */
@property (nonatomic) NSUInteger loggingDestinations;

/**
 * Log message to loggingDestinations.
 */
- (void)log:(NSString *)format, ...;

/**
 * Log message to loggingDestinations.
 */
- (void)log:(NSString *)format args:(va_list)args;


#pragma mark - Esoteric stuff

/**
 * Enable debug logging. Will output extra information about the TPA library to the destinations specified in loggingDestinations.
 * Cannot be enabled for apps distributed via the App Store.
 * Disabled by default.
 */
@property (nonatomic, readwrite, getter = isDebugLoggingEnabled) BOOL debugLoggingEnabled;

/**
 * Show the update view. Same as calling [TPAManager showUpdateViewReload:NO].
 */
- (void)showUpdateView;

/**
 * Show the update view.
 *
 * @param reload Force reloading version history from TPAW server even if it has already been loaded.
 */
- (void)showUpdateViewReload:(BOOL)reload;

@end


/**
 * Logs message to the destinations specified by [TPAManager loggingDestinations].
 */
extern void TPALog(NSString *format, ...);

/**
 * Logs message to the destinations specified by [TPAManager loggingDestinations].
 */
extern void TPALogv(NSString *format, va_list args);
