//
//  AppSettings.m
//  LEGODeviceDemo
//
//  Created by Bartlomiej Hyzy on 19/02/2015.
//  Copyright (c) 2015 lego. All rights reserved.
//

#import "AppSettings.h"


static NSString *const UserDefaultsKeyEnableAutoReconnect = @"EnableAutoReconnect";
static NSString *const UserDefaultsKeyRSSIThreshold = @"RSSIThreshold";

static BOOL const DefaultEnableAutoReconnect = NO;
static NSInteger const DefaultRSSIThreshold = -150;


@interface AppSettings ()

@property (strong, nonatomic) NSUserDefaults * userDefaults;

@end


@implementation AppSettings

+ (AppSettings *)defaultSettings
{
    static AppSettings *instance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [[self alloc] initWithUserDefaults:[NSUserDefaults standardUserDefaults]];
    });
    return instance;
}

- (instancetype)initWithUserDefaults:(NSUserDefaults *)userDefaults
{
    self = [super init];
    if (self) {
        _userDefaults = userDefaults;
    }
    return self;
}

- (BOOL)autoReconnectEnabled
{
    if ([self.userDefaults valueForKey:UserDefaultsKeyEnableAutoReconnect]) {
        return [self.userDefaults boolForKey:UserDefaultsKeyEnableAutoReconnect];
    } else {
        return DefaultEnableAutoReconnect;
    }
}

- (void)setAutoReconnectEnabled:(BOOL)autoReconnectEnabled
{
    [self.userDefaults setBool:autoReconnectEnabled forKey:UserDefaultsKeyEnableAutoReconnect];
    [self.userDefaults synchronize];
}

- (NSInteger)rssiThreshold
{
    if ([self.userDefaults valueForKey:UserDefaultsKeyRSSIThreshold]) {
        return [self.userDefaults integerForKey:UserDefaultsKeyRSSIThreshold];
    } else {
        return DefaultRSSIThreshold;
    }
}

- (void)setRssiThreshold:(NSInteger)rssiThreshold
{
    [self.userDefaults setInteger:rssiThreshold forKey:UserDefaultsKeyRSSIThreshold];
    [self.userDefaults synchronize];
}

@end
