//
// Created by Søren Toft Odgaard on 24/04/14.
// Copyright (c) 2014 LEGO. All rights reserved.
//

#import <AudioToolbox/AudioServices.h>

#import "DeviceServiceCell.h"
#import "LEIO.h"
#import "LEInputFormat.h"
#import "LEMotor.h"


@interface DeviceServiceCell ()
@end

@implementation DeviceServiceCell

+ (DeviceServiceCell *)deviceServiceCellWithService:(LEService *)service
{
    DeviceServiceCell *cell = [[self alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:nil];
    [cell setService:service];
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    return cell;
}



- (void)setService:(LEService *)service
{
    _service = service;
    [_service addDelegate:self];
    [self setValue:_service.valueData];
    self.detailTextLabel.text = service.connectInfo.description;
}


- (void)setValue:(NSData *)value
{
    switch (_service.inputFormat.unit) {
        case LEInputFormatUnitRaw:
            self.textLabel.text = [NSString stringWithFormat:@"%@ (raw): %@", self.service.serviceName, value.description];
            break;
        case LEInputFormatUnitPercentage:
            self.textLabel.text = [NSString stringWithFormat:@"%@ (pct): %@", self.service.serviceName, value.description];
            break;
        case LEInputFormatUnitSI:
            self.textLabel.text = [NSString stringWithFormat:@"%@ (SI): %@", self.service.serviceName, value.description];
            break;
        case LEInputFormatUnitUnknown:
            self.textLabel.text = [NSString stringWithFormat:@"%@ (?): %@", self.service.serviceName, value.description];
            break;
    }
    [self setNeedsLayout];
}

#pragma mark - LEServiceDelegate
- (void)service:(LEService *)service didUpdateValueDataFrom:(NSData *)oldValue to:(NSData *)newValue
{
    [self setValue:_service.valueData];
}

@end