//
// Created by Søren Toft Odgaard on 24/04/14.
// Copyright (c) 2014 LEGO. All rights reserved.
//

#import "DeviceTableViewCell.h"
#import "LEDevice.h"
#import "LEBluetoothDevice.h"
#import "LEDeviceManager.h"


@implementation DeviceTableViewCell {

    IBOutlet UILabel *_deviceNameLabel;
    IBOutlet UILabel *_averageRSSILabel;
    IBOutlet UIImageView *_buttonPressedImageView;
    IBOutlet UIImageView *_lowVoltageImageView;

    LEDevice *_device;
}
- (void)configureWithDevice:(LEDevice *)device
{
    _device = device;
    _deviceNameLabel.text = device.name;
    _buttonPressedImageView.hidden = !device.buttonPressed;
    _lowVoltageImageView.hidden = !device.lowVoltage;

    self.accessoryType = UITableViewCellAccessoryNone;
    self.accessoryView = nil;

    [self updateDeviceNameLabelStyle:device];
    [self updateDetailLabel:device];

    if (device.connectState == LEDeviceStateConnecting || device.connectState == LEDeviceStateInterrogating) {
        UIButton *button = [[UIButton alloc] init];
        [button setTitle:@"Cancel" forState:UIControlStateNormal];
        [button setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        [button addTarget:self action:@selector(didTouchCancel) forControlEvents:UIControlEventTouchUpInside];
        self.accessoryView = button;
    }

    if (device.connectState == LEDeviceStateInterrogationFinished) {
        self.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    }

}

- (void)updateDeviceNameLabelStyle:(LEDevice *)device
{
    switch (device.connectState) {
        case LEDeviceStateInterrogationFinished:
            _deviceNameLabel.textColor = [UIColor blackColor];
            _deviceNameLabel.font = [UIFont boldSystemFontOfSize:16];
            break;
        case LEDeviceStateDisconnectedAdvertising:
            _deviceNameLabel.textColor = [UIColor blackColor];
            _deviceNameLabel.font = [UIFont systemFontOfSize:16];
            break;
        case LEDeviceStateDisconnectedNotAdvertising: //intentionally fall through
        case LEDeviceStateConnecting:
        case LEDeviceStateInterrogating:
            _deviceNameLabel.textColor = [UIColor lightGrayColor];
            _deviceNameLabel.font = [UIFont systemFontOfSize:16];
            break;
    }
}


- (void)updateDetailLabel:(LEDevice *)device
{
    switch (device.connectState) {
        case LEDeviceStateDisconnectedAdvertising: {
            NSString *rssiString = ((LEBluetoothDevice *) device).RSSI ? ((LEBluetoothDevice *) device).RSSI.stringValue : @"?";
            _averageRSSILabel.text = [NSString stringWithFormat:@"Average RSSI: %@", rssiString];
        }
            break;
        case LEDeviceStateDisconnectedNotAdvertising:
            _averageRSSILabel.text = @"No longer advertising";
            break;
        case LEDeviceStateConnecting:
            _averageRSSILabel.text = @"Connecting";
            break;
        case LEDeviceStateInterrogating:
            _averageRSSILabel.text = @"Interrogating";
            break;
        case LEDeviceStateInterrogationFinished:
            _averageRSSILabel.text = @"Connected";
            break;
    }
}

- (void)didTouchCancel
{
    [[LEDeviceManager sharedDeviceManager] cancelDeviceConnection:_device];
}

@end