//
// Created by Søren Toft Odgaard on 22/05/14.
// Copyright (c) 2014 LEGO. All rights reserved.
//

#import "DocumentationCodeSnippets.h"
#import "LEDeviceManager.h"
#import "LEService.h"
#import "LETiltSensor.h"
#import "LEInputFormat.h"
#import "LEGenericService.h"
#import "LEDataFormat.h"


@implementation DocumentationCodeSnippets {

}


- (void)startScanningForDevices
{
    [[LEDeviceManager sharedDeviceManager] addDelegate:self];
    [[LEDeviceManager sharedDeviceManager] scan];
}

#pragma mark - LEDeviceManagerDelegate

- (void)deviceManager:(LEDeviceManager *)manager deviceDidAppear:(LEDevice *)device
{
    NSLog(@"Did dicover device %@", device.name);

    [device addDelegate:self];
    [[LEDeviceManager sharedDeviceManager] connectToDevice:device];
}


- (void)connectToDevice:(LEDevice *)device
{
    [device addDelegate:self];
    [[LEDeviceManager sharedDeviceManager] connectToDevice:device];
}

#pragma mark - LEDeviceManagerDelegate

- (void)deviceManager:(LEDeviceManager *)manager didConnectToDevice:(LEDevice *)device
{
    NSLog(@"Did successfully connect to device: %@", device);
}

#pragma mark - LEDeviceDelegate

- (void)device:(LEDevice *)device didAddService:(LEService *)service
{
    NSLog(@"Device did add service: %@", service.serviceName);
}

- (void)startUsingService:(LEService *)service
{
    if ([service isKindOfClass:[LETiltSensor class]]) {
        LETiltSensor *tiltSensor = (LETiltSensor *) service;
        [tiltSensor addDelegate:self];
    }
}

#pragma mark - LETiltSensorDelegate

- (void)tiltSensor:(LETiltSensor *)sensor didUpdateAngleFrom:(LETiltSensorAngle)oldAngle to:(LETiltSensorAngle)newAngle
{
    NSLog(@"Tilt sensor changed agnle to %f, %f", newAngle.x, newAngle.y);
}


- (void)useGenericService:(LEService *)service
{
    uint8_t temperatureSensorTypeNumber = 0;

    if ([service isKindOfClass:[LEGenericService class]] &&
            service.connectInfo.type == temperatureSensorTypeNumber) {

        LEGenericService *temperatureSensor = (LEGenericService *) service;

        //The temperature sensor is yet unknown to the SDK (there is no
        //LETemperatureSensor class) so we need to configure
        //the service ourselves.

        //As a generic sensor does not how a defaultInputFormat defined
        //we must create one and send it to the device.
        //Look in documentation for the temperature sensor to se which modes it supports.
        LEInputFormat *inputFormat = [LEInputFormat
                inputFormatWithConnectID:service.connectInfo.connectID
                typeID:service.connectInfo.type
                mode:0 //See the documentation for the sensor for supported modes
                deltaInterval:1 //Receive updates when the value changes with delta 1
                unit:LEInputFormatUnitSI
                notificationsEnabled:YES];

        //Tell the device to configure the sensor with the input format
        [temperatureSensor updateInputFormat:inputFormat];


        //We know from the documentation that the temperature sensor produces readings in
        //kelvin as a 4 byte floats when in mode 0 with unit set to SI.
        LEDataFormat *dataFormat = [LEDataFormat
                formatWithModeName:@"Kelvin"
                mode:0 //must match the mode for the inputFormat
                unit:LEInputFormatUnitSI //must match the unit for the inputFormat
                sizeOfDataSet:4 //a 4 byte float
                dataSetCount:1]; //only one value in the data set (the temperature)

        [temperatureSensor addValidDataFormat:dataFormat];

        //Now, add a delegate to receive notifications when the service has a new
        //temperature reading
        [temperatureSensor addDelegate:self];
    }

}

- (void)service:(LEService *)service didUpdateValueDataFrom:(NSData *)oldValue to:(NSData *)newValue
{

    //As we have defined a valid data format stating that the received value can be parsed as a 4 byte
    //float we can use the convenience method to retrieve the value as a float.
    Float32 temperatureReading = service.valueAsFloat;

}

- (void)createTiltSensorDataFormat
{
    LEDataFormat *tiltSensorFormat = [LEDataFormat formatWithModeName:@"Angle" mode:0 unit:LEInputFormatUnitSI sizeOfDataSet:3 dataSetCount:4];
}


@end