//
// Created by Søren Toft Odgaard on 9/3/13.
// Copyright (c) 2013 LEGO. All rights reserved.
//
// To change the template use AppCode | Preferences | File Templates.
//

#import "HubSimulator.h"
#import "LEIO.h"
#import "LEConnectInfo.h"
#import "LEBluetoothServiceDefinition.h"
#import "LEDeviceServiceDefinition.h"

#define IO_CONNECT_ID_INDEX 0
#define IO_HUB_PORT_INDEX 1
#define IO_TYPE_INDEX 2

static NSString *const kHubNameUserDefaultsKey = @"HubName";

static NSString *const kHubColorUserDefaultsKey = @"HubColor";

@implementation HubSimulator {
    BOOL _sensor0Attached;
    BOOL _sensor1Attached;
    BOOL _sensor2Attached;
    BOOL _sensor3Attached;
    LEIOType _sensor2Type;
    LEIOType _sensor3Type;
    LEDeviceServiceDefinition *_serviceDefinition;
}


- (id)init {
    self = [super initWithServiceDefinition:[LEBluetoothServiceDefinition deviceServiceDefinition]];
    if (self) {
        _serviceDefinition = (LEDeviceServiceDefinition *) self.serviceDefinition;
        _sensor0Attached = YES;
                _sensor1Attached = YES;
                _sensor2Attached = NO;
                _sensor3Attached = NO;
    }

    return self;
}

- (CBMutableCharacteristic *)hubNameCharacteristic
{
    return [self characteristicWithDefinition:_serviceDefinition.deviceName];
}

- (CBMutableCharacteristic *)ioCharacteristic
{
    return [self characteristicWithDefinition:_serviceDefinition.attachedIO];
}


- (void)setName:(NSString *)name {
    [[NSUserDefaults standardUserDefaults] setValue:name forKey:kHubNameUserDefaultsKey];
}

- (NSString *)name {
    NSString *name = [[NSUserDefaults standardUserDefaults] stringForKey:kHubNameUserDefaultsKey];
    if (!name) {
        name = @"Hub Sim";
        [self setName:name];
    }
    return name;
}

- (NSData *)ioValues {
    NSMutableData *data = [NSMutableData new];
    if (_sensor0Attached) {
        [self addIOValueToData:data connectID:0 hubIndex:0 type:LEIOTypeVoltage];
    }
    if (_sensor1Attached) {
        [self addIOValueToData:data connectID:1 hubIndex:0 type:LEIOTypeCurrent];
    }
    if (_sensor2Attached) {
        [self addIOValueToData:data connectID:2 hubIndex:1 type:_sensor2Type];
    }
    if (_sensor3Attached) {
        [self addIOValueToData:data connectID:3 hubIndex:2 type:_sensor3Type];
    }
    return data;
}

- (void)addIOValueToData:(NSMutableData *)data connectID:(unsigned char)index hubIndex:(unsigned char)port type:(LEIOType)type {
    unsigned char ioData[3];
    ioData[IO_CONNECT_ID_INDEX] = index;
    ioData[IO_HUB_PORT_INDEX] = port;
    ioData[IO_TYPE_INDEX] = (NSUInteger) type;
    [data appendBytes:ioData length:3];
}


- (void)attached:(BOOL)attached type:(LEIOType)type index:(NSUInteger)index {
    if (index == 2) {
        _sensor2Attached = attached;
        _sensor2Type = attached ? type : LEIOTypeGeneric;
    } else if (index == 3) {
        _sensor3Attached = attached;
        _sensor3Type = attached ? type : LEIOTypeGeneric;
    }

}


@end