//
//  LEDevice+UI.m
//  LEGODeviceDemo-OSX
//
//  Created by Bartlomiej Hyzy on 11/02/2015.
//  Copyright (c) 2015 lego. All rights reserved.
//

#import "LEDevice+UI.h"
#import "LEGODevice/LEBluetoothDevice.h"

@implementation LEDevice (UI)

- (NSString *)deviceStatus
{
    switch (self.connectState) {
        case LEDeviceStateDisconnectedAdvertising:
            return @"Advertising";
        case LEDeviceStateDisconnectedNotAdvertising:
            return @"No longer advertising";
        case LEDeviceStateConnecting:
            return @"Connecting";
        case LEDeviceStateInterrogating:
            return @"Interrogating";
        case LEDeviceStateInterrogationFinished:
            return @"Connected";
        default:
            return nil;
    }
}

- (NSNumber *)deviceRSSI
{
    if (![self isKindOfClass:[LEBluetoothDevice class]]) {
        return nil;
    }
    if (self.connectState != LEDeviceStateDisconnectedAdvertising) {
        return nil;
    }
    return ((LEBluetoothDevice *)self).RSSI;
}

- (BOOL)canChangeConnectionState
{
    return self.connectState == LEDeviceStateDisconnectedAdvertising ||
           self.connectState == LEDeviceStateInterrogationFinished;
}

- (NSString *)connectionStatus
{
    NSString *status;
    
    switch (self.connectState) {
        case LEDeviceStateDisconnectedAdvertising:
        case LEDeviceStateDisconnectedNotAdvertising:
            status = @"Connect";
            break;
        case LEDeviceStateConnecting:
            status = @"Connecting...";
            break;
        case LEDeviceStateInterrogating:
            status = @"Interrogating...";
            break;
        case LEDeviceStateInterrogationFinished:
            status = @"Disconnect";
            break;
    }
    
    return status;
}

+ (NSSet *)keyPathsForValuesAffectingValueForKey:(NSString *)key
{
    NSSet *keyPaths = [super keyPathsForValuesAffectingValueForKey:key];
    if ([@[@"deviceStatus", @"canChangeConnectionState", @"connectionStatus", @"deviceRSSI"] containsObject:key]) {
        keyPaths = [keyPaths setByAddingObject:@"connectState"];
    }
    if ([key isEqualToString:@"deviceRSSI"]) {
        keyPaths = [keyPaths setByAddingObject:@"RSSI"];
    }
    return keyPaths;
}

@end
