//
// Created by Søren Toft Odgaard on 24/04/14.
// Copyright (c) 2014 LEGO. All rights reserved.
//

#import "LogCollector.h"
#import "TPAManager.h"

NSString *const LogCollectorLoggedErrorNotification = @"DidWriteToLogCollector";

@implementation LogCollector {
    NSMutableAttributedString *_errorLog;
    NSMutableAttributedString *_debugLog;
}

+ (instancetype)sharedInstance
{
    static dispatch_once_t pred = 0;
    __strong static id _sharedObject = nil;
    dispatch_once(&pred, ^{
        _sharedObject = [[self alloc] init]; // or some other init method
    });
    return _sharedObject;
}

- (id)init
{
    self = [super init];
    if (self) {
        _errorLog = [[NSMutableAttributedString alloc] init];
        _debugLog = [[NSMutableAttributedString alloc] init];
    }

    return self;
}

- (void)clearLog
{
    _errorLog = [[NSMutableAttributedString alloc] init];
    _debugLog = [[NSMutableAttributedString alloc] init];
    _errorCount = 0;
    _warningCount = 0;
}


- (void)writeMessage:(NSString *)message logLevel:(LELoggerLevel)level
{
    NSAttributedString *str;

    if (level >= LELoggerLevelWarn) {
        str = [[NSAttributedString alloc] initWithString:message attributes:@{ NSForegroundColorAttributeName : [UIColor redColor]}];
    } else if (level == LELoggerLevelInfo) {
        str = [[NSAttributedString alloc] initWithString:message attributes:@{ NSForegroundColorAttributeName : [UIColor blueColor]}];
    } else {
        str = [[NSAttributedString alloc] initWithString:message];
    }

    if (level >= LELoggerLevelError) {
        _errorCount++;
    }
    if (level == LELoggerLevelWarn) {
        _warningCount++;
    }


    if (level >= LELoggerLevelWarn) {
        [_errorLog appendAttributedString:str];
        [_errorLog appendAttributedString:[[NSAttributedString alloc] initWithString:@"\n"]];
    } else {
        [_debugLog appendAttributedString:str];
        [_debugLog appendAttributedString:[[NSAttributedString alloc] initWithString:@"\n"]];
    }

#ifdef TPA_ENABLED
    if (level >= LELoggerLevelWarn) {
        TPALog(message);
    }
#endif

    if (level >= LELoggerLevelWarn) {
        [[NSNotificationCenter defaultCenter] postNotificationName:LogCollectorLoggedErrorNotification object:nil];
    }

    NSLog(message, nil);

}

- (NSAttributedString *)errorLog
{
    return _errorLog;
}

@end