//
//  MessageLogViewController.m
//  LEGODeviceDemo
//
//  Created by Bartlomiej Hyzy on 20/02/2015.
//  Copyright (c) 2015 lego. All rights reserved.
//

#import "MessageLogViewController.h"
#import "LogCollector.h"
#import "LogMessage+UI.h"
#import "MessageLogFilter.h"
#import <LEGODevice/LEGODevice.h>
#import <LEGODevice/LELogger+Project.h>


@interface MessageLogViewController () <NSTableViewDelegate>

@property (strong, nonatomic) LogCollector *logCollector;

@property (weak, nonatomic) IBOutlet NSTableView *tableView;
@property (weak, nonatomic) IBOutlet NSArrayController *listController;
@property (weak, nonatomic) IBOutlet NSArrayController *filterController;

@property (strong, nonatomic) NSArray *levelFilters;

@end


@implementation MessageLogViewController

#pragma mark - Factory Methods

+ (NSWindowController *)instantiateWindowController
{
    NSStoryboard *storyboard = [NSStoryboard storyboardWithName:@"Main" bundle:nil];
    return [storyboard instantiateControllerWithIdentifier:@"messageLogController"];
}

#pragma mark - View Appearance Methods

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.logCollector = [[LogCollector alloc] initWithLogger:[LELogger defaultLogger]];
    
    self.levelFilters = [MessageLogFilter levelFilters];
    self.filterController.selectionIndex = 1; // select "Debug" log level by default
}

#pragma mark - UI Actions

- (IBAction)clearLog:(id)sender
{
    [self.logCollector clear];
}

#pragma mark - NSTableViewDelegate

- (void)tableView:(NSTableView *)tableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
    LogMessage *message = self.listController.arrangedObjects[row];
    [cell setTextColor:[message color]];
}

- (void)tableView:(NSTableView *)tableView didAddRowView:(NSTableRowView *)rowView forRow:(NSInteger)row
{
    // check if the table view is scrolled to the bottom (with some small tolerance)
    NSView *contentView = self.tableView.enclosingScrollView.contentView;
    BOOL scrolledToBottom = ((contentView.bounds.origin.y + contentView.bounds.size.height) >= (self.tableView.bounds.size.height - 80));
    if (!scrolledToBottom) {
        return;
    }
    
    // scroll the table view so that the last, newly added row is visible
    NSInteger lastRow = self.tableView.numberOfRows - 1;
    if (lastRow >= 0) {
        [self.tableView scrollRowToVisible:lastRow];
    }
}

@end
