//
// Created by Søren Toft Odgaard on 08/05/14.
// Copyright (c) 2014 LEGO. All rights reserved.
//

#import "MotorDetailDataSource.h"
#import "LEMotor.h"
#import "Layout.h"


@implementation MotorDetailDataSource {
    UIStepper *_powerStepper;
    UISegmentedControl *_directionSegmentControl;

    UITableViewCell *_motorPowerCell;
    NSArray *_cells;

    LEMotor *_motor;
}

- (id)initWithService:(LEService *)service tableView:(UITableView *)tableView {
    self = [super initWithService:service tableView:tableView];
    if (self) {
        _motor = (LEMotor *) service;
        _cells = @[
                [self createAdjustPowerCell],
                [self createAdjustDirectionCell],
                [self createBrakeCell],
                [self createDriftCell]
        ];
        _motorPowerCell = _cells.firstObject;

        [self updateMotorPowerLabel];
        _powerStepper.value = _motor.power;
    }


    return self;
}


- (void)didTouchBrake:(id)sender
{
    [_motor brake];
    _powerStepper.value = 0.0;
    [self updateMotorPowerLabel];
}

- (void)didTouchDrift:(id)sender
{
    [_motor drift];
    _powerStepper.value = 0.0;
    [self updateMotorPowerLabel];
}


- (void)updateMotorPowerLabel
{
    switch (_motor.direction) {
        case LEMotorDirectionDrifting:
            _motorPowerCell.textLabel.text = @"Drifting";
            break;
        case LEMotorDirectionBraking:
            _motorPowerCell.textLabel.text = @"Braking";
            break;
        case LEMotorDirectionLeft:
            _motorPowerCell.textLabel.text = [NSString stringWithFormat:@"Motor left %lu", (unsigned long) _motor.power];
            break;
        case LEMotorDirectionRight:
            _motorPowerCell.textLabel.text = [NSString stringWithFormat:@"Motor right %lu", (unsigned long) _motor.power];
            break;
    }
}

- (void)updateMotorPower
{

    [_motor runInDirection:[self directionFromSegmentControl] power:_powerStepper.value];
    [self updateMotorPowerLabel];
}

- (LEMotorDirection)directionFromSegmentControl
{
    return (_directionSegmentControl.selectedSegmentIndex == 0)? LEMotorDirectionLeft : LEMotorDirectionRight;
}


#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return [super numberOfSectionsInTableView:tableView] + 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if (section < [super numberOfSectionsInTableView:tableView]) {
        return [super tableView:tableView numberOfRowsInSection:section];
    }
    return [_cells count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (indexPath.section < [super numberOfSectionsInTableView:tableView]) {
        return [super tableView:tableView cellForRowAtIndexPath:indexPath];
    }

    return _cells[indexPath.row];
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    if (section == [super numberOfSectionsInTableView:tableView]) {
        return @"Motor";
    }
    return [super tableView:tableView titleForHeaderInSection:section];
}



#pragma mark - Create Cells
- (UITableViewCell *)createAdjustPowerCell
{
    UITableViewCell *cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:nil];
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.textLabel.text = @"Power";

    _powerStepper = [[UIStepper alloc] init];
    _powerStepper.value = 0;
    [_powerStepper addTarget:self action:@selector(updateMotorPower) forControlEvents:UIControlEventValueChanged];

    cell.accessoryView = _powerStepper;

    return cell;
}

- (UITableViewCell *)createAdjustDirectionCell
{
    UITableViewCell *cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:nil];
    cell.textLabel.text = @"Direction";
    cell.selectionStyle = UITableViewCellSelectionStyleNone;

    _directionSegmentControl = [[UISegmentedControl alloc] initWithItems:@[ @"Left", @"Right" ]];
    [_directionSegmentControl setSelectedSegmentIndex:1];
    [_directionSegmentControl addTarget:self action:@selector(updateMotorPower) forControlEvents:UIControlEventValueChanged];

    cell.accessoryView = _directionSegmentControl;

    return cell;
}

- (UITableViewCell *)createBrakeCell
{
    return [Layout createButtonCellWithTitle:@"Brake" target:self action:@selector(didTouchBrake:)];
}


- (UITableViewCell *)createDriftCell
{
    return [Layout createButtonCellWithTitle:@"Drift" target:self action:@selector(didTouchDrift:)];
}


@end