//
//  MotorViewController.m
//  LEGODeviceDemo
//
//  Created by Bartlomiej Hyzy on 24/02/2015.
//  Copyright (c) 2015 lego. All rights reserved.
//

#import "MotorViewController.h"
#import "LEMotor+UI.h"


@interface MotorViewController ()

@property (strong, nonatomic, readonly) LEMotor *motor;

@property (weak, nonatomic) IBOutlet NSSlider *powerSlider;
@property (weak, nonatomic) IBOutlet NSSegmentedControl *directionSegmentedControl;
@property (weak, nonatomic) IBOutlet NSTextField *statusLabel;

@end


@implementation MotorViewController

#pragma mark - UI Appearance Methods

- (void)setRepresentedObject:(id)representedObject
{
    [super setRepresentedObject:representedObject];
    
    [self updateStatusLabel];
    [self updateDirectionControl];
    [self updatePowerSlider];
}

#pragma mark - UI Actions

- (IBAction)steerMotor:(id)sender
{
    [self.motor runInDirection:[self directionFromSegmentControl] power:(NSUInteger)self.powerSlider.integerValue];
    [self updateStatusLabel];
}

- (IBAction)brake:(id)sender
{
    [self.motor brake];
    [self updateStatusLabel];
}

- (IBAction)drift:(id)sender
{
    [self.motor drift];
    [self updateStatusLabel];
}

#pragma mark - Private UI Methods

- (LEMotor *)motor
{
    return (LEMotor *)self.representedObject;
}

- (LEMotorDirection)directionFromSegmentControl
{
    return self.directionSegmentedControl.selectedSegment == 0 ? LEMotorDirectionLeft : LEMotorDirectionRight;
}

- (void)updateStatusLabel
{
    self.statusLabel.stringValue = self.motor.statusDescription ?: @"?";
}

- (void)updateDirectionControl
{
    self.directionSegmentedControl.selectedSegment = (self.motor.direction == LEMotorDirectionLeft ? 0 : 1);
}

- (void)updatePowerSlider
{
    self.powerSlider.integerValue = (NSInteger)self.motor.power;
}

@end
