//
//  PiezoPlayerKey.m
//  LEGODeviceDemo
//
//  Created by Bartlomiej Hyzy on 25/02/2015.
//  Copyright (c) 2015 lego. All rights reserved.
//

#import "PiezoPlayerKey.h"


@interface PiezoPlayerKey ()

@property (assign, nonatomic, readwrite, getter=isPressed) BOOL pressed;
@property (strong, nonatomic) NSTextField *titleLabel;

@end


@implementation PiezoPlayerKey

#pragma mark - Initialization Methods

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    if (self != nil) {
        [self initialize];
    }
    return self;
}

- (void)awakeFromNib
{
    [super awakeFromNib];
    [self initialize];
}

- (void)initialize
{
    self.titleLabel = [[NSTextField alloc] initWithFrame:CGRectZero];
    self.titleLabel.stringValue = self.title ?: @"";
    self.titleLabel.bezeled = NO;
    self.titleLabel.selectable = NO;
    self.titleLabel.editable = NO;
    self.titleLabel.drawsBackground = NO;
    self.titleLabel.translatesAutoresizingMaskIntoConstraints = NO;
    
    [self addSubview:self.titleLabel];
    
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.titleLabel attribute:NSLayoutAttributeCenterX relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeCenterX multiplier:1 constant:0]];
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.titleLabel attribute:NSLayoutAttributeCenterY relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeCenterY multiplier:1 constant:0]];
}

#pragma mark - Public Properties

- (void)setTitle:(NSString *)title
{
    _title = [title copy];
    self.titleLabel.stringValue = _title;
}

- (LEPiezoTonePlayerNote)note
{
    if ([self.title isEqualToString:@"C"]) {
        return LEPiezoTonePlayerNoteC;
    } else if ([self.title isEqualToString:@"D"]) {
        return LEPiezoTonePlayerNoteD;
    } else if ([self.title isEqualToString:@"E"]) {
        return LEPiezoTonePlayerNoteE;
    } else if ([self.title isEqualToString:@"F"]) {
        return LEPiezoTonePlayerNoteF;
    } else if ([self.title isEqualToString:@"G"]) {
        return LEPiezoTonePlayerNoteG;
    } else if ([self.title isEqualToString:@"A"]) {
        return LEPiezoTonePlayerNoteA;
    } else {
        NSAssert(NO, @"Unknown note set for the piezo key");
        return LEPiezoTonePlayerNoteA;
    }
}

#pragma mark - Event Handling

- (void)mouseDown:(NSEvent *)theEvent
{
    self.pressed = YES;
    [self setNeedsDisplay];
    [NSApp sendAction:self.action to:self.target from:self];
}

- (void)mouseUp:(NSEvent *)theEvent
{
    self.pressed = NO;
    [self setNeedsDisplay];
    [NSApp sendAction:self.action to:self.target from:self];
}

#pragma mark - Drawing

- (void)drawRect:(NSRect)rect
{
    // Prepare gradient
    NSColor* gradientColor = [NSColor colorWithCalibratedRed: 0.712 green: 0.712 blue: 0.712 alpha: 1];
    NSColor* gradientColor2 = [NSColor colorWithCalibratedRed: 0.955 green: 0.955 blue: 0.955 alpha: 1];
    NSGradient* gradient = [NSGradient.alloc initWithColorsAndLocations:
                            gradientColor2, 0.0,
                            [gradientColor2 blendedColorWithFraction: 0.5 ofColor: gradientColor], 0.71,
                            gradientColor, 1.0, nil];
    
    // Draw border
    NSBezierPath* rectanglePath = [NSBezierPath bezierPathWithRoundedRect: rect xRadius: 3 yRadius: 3];
    [NSColor.darkGrayColor setFill];
    [rectanglePath fill];

    // Draw inner part - gradient if not pressed and solid fill if pressed
    NSBezierPath* rectangle2Path = [NSBezierPath bezierPathWithRoundedRect: NSInsetRect(rect, 1, 1) xRadius: 3 yRadius: 3];
    if (self.isPressed) {
        [NSColor.lightGrayColor setFill];
        [rectangle2Path fill];
    } else {
        [gradient drawInBezierPath: rectangle2Path angle: -90];
    }
}

@end
