//
// Created by Søren Toft Odgaard on 19/05/14.
// Copyright (c) 2014 LEGO. All rights reserved.
//

#import "PiezoToneDetailDataSource.h"
#import "LEPiezoTonePlayer.h"
#import "Layout.h"
#import "LabeledTextFieldCell.h"
#import "PiezoTonePiano.h"

@interface PiezoToneDetailDataSource ()
@property (nonatomic, weak) LEPiezoTonePlayer *piezoPlayer;
@property (nonatomic, strong) NSArray *cells;
@end


@implementation PiezoToneDetailDataSource {
    LabeledTextFieldCell *_durationCell;
    LabeledTextFieldCell *_frequencyCell;
    NSTimer *_sequencePlayerTimer;
    NSInteger _frequencyJump;
    int _currentFrequency;
}

- (instancetype)initWithService:(LEService *)service tableView:(UITableView *)tableView
{
    self = [super initWithService:service tableView:tableView];
    if (self) {
        self.piezoPlayer = (LEPiezoTonePlayer *) self.service;
        self.cells = @[
                [self createEnterFrequencyCell],
                [self createEnterDurationCell],
                [self createStartPlayingCell],
                [self createStartPlayingSequenceCell],
                [self createStopPlayingCell],
        ];
    }

    self.tableView.tableFooterView = [[PiezoTonePiano alloc] initWithFrame:CGRectMake(0, 0, 320, 100) piezoPlayer:self.piezoPlayer];
    return self;
}

- (id)createEnterFrequencyCell
{
    _frequencyCell = [self.tableView dequeueReusableCellWithIdentifier:@"LabeledTextFieldCellID"];
    _frequencyCell.titleLabel.text = @"Frequency";
    _frequencyCell.textField.text = @"10";
    return _frequencyCell;
}

- (UITableViewCell *)createEnterDurationCell
{
    _durationCell = [self.tableView dequeueReusableCellWithIdentifier:@"LabeledTextFieldCellID"];
    _durationCell.titleLabel.text = @"Duration/Interval (ms)";
    _durationCell.textField.text = @"100";
    return _durationCell;
}


- (UITableViewCell *)createStartPlayingCell
{
    return [Layout createButtonCellWithTitle:@"Start Playing" target:self action:@selector(startPlayingTouched)];
}

- (UITableViewCell *)createStartPlayingSequenceCell
{
    return [Layout createButtonCellWithTitle:@"Start Playing Sequence" target:self action:@selector(startPlayingSequenceTouched)];
}


- (UITableViewCell *)createStopPlayingCell
{
    return [Layout createButtonCellWithTitle:@"Stop Playing" target:self action:@selector(stopPlayingTouched)];
}

- (void)startPlayingTouched
{
    [self.piezoPlayer playFrequency:_frequencyCell.textField.text.integerValue forMilliseconds:_durationCell.textField.text.integerValue];
}

- (void)startPlayingSequenceTouched
{
    [_sequencePlayerTimer invalidate];

    NSInteger duration = _durationCell.textField.text.integerValue;
    if (duration <= 0) {
        NSLog(@"Cannot play sequence with duration 0");
        return;
    }

    _frequencyJump = _frequencyCell.textField.text.integerValue;
    _currentFrequency = 10;
    _sequencePlayerTimer = [NSTimer scheduledTimerWithTimeInterval:(duration / 1000.0) target:self selector:@selector(playSequence) userInfo:nil repeats:YES];
}

- (void)playSequence
{
    [self.piezoPlayer playFrequency:_currentFrequency forMilliseconds:0];
    _currentFrequency += _frequencyJump;
    if (_currentFrequency > LEPiezoToneMaxFrequency) {
        _currentFrequency = 10;
    }
}


- (void)stopPlayingTouched
{
    [_sequencePlayerTimer invalidate];
    _sequencePlayerTimer = nil;
    [self.piezoPlayer stopPlaying];
}


#pragma mark - UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return [super numberOfSectionsInTableView:tableView] + 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if (section < [super numberOfSectionsInTableView:tableView]) {
        return [super tableView:tableView numberOfRowsInSection:section];
    }
    return [_cells count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (indexPath.section < [super numberOfSectionsInTableView:tableView]) {
        return [super tableView:tableView cellForRowAtIndexPath:indexPath];
    }

    return _cells[indexPath.row];
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    if (section == [super numberOfSectionsInTableView:tableView]) {
        return @"Piezo Tone";
    }
    return [super tableView:tableView titleForHeaderInSection:section];
}

- (NSString *)tableView:(UITableView *)tableView titleForFooterInSection:(NSInteger)section
{
    if (section == [super numberOfSectionsInTableView:tableView]) {
        return @"A sequence will start at 10Hz and increment by 'frequenzy' on each 'interval'";
    }
    return [super tableView:tableView titleForHeaderInSection:section];
}


@end