//
//  PiezoViewController.m
//  LEGODeviceDemo
//
//  Created by Bartlomiej Hyzy on 24/02/2015.
//  Copyright (c) 2015 lego. All rights reserved.
//

#import "PiezoViewController.h"
#import <LEGODevice/LEGODevice.h>
#import "PiezoPlayerKey.h"


@interface PiezoViewController ()

@property (assign, nonatomic) NSUInteger frequency; // bound to UI
@property (assign, nonatomic) NSUInteger time;      // bound to UI

@property (strong, nonatomic) NSTimer *sequencePlayerTimer;
@property (assign, nonatomic) NSUInteger frequencyJump;
@property (assign, nonatomic) NSUInteger currentFrequency;

@end


@implementation PiezoViewController

#pragma mark - UI Appearance Methods

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // Set defaults
    self.frequency = 10;
    self.time = 100;
}

#pragma mark - UI Actions

- (IBAction)startPlaying:(id)sender
{
    [self.piezoTonePlayer playFrequency:self.frequency forMilliseconds:self.time];
}

- (IBAction)startSequence:(id)sender
{
    self.frequencyJump = self.frequency;
    self.currentFrequency = 10;
    self.sequencePlayerTimer = [NSTimer scheduledTimerWithTimeInterval:(self.time / 1000.0) target:self selector:@selector(playSequence) userInfo:nil repeats:YES];
}

- (IBAction)stopPlaying:(id)sender
{
    [self.sequencePlayerTimer invalidate];
    self.sequencePlayerTimer = nil;
    
    [self.piezoTonePlayer stopPlaying];
}

- (void)playSequence
{
    [self.piezoTonePlayer playFrequency:self.currentFrequency forMilliseconds:0];
    self.currentFrequency += self.frequencyJump;
    if (self.currentFrequency > LEPiezoToneMaxFrequency) {
        self.currentFrequency = 10;
    }
}

- (IBAction)noteButtonStateChanged:(PiezoPlayerKey *)keyboardKey
{
    if (keyboardKey.isPressed) {
        [self.piezoTonePlayer playNote:keyboardKey.note octave:4 forMilliSeconds:0];
    } else {
        [self.piezoTonePlayer stopPlaying];
    }
}

#pragma mark - Private Helpers

- (LEPiezoTonePlayer *)piezoTonePlayer
{
    return (LEPiezoTonePlayer *)self.representedObject;
}

@end
