//
// Created by Søren Toft Odgaard on 21/05/14.
// Copyright (c) 2014 LEGO. All rights reserved.
//

#import "RGBLightDetailDataSource.h"
#import "LERGBLight.h"
#import "Layout.h"
#import "LabeledTextFieldCell.h"


@implementation RGBLightDetailDataSource {
    LERGBLight *_rgbLight;
    NSArray *_absoluteModeCells;
    NSArray *_discreteModeCells;
    
    LabeledTextFieldCell *_redCell;
    LabeledTextFieldCell *_greenCell;
    LabeledTextFieldCell *_blueCell;
    
    UITableViewCell *_enterColorIndexCell;
    UIStepper *_colorIndexStepper;
}

- (instancetype)initWithService:(LEService *)service tableView:(UITableView *)tableView
{
    self = [super initWithService:service tableView:tableView];
    if (self) {
        _rgbLight = (LERGBLight *) self.service;
        _absoluteModeCells = @[
                               [self createEnterRedCell],
                               [self createEnterGreenCell],
                               [self createEnterBlueCell],
                               [self createSendColorButtonCell],
                               [self createSendLightOffButtonCell],
                               [self createSendDefaultColorButtonCell]
                               ];
        _discreteModeCells = @[
                               [self createEnterColorIndexCell],
                               [self createSendLightOffButtonCell],
                               [self createSendDefaultColorButtonCell]
                               ];
    }
    
    [self updateColorTextFieldsFromRGBLightColor];
    
    return self;
}

- (void)updateColorTextFieldsFromRGBLightColor {
    _redCell.textField.text = [NSString stringWithFormat:@"%i", (uint8_t) round(_rgbLight.color.red * 255.0)];
    _greenCell.textField.text = [NSString stringWithFormat:@"%i", (uint8_t) round(_rgbLight.color.green * 255.0)];
    _blueCell.textField.text = [NSString stringWithFormat:@"%i", (uint8_t) round(_rgbLight.color.blue * 255.0)];
    [self updateColorIndexCellLabel];
}

- (UITableViewCell *)createEnterRedCell
{
    _redCell = [self.tableView dequeueReusableCellWithIdentifier:@"LabeledTextFieldCellID"];
    _redCell.titleLabel.text = @"Red:";
    return _redCell;
}

- (UITableViewCell *)createEnterGreenCell
{
    _greenCell = [self.tableView dequeueReusableCellWithIdentifier:@"LabeledTextFieldCellID"];
    _greenCell.titleLabel.text = @"Green:";
    return _greenCell;
}

- (UITableViewCell *)createEnterBlueCell
{
    _blueCell = [self.tableView dequeueReusableCellWithIdentifier:@"LabeledTextFieldCellID"];
    _blueCell.titleLabel.text = @"Blue:";
    return _blueCell;
}


- (UITableViewCell *)createEnterColorIndexCell
{
    _enterColorIndexCell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:nil];
    _enterColorIndexCell.selectionStyle = UITableViewCellSelectionStyleNone;
    [self updateColorIndexCellLabel];

    _colorIndexStepper = [[UIStepper alloc] init];
    _colorIndexStepper.value = 0;
    _colorIndexStepper.minimumValue = 0;
    [_colorIndexStepper addTarget:self action:@selector(updateColorIndex) forControlEvents:UIControlEventValueChanged];

    _enterColorIndexCell.accessoryView = _colorIndexStepper;

    return _enterColorIndexCell;
}

- (void)updateColorIndexCellLabel
{
    _enterColorIndexCell.textLabel.text = [NSString stringWithFormat:@"Color Index %lu", (unsigned long) _colorIndexStepper.value];
}

- (UITableViewCell *)createSendColorButtonCell
{
    return [Layout createButtonCellWithTitle:@"Write Color" target:self action:@selector(writeColorTouched)];
    
}

- (UITableViewCell *)createSendLightOffButtonCell
{
    return [Layout createButtonCellWithTitle:@"Switch Off" target:self action:@selector(switchOffTouched)];
    
}

- (UITableViewCell *)createSendDefaultColorButtonCell
{
    return [Layout createButtonCellWithTitle:@"Switch to Default Color" target:self action:@selector(switchToDefaultColorTouched)];
}

- (void)writeColorTouched
{
    CGFloat red = (CGFloat) ((_redCell.textField.text.integerValue) / 255.0);
    CGFloat green = (CGFloat) ((_greenCell.textField.text.integerValue) / 255.0);
    CGFloat blue = (CGFloat) ((_blueCell.textField.text.integerValue) / 255.0);
    _rgbLight.color = [CIColor colorWithRed:red green:green blue:blue];
}


- (void)updateColorIndex
{
    _rgbLight.colorIndex = _colorIndexStepper.value;
    [self updateColorIndexCellLabel];
}


- (void)switchOffTouched
{
    [_rgbLight switchOff];
}

- (void)switchToDefaultColorTouched
{
    [_rgbLight switchToDefaultColor];
}

#pragma mark - UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return [super numberOfSectionsInTableView:tableView] + 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if (section < [super numberOfSectionsInTableView:tableView]) {
        return [super tableView:tableView numberOfRowsInSection:section];
    }
    if (_rgbLight.rgbMode == _rgbLight.absoluteModeIndex) {
        return [_absoluteModeCells count];
    } else {
        return [_discreteModeCells count];
    }

}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (indexPath.section < [super numberOfSectionsInTableView:tableView]) {
        return [super tableView:tableView cellForRowAtIndexPath:indexPath];
    }
    if (_rgbLight.rgbMode == _rgbLight.absoluteModeIndex) {
        return _absoluteModeCells[indexPath.row];
    } else {
        return _discreteModeCells[indexPath.row];
    }
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    if (section == [super numberOfSectionsInTableView:tableView]) {
        return @"RGB Light";
    }
    return [super tableView:tableView titleForHeaderInSection:section];
}


@end