//
//  ServiceControlViewController.m
//  LEGODeviceDemo
//
//  Created by Bartlomiej Hyzy on 23/02/2015.
//  Copyright (c) 2015 lego. All rights reserved.
//

#import "ServiceControlViewController.h"


@interface ServiceControlViewController ()

@property (strong, nonatomic) NSViewController *contentViewController;

@end


@implementation ServiceControlViewController

- (void)setControlledService:(LEService *)controlledService
{
    _controlledService = controlledService;
    
    NSString *segueIdentifier = [self segueIdentifierForService:controlledService];
    if (segueIdentifier != nil) {
        [self performSegueWithIdentifier:segueIdentifier sender:self];
    } else {
        self.contentViewController = nil;
    }
}

- (NSString *)segueIdentifierForService:(LEService *)service
{
    if (service == nil) {
        return @"controlEmpty";
    }
    
    if ([service isKindOfClass:[LERGBLight class]]) {
        return @"controlRGBLight";
    } else if ([service isKindOfClass:[LEMotor class]]) {
        return @"controlMotor";
    } else if ([service isKindOfClass:[LEPiezoTonePlayer class]]) {
        return @"controlPiezo";
    } else {
        return nil;
    }
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.controlledService = nil;
}

- (void)prepareForSegue:(NSStoryboardSegue *)segue sender:(id)sender
{
    self.contentViewController = (NSViewController *)segue.destinationController;
    self.contentViewController.representedObject = self.controlledService;
}

- (void)setContentViewController:(NSViewController *)contentViewController
{
    NSViewController *oldContentVC = _contentViewController;
    NSViewController *newContentVC = contentViewController;
    
    _contentViewController = newContentVC;
    
    if (newContentVC == nil) {
        [oldContentVC removeFromParentViewController];
        [oldContentVC.view removeFromSuperview];
        return;
    }
    
    [self addChildViewController:newContentVC];
    
    if (oldContentVC != nil) {
        [self transitionFromViewController:oldContentVC toViewController:newContentVC options:NSViewControllerTransitionNone completionHandler:^{
            [oldContentVC removeFromParentViewController];
        }];
    } else {
        [self.view addSubview:newContentVC.view];
    }
    
    newContentVC.view.translatesAutoresizingMaskIntoConstraints = NO;
    NSDictionary *views = @{@"content": newContentVC.view};
    [self.view addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"H:|[content]|" options:0 metrics:0 views:views]];
    [self.view addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|[content]|" options:0 metrics:0 views:views]];
}

@end
