//
//  ServiceDetailTableViewController.m
//  LEGODeviceDemo
//
//  Created by Søren Toft Odgaard on 29/04/14.
//  Copyright (c) 2014 LEGO. All rights reserved.
//

#import "ServiceDetailTableViewController.h"
#import "LEService.h"
#import "ServiceInputFormatViewController.h"
#import "GenericServiceDetailDataSource.h"
#import "LETiltSensor.h"
#import "TiltSensorDetailDataSource.h"
#import "LEDevice.h"
#import "LEVoltageSensor.h"
#import "VoltageSensorDetailDataSource.h"
#import "LEMotionSensor.h"
#import "MotionSensorDetailDataSource.h"
#import "CurrentSensorDetailDataSource.h"
#import "LECurrentSensor.h"
#import "LEMotor.h"
#import "MotorDetailDataSource.h"
#import "LEPiezoTonePlayer.h"
#import "PiezoToneDetailDataSource.h"
#import "LERGBLight.h"
#import "RGBLightDetailDataSource.h"

@interface ServiceDetailTableViewController () <LEDeviceDelegate>
@end

@implementation ServiceDetailTableViewController {
    GenericServiceDetailDataSource *_myDataSource;
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    if ([_service isKindOfClass:[LETiltSensor class]]) {
        _myDataSource = [TiltSensorDetailDataSource dataSourceWithService:_service tableView:self.tableView];
    } else if ([_service isKindOfClass:[LEMotionSensor class]]) {
        _myDataSource = [MotionSensorDetailDataSource dataSourceWithService:_service tableView:self.tableView];
    } else if ([_service isKindOfClass:[LEVoltageSensor class]]) {
        _myDataSource = [VoltageSensorDetailDataSource dataSourceWithService:_service tableView:self.tableView];
    } else if ([_service isKindOfClass:[LECurrentSensor class]]) {
        _myDataSource = [CurrentSensorDetailDataSource dataSourceWithService:_service tableView:self.tableView];
    } else if ([_service isKindOfClass:[LEMotor class]]) {
        _myDataSource = [MotorDetailDataSource dataSourceWithService:_service tableView:self.tableView];
    } else if ([_service isKindOfClass:[LEPiezoTonePlayer class]]) {
        _myDataSource = [PiezoToneDetailDataSource dataSourceWithService:_service tableView:self.tableView];
    } else if ([_service isKindOfClass:[LERGBLight class]]) {
        _myDataSource = [RGBLightDetailDataSource dataSourceWithService:_service tableView:self.tableView];
    }
    else {
        _myDataSource = [GenericServiceDetailDataSource dataSourceWithService:_service tableView:self.tableView];
    }

    self.tableView.dataSource = _myDataSource;
    self.tableView.delegate = _myDataSource;

    __weak __typeof__(self) weakSelf = self;
    _myDataSource.showInputFormatViewController = ^{
        [weakSelf performSegueWithIdentifier:@"InputFormatSegue" sender:weakSelf];
    };

    [self.device addDelegate:self];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self.tableView reloadData];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([segue.identifier isEqualToString:@"InputFormatSegue"]) {
        ServiceInputFormatViewController *vc = segue.destinationViewController;
        vc.service = self.service;
    }
}


#pragma mark - LEDeviceDelegate

- (void)device:(LEDevice *)device didRemoveService:(LEService *)service
{
    //We cannot just use 'popViewController' as this will not work if 'this' VC is not
    //not visible (for instance if we are showing the Connect Info child view controller).
    NSUInteger index = [self.navigationController.viewControllers indexOfObject:self];
    if (index > 0) {
        UIViewController *previousOnStack = self.navigationController.viewControllers[index - 1];
        [self.navigationController popToViewController:previousOnStack animated:YES];
    }
}


@end
