//
//  ServiceInputFormatViewController.m
//  LEGODeviceDemo
//
//  Created by Søren Toft Odgaard on 29/04/14.
//  Copyright (c) 2014 LEGO. All rights reserved.
//

#import "ServiceInputFormatViewController.h"
#import "LEInputFormat.h"


@implementation ServiceInputFormatViewController {

    IBOutlet UILabel *_formatRevisionLabel;
    IBOutlet UILabel *_connectID;
    IBOutlet UILabel *_typeIdLabel;
    IBOutlet UITextField *_modeTextField;
    IBOutlet UITextField *_deltaIntervalField;    
    IBOutlet UITextField *_unitField;
    IBOutlet UISwitch *_notificationsEnabledSwitch;
    IBOutlet UILabel *_numberOfBytesLabel;

}

- (void)setService:(LEService *)service
{
    _service = service;
    [_service addDelegate:self];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    self.title = [NSString stringWithFormat:@"Type: %@", self.service.connectInfo.typeString];
    [self updateAllFieldsAnimated:NO oldFormat:nil newFormat:_service.inputFormat];
    [self.tableView reloadData];
}


- (void)updateAllFieldsAnimated:(BOOL)animated oldFormat:(LEInputFormat *)oldFormat newFormat:(LEInputFormat *)newFormat
{

    [UIView animateWithDuration:(animated ? 0.2 : 0.0) animations:^{
        CGFloat alpha = 0.2;
        if (newFormat.revision != oldFormat.revision) _formatRevisionLabel.alpha = alpha;
        if (newFormat.mode != oldFormat.mode) _modeTextField.alpha = alpha;
        if (newFormat.deltaInterval != oldFormat.deltaInterval) _deltaIntervalField.alpha = alpha;
        if (newFormat.unit != oldFormat.unit) _unitField.alpha = alpha;
        if (newFormat.numberOfBytes != oldFormat.numberOfBytes) _numberOfBytesLabel.alpha = alpha;
    } completion:^(BOOL finished) {
        _formatRevisionLabel.text = [NSString stringWithFormat:@"%u", self.service.inputFormat.revision];
        _connectID.text = [NSString stringWithFormat:@"%u", self.service.connectInfo.connectID];
        _typeIdLabel.text = [NSString stringWithFormat:@"%u", self.service.connectInfo.type];
        _modeTextField.text = [NSString stringWithFormat:@"%u", self.service.inputFormat.mode];
        _deltaIntervalField.text = [NSString stringWithFormat:@"%u", self.service.inputFormat.deltaInterval];
        _unitField.text = [NSString stringWithFormat:@"%u", self.service.inputFormat.unit];
        [_notificationsEnabledSwitch setOn:self.service.inputFormat.notificationsEnabled animated:animated];
        _numberOfBytesLabel.text = [NSString stringWithFormat:@"%u", self.service.inputFormat.numberOfBytes];
        [UIView animateWithDuration:(animated ? 0.2 : 0.0) animations:^{
            _formatRevisionLabel.alpha = 1.0;
            _modeTextField.alpha = 1.0;
            _deltaIntervalField.alpha = 1.0;
            _unitField.alpha = 1.0;
            _numberOfBytesLabel.alpha = 1.0;
        }];
    }
    ];
}


- (IBAction)sendUpdateButtonTouched:(id)sender
{
    LEInputFormat *inputFormat = [LEInputFormat
            inputFormatWithConnectID:self.service.connectInfo.connectID
            typeID:self.service.inputFormat.typeID
            mode:(uint8_t) _modeTextField.text.integerValue
            deltaInterval:(uint32_t)_deltaIntervalField.text.integerValue
            unit:(LEInputFormatUnit) _unitField.text.integerValue
            notificationsEnabled:_notificationsEnabledSwitch.on];

    [self.service updateInputFormat:inputFormat];
}


#pragma mark - LEServiceDelegate
- (void)service:(LEService *)service didUpdateInputFormatFrom:(LEInputFormat *)oldFormat to:(LEInputFormat *)newFormat
{
    [self updateAllFieldsAnimated:YES oldFormat:oldFormat newFormat:newFormat];
}


@end
