//
//  SettingsViewController.m
//  LEGODeviceDemo
//
//  Created by Søren Toft Odgaard on 04/07/14.
//  Copyright (c) 2014 LEGO. All rights reserved.
//

#import "SettingsViewController.h"
#import "LEDeviceManager.h"

static NSString *const UserDefaultsKeyEnableAutoReconnect = @"EnableAutoReconnect";
static NSString *const UserDefaultsKeyRSSIThreshold = @"RSSIThreshold";

static NSInteger const DefaultRSSIThreshold = -150;

@interface SettingsViewController ()

@property (weak, nonatomic) IBOutlet UISwitch *enableAutoReconnectSwitch;

@property (weak, nonatomic) IBOutlet UIStepper *rssiThresholdStepper;
@property (weak, nonatomic) IBOutlet UILabel *rssiThresholdLabel;


@end


@implementation SettingsViewController

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];

    [self.enableAutoReconnectSwitch setOn:[LEDeviceManager sharedDeviceManager].automaticReconnectOnConnectionLostEnabled];
    self.rssiThresholdLabel.text = [NSString stringWithFormat:@"%li", (long) [SettingsViewController rssiThreshold]];

    self.rssiThresholdStepper.value = [SettingsViewController rssiThreshold];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (IBAction)doneTouched:(id)sender
{
    [LEDeviceManager sharedDeviceManager].automaticReconnectOnConnectionLostEnabled = self.enableAutoReconnectSwitch.on;
    [[NSUserDefaults standardUserDefaults] setBool:self.enableAutoReconnectSwitch.on forKey:UserDefaultsKeyEnableAutoReconnect];
    [[NSUserDefaults standardUserDefaults] synchronize];
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)rssiThresholdStepperChangedValue:(id)sender
{
    [[NSUserDefaults standardUserDefaults] setInteger:(NSInteger) self.rssiThresholdStepper.value forKey:UserDefaultsKeyRSSIThreshold];
    self.rssiThresholdLabel.text = [NSString stringWithFormat:@"%li", (long) [SettingsViewController rssiThreshold]];
}


+ (BOOL)autoReconnectEnabled
{
    return [[NSUserDefaults standardUserDefaults] boolForKey:UserDefaultsKeyEnableAutoReconnect];
}

+ (NSInteger)rssiThreshold
{
    if ([[NSUserDefaults standardUserDefaults] valueForKey:UserDefaultsKeyRSSIThreshold]) {
        return [[NSUserDefaults standardUserDefaults] integerForKey:UserDefaultsKeyRSSIThreshold];
    } else {
        return DefaultRSSIThreshold;
    }
}


@end
