//
// Created by Søren Toft Odgaard on 17/04/14.
// Copyright (c) 2014 LEGO. All rights reserved.
//

#import "SimulatorAddServiceViewController.h"
#import "SimulatorService.h"
#import "SimulatorServiceStore.h"

@interface SimulatorAddServiceViewController () <UITextFieldDelegate>
@end

@implementation SimulatorAddServiceViewController {

    IBOutlet UITextField *_nameTextField;
    IBOutlet UITextField *_connectIdTextField;
    IBOutlet UITextField *_hubIndexTextField;
    IBOutlet UITextField *_typeIdTextField;
    IBOutlet UIBarButtonItem *_saveButton;

    SimulatorServiceStore *_store;
}

- (void)viewDidLoad
{
    _store = [SimulatorServiceStore sharedInstance];
    _saveButton.enabled = NO;
    _nameTextField.delegate = _connectIdTextField.delegate = _typeIdTextField.delegate = _hubIndexTextField.delegate = self;
}

- (IBAction)cancelTouched:(id)sender
{
    [self.navigationController dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)saveTouched:(id)sender
{
    SimulatorService *service = [SimulatorService
            serviceWithName:_nameTextField.text
            connectID:_connectIdTextField.text.integerValue
            hubIndex:_hubIndexTextField.text.integerValue
            typeID:_typeIdTextField.text.integerValue];


    if ([_store.services containsObject:service]) {
        [self showAlertViewWithTitle:@"Duplicate entry" message:@"A service with the specified attributes already exists"];
        return;
    }

    [_store addSimulatorService:service];
    [self.navigationController dismissViewControllerAnimated:YES completion:nil];
}

- (void)showAlertViewWithTitle:(NSString *)title message:(NSString *)message
{
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:title message:message delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alertView show];
}


- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    NSString *newString = [textField.text stringByReplacingCharactersInRange:range withString:string];
    _saveButton.enabled = (newString.length > 0);

    if ((textField != _nameTextField && _nameTextField.text.length == 0) ||
            (textField != _connectIdTextField && _connectIdTextField.text.length == 0) ||
            (textField != _hubIndexTextField && _hubIndexTextField.text.length == 0) ||
            (textField != _typeIdTextField && _typeIdTextField.text.length == 0)) {
        _saveButton.enabled = NO;
    }

    return YES;
}


@end