//
// Created by Søren Toft Odgaard on 17/04/14.
// Copyright (c) 2014 LEGO. All rights reserved.
//

#import "SimulatorServiceStore.h"
#import "SimulatorService.h"


@implementation SimulatorServiceStore {

    NSMutableArray *_services;
}

+ (instancetype)sharedInstance
{
    static dispatch_once_t pred = 0;
    __strong static id _sharedObject = nil;
    dispatch_once(&pred, ^{
        _sharedObject = [[self alloc] init]; // or some other init method
    });
    return _sharedObject;
}

- (id)init
{
    self = [super init];
    if (self) {
        [self loadFromUserDefaults];
    }
    return self;
}

- (NSArray *)services
{
    return _services;
}


- (void)addSimulatorService:(SimulatorService *)service
{
    [_services addObject:service];
    [self saveToUserDefaults];
}

- (void)removeSimulatorService:(SimulatorService *)service
{
    [_services removeObject:service];
    [self saveToUserDefaults];
}

- (void)saveToUserDefaults
{
    NSMutableArray *archive = [NSMutableArray arrayWithCapacity:_services.count];
    for (SimulatorService *service in _services) {
        [archive addObject:[NSKeyedArchiver archivedDataWithRootObject:service]];
    }

    [[NSUserDefaults standardUserDefaults] setValue:archive forKey:@"SimulatorServices"];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (void)loadFromUserDefaults {
    NSArray *archive = [[NSUserDefaults standardUserDefaults] valueForKey:@"SimulatorServices"];
    _services = [NSMutableArray arrayWithCapacity:archive.count];
    for (NSData *serviceData in archive) {
        [_services addObject:[NSKeyedUnarchiver unarchiveObjectWithData:serviceData]];
    }
}


@end