//
// Created by Søren Toft Odgaard on 08/05/14.
// Copyright (c) 2014 LEGO. All rights reserved.
//

#import "TiltSensorDetailDataSource.h"
#import "LEInputFormat+Project.h"


@implementation TiltSensorDetailDataSource {
    LETiltSensor *_tiltSensor;
}

- (instancetype)initWithService:(LEService *)service tableView:(UITableView *)tableView
{
    self = [super initWithService:service tableView:tableView];
    if (self) {
        _tiltSensor = (LETiltSensor *) self.service;
    }

    [self updateInputValue];
    [self updateInputFormat];

    return self;
}


- (void)updateInputValue
{
    [super updateInputValue];
    switch (_tiltSensor.tiltSensorMode) {
        case LETiltSensorModeAngle: {
            if (_tiltSensor.inputFormat.unit == LEInputFormatUnitRaw) {
                self.inputValueLabel.text = [NSString stringWithFormat:@"%ld, %ld", (long) _tiltSensor.angle.x, (long) _tiltSensor.angle.y];
            } else if (_tiltSensor.inputFormat.unit == LEInputFormatUnitSI) {
                self.inputValueLabel.text = [NSString stringWithFormat:@"%.2f, %.2f", _tiltSensor.angle.x, _tiltSensor.angle.y];
            } else if (_tiltSensor.inputFormat.unit == LEInputFormatUnitPercentage) {
                self.inputValueLabel.text = [NSString stringWithFormat:@"%lu%%, %lu%%", (unsigned long) _tiltSensor.angle.x, (unsigned long) _tiltSensor.angle.y];
            }
        }
            break;
        case LETiltSensorModeTilt:
            self.inputValueLabel.text = [self stringFromDirection:_tiltSensor.direction];
            break;
        case LETiltSensorModeCrash:
            self.inputValueLabel.text = [NSString stringWithFormat:@"%ld, %ld, %ld", (long) _tiltSensor.crash.x, (long) _tiltSensor.crash.y, (long) _tiltSensor.crash.z];
            break;
        case LETiltSensorModeUnknown:
            break;
    }
}


- (void)updateInputFormat
{
    [super updateInputFormat];
    switch (_tiltSensor.tiltSensorMode) {
        case LETiltSensorModeAngle:
            self.inputModeLabel.text = @"Angle";
            break;
        case LETiltSensorModeTilt:
            self.inputModeLabel.text = @"Tilt";
            break;
        case LETiltSensorModeCrash:
            self.inputModeLabel.text = @"Crash";
            break;
        case LETiltSensorModeUnknown:
            self.inputModeLabel.text = @"Unknown";
            break;
    }
}

- (NSString *)stringFromDirection:(LETiltSensorDirection)direction
{
    switch (direction) {
        case LETiltSensorDirectionBackward:
            return @"Backward";
        case LETiltSensorDirectionRight:
            return @"Right";
        case LETiltSensorDirectionNeutral:
            return @"Neutral";
        case LETiltSensorDirectionForward:
            return @"Forward";
        case LETiltSensorDirectionLeft:
            return @"Left";
        case LETiltSensorDirectionUnknown:
            return @"Unkown";
    }
    return @"Unkown";
}

@end