//
//  WindmillTestViewController.m
//  LEGODeviceDemo
//
//  Created by Søren Toft Odgaard on 04/07/14.
//  Copyright (c) 2014 LEGO. All rights reserved.
//

#import "WindmillTestViewController.h"
#import "LEDevice.h"
#import "LEMotor.h"
#import "LERGBLight.h"
#import "LEMotionSensor.h"

@interface WindmillTestViewController () <LEMotionSensorDelegate>

@property (weak, nonatomic) IBOutlet UILabel *speedLabel;
@property (weak, nonatomic) IBOutlet UIStepper *speedStepper;

@property (weak, nonatomic) IBOutlet UILabel *currentValueLabel;
@property (weak, nonatomic) IBOutlet UILabel *triggerHighValueLabel;
@property (weak, nonatomic) IBOutlet UILabel *triggerLowValueLabel;
@property (weak, nonatomic) IBOutlet UIStepper *triggerHighValueStepper;
@property (weak, nonatomic) IBOutlet UIStepper *triggerLowValueStepper;
@property (weak, nonatomic) IBOutlet UISwitch *shiftDirectionWhenAboveSwitch;
@property (weak, nonatomic) IBOutlet UILabel *shiftCountLabel;

@property (weak, nonatomic) LEMotor *motor;
@property (weak, nonatomic) LEMotionSensor *motionSensor;
@property (weak, nonatomic) LERGBLight *rgbLight;

@property (readwrite) NSUInteger shiftDirectionCount;

@property (readwrite) BOOL didShiftToHigh;

@end

@implementation WindmillTestViewController


#pragma mark - Motor

- (void)viewDidLoad
{
    [super viewDidLoad];

    for (LEService *service in self.device.services) {
        if ([service isKindOfClass:[LEMotor class]]) {
            self.motor = (LEMotor *) service;
        }
        if ([service isKindOfClass:[LEMotionSensor class]]) {
            self.motionSensor = (LEMotionSensor *) service;
            [self.motionSensor addDelegate:self];
        }
        if ([service isKindOfClass:[LERGBLight class]]) {
            self.rgbLight = (LERGBLight *) service;
        }
    }

    self.triggerHighValueStepper.value = 8;
    self.triggerLowValueStepper.value = 3;
    [self updateTriggerHighValueLabel];
    [self updateTriggerLowValueLabel];
    [self updateCurrentValueLabel];
}


- (IBAction)didTouchStopMotor:(id)sender
{
    [self.motor brake];
    [self updateSpeedLabel];
}

- (IBAction)didTouchFullSpeed:(id)sender
{
    LEMotorDirection direction = (self.motor.direction == LEMotorDirectionRight) ? LEMotorDirectionRight : LEMotorDirectionLeft;
    [self.motor runInDirection:direction power:LEMotorMaxSpeed];
    [self updateSpeedLabel];
}

- (void)updateSpeedLabel
{
    self.speedLabel.text = [NSString stringWithFormat:@"Power %lu", (unsigned long) self.motor.power];
    self.speedStepper.value = self.motor.power;
}

- (IBAction)didTouchSpeedStepper:(id)sender
{
    LEMotorDirection direction = (self.motor.direction == LEMotorDirectionRight) ? LEMotorDirectionRight : LEMotorDirectionLeft;
    [self.motor runInDirection:direction power:(NSUInteger) self.speedStepper.value];
    [self updateSpeedLabel];
}

- (void)shiftDirection
{
    LEMotorDirection newDirection = LEMotorDirectionBraking;
    NSUInteger colorIndex;
    if (self.motor.direction == LEMotorDirectionLeft) {
        newDirection = LEMotorDirectionRight;
        colorIndex = 2;
    } else if (self.motor.direction == LEMotorDirectionRight) {
        newDirection = LEMotorDirectionLeft;
        colorIndex = 6;
    }

    if (newDirection != LEMotorDirectionBraking) {
        self.rgbLight.colorIndex = colorIndex;

        //..and change direction
        [self.motor runInDirection:newDirection power:self.motor.power];
        self.shiftDirectionCount++;
        self.shiftCountLabel.text = [NSString stringWithFormat:@"%lu", (unsigned long) self.shiftDirectionCount];
    }
}

#pragma mark - Motion Sensor

- (IBAction)didTouchTriggerHighValueStepper:(id)sender
{
    [self updateTriggerHighValueLabel];
}

- (IBAction)didTouchTriggerLowValueStepper:(id)sender
{
    [self updateTriggerLowValueLabel];
}


- (IBAction)didTouchShiftDirectionWhenAboveSwitch:(id)sender
{
}

- (IBAction)didTouchResetShiftCountButton:(id)sender
{
    self.shiftDirectionCount = 0;
    self.shiftCountLabel.text = [NSString stringWithFormat:@"%lu", (unsigned long) self.shiftDirectionCount];

}

- (void)updateTriggerHighValueLabel
{
    self.triggerHighValueLabel.text = [NSString stringWithFormat:@"Trigger high: %lu", (unsigned long) self.triggerHighValueStepper.value];
}

- (void)updateTriggerLowValueLabel
{
    self.triggerLowValueLabel.text = [NSString stringWithFormat:@"Trigger low: %lu", (unsigned long) self.triggerLowValueStepper.value];
}


- (void)updateCurrentValueLabel
{
    self.currentValueLabel.text = [NSString stringWithFormat:@"%lu", (unsigned long) self.motionSensor.distance];
}

- (NSInteger)triggerHighValue
{
    return (NSInteger) self.triggerHighValueStepper.value;
}

- (NSInteger)triggerLowValue
{
    return (NSInteger) self.triggerLowValueStepper.value;
}


#pragma mark - LEMotionSensorDelegate

- (void)motionSensor:(LEMotionSensor *)sensor didUpdateDistanceFrom:(CGFloat)oldDistance to:(CGFloat)newDistance
{
    NSInteger newValue = (NSInteger) newDistance;
    self.currentValueLabel.text = [NSString stringWithFormat:@"%lu", (unsigned long) newValue];
    if (self.shiftDirectionWhenAboveSwitch.on) {
        if (newValue >= self.triggerHighValue && !self.didShiftToHigh) {
            [self shiftDirection];
            self.didShiftToHigh = YES;
        } else if (newValue <= self.triggerLowValue && self.didShiftToHigh) {
            self.didShiftToHigh = NO;
        }
    }

    [self updateCurrentValueLabel];
}


@end
