﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using LegoDeviceSDK.Interfaces;
using LegoDeviceSDK.Wrapper.Helpers;
using LEGODeviceUnitySDK;
using System.Diagnostics;

namespace LegoDeviceSDK.Wrapper
{
    public class DeviceWrapper : IDeviceWrapperImpl
    {
        private readonly IDeviceManagerWrapperImpl deviceManagerWrapper;
        private readonly WrapperSerialization wrapperSerialization;

        public IServiceWrapperImpl ServiceWrapper { get; set; }
        public ILEGODeviceNativeDelegate UnityDeviceDelegate { get; set; }

        public DeviceWrapper(IDeviceManagerWrapperImpl managerWrapper, WrapperSerialization serialization)
        {
            deviceManagerWrapper = managerWrapper;
            wrapperSerialization = serialization;
        }

        internal DeviceWrapper(DeviceManagerWrapper managerWrapper)
            : this(managerWrapper, new WrapperSerialization())
        { }

        public void ChangeDeviceName(string deviceID, string deviceName)
        {
            IDevice device = deviceManagerWrapper.FindDevice(deviceID);
            if (device != null)
                device.Name = deviceName;
        }

        #region IDeviceDelegate

        private void InvokeUnityMethod(Action<string> method, Dictionary<string, object> dataDictionary)
        { 
            UnityInvoker.InvokeUnityMethod(method, wrapperSerialization.ConvertToJSONString(dataDictionary));
        }

        public void DidUpdateDeviceInfo(IDevice device, Generic.DeviceInfo deviceInfo, Exception error)
        {
            InvokeUnityMethod(UnityDeviceDelegate.LEDeviceDidUpdateDeviceInfo, wrapperSerialization.SerializeDevice(device));
        }

        public void DidChangeButtonState(IDevice device, bool buttonState)
        {
            InvokeUnityMethod(UnityDeviceDelegate.LEDeviceDidChangeButtonState, wrapperSerialization.SerializeDeviceButtonStateChange(device, buttonState));
        }

        public void DidChangeLowVoltageState(IDevice device, bool lowVoltage) {
            InvokeUnityMethod(UnityDeviceDelegate.LEDeviceDidUpdateLowVoltageState, wrapperSerialization.SerializeDeviceLowVoltageState(device, lowVoltage));
        }

        public void DidChangeName(IDevice device, string fromName, string toName)
        {
            InvokeUnityMethod(UnityDeviceDelegate.LEDeviceDidChangeName, wrapperSerialization.SerializeDeviceNameChange(device, fromName, toName));
        }

        public void DidUpdateBatteryLevel(IDevice device, uint newLevel)
        {
            InvokeUnityMethod(UnityDeviceDelegate.LEDeviceDidUpdateBatteryLevel, wrapperSerialization.SerializeDeviceBatteryLevel(device, (int)newLevel));
        }

        public void DidAddService(IDevice device, IService service)
        {
            Debug.Assert(ServiceWrapper != null, "Service wrapper should have been set by now!");
            service.AddDelegate(ServiceWrapper);
            InvokeUnityMethod(UnityDeviceDelegate.LEDeviceDidAddService, wrapperSerialization.SerializeService(service));
        }

        public void DidRemoveService(IDevice device, IService service)
        {
            Debug.Assert(ServiceWrapper != null, "Service wrapper should have been set by now!");
            service.RemoveDelegate(ServiceWrapper);
            InvokeUnityMethod(UnityDeviceDelegate.LEDeviceDidRemoveService, wrapperSerialization.SerializeService(service, true));
        }

        public void DidFailToAddServiceWithError(IDevice device, Exception error)
        {
            InvokeUnityMethod(UnityDeviceDelegate.LEDeviceDidFailToAddServiceWithError, wrapperSerialization.SerializeDeviceError(device, error));
        }

        #endregion
    }
}
