﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using LegoDeviceSDK.Generic;
using LegoDeviceSDK.Wrapper.Helpers;
using LegoDeviceSDK.Interfaces;
using System.Diagnostics;

namespace LegoDeviceSDK.Wrapper
{
    public class LoggerWrapper : ILoggerWrapper, ILogWriter
    {
        private readonly SdkLogger wrappedLogger;

        public LoggerWrapper(SdkLogger logger)
        {
            wrappedLogger = logger;
            logger.LogWriter = this;
        }

        internal LoggerWrapper() :
            this(SdkLogger.DefaultLogger)
        { }

        public void SetLogLevel(int level)
        {
            wrappedLogger.LogLevel = (LoggerLevel)level;
        }

        public void WriteMessage(string message, LoggerLevel level)
        {
            Debug.WriteLine(string.Format("[LegoDeviceSDK] {0}: {1}", LevelToString(level), message));
        }

        private string LevelToString(LoggerLevel level) {
            switch (level) {
                case LoggerLevel.LoggerLevelVerbose:
                    return "Verbose";
                case LoggerLevel.LoggerLevelDebug:
                    return "Debug";
                case LoggerLevel.LoggerLevelInfo:
                    return "Info";
                case LoggerLevel.LoggerLevelWarn:
                    return "Warn";
                case LoggerLevel.LoggerLevelError:
                    return "Error";
                case LoggerLevel.LoggerLevelFatal:
                    return "Fatal";
                default:
                    return "Info";
            }
        }
    }
}
