﻿using System;
using System.Linq;
using System.Reflection;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;
using LegoBleTestApp.Services;
using LegoBleTestApp.ViewModel;
using Microsoft.Practices.ServiceLocation;

namespace LegoBleTestApp.Common
{
    [Windows.Foundation.Metadata.WebHostHidden]
    public class BasePage : Page
    {

        protected NavigationHelper NavigationHelper { get; private set; }

        public BasePage()
        {
            NavigationHelper = new NavigationHelper(this);
            NavigationHelper.LoadState += navigationHelper_LoadState;
            NavigationHelper.SaveState += navigationHelper_SaveState;
        }

        #region NavigationHelper registration

        /// The methods provided in this section are simply used to allow
        /// NavigationHelper to respond to the page's navigation methods.
        /// 
        /// Page specific logic should be placed in event handlers for the  
        /// <see cref="GridCS.Common.NavigationHelper.LoadState"/>
        /// and <see cref="GridCS.Common.NavigationHelper.SaveState"/>.
        /// The navigation parameter is available in the LoadState method 
        /// in addition to page state preserved during an earlier session.
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            NavigationHelper.OnNavigatedTo(e);
	        ((ExtendedViewModelBase) DataContext).OnNavigatedTo();
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            NavigationHelper.OnNavigatedFrom(e);
						((ExtendedViewModelBase)DataContext).OnNavigatedFrom();
				}


        private Type GenericTypeImplemented(Type toCheck, Type generic)
        {
            var type = toCheck.GetTypeInfo().ImplementedInterfaces.FirstOrDefault(t => t.GetTypeInfo().IsGenericType && t.GetGenericTypeDefinition() == generic);
            return type;
        }

        private void navigationHelper_LoadState(object sender, LoadStateEventArgs e)
        {

            if (this.DataContext is IStateLocation)
            {
                var stateLocation = this.DataContext as IStateLocation;
                var genericType = GenericTypeImplemented(stateLocation.GetType(), typeof(IStateLocation<>));
                if (genericType != null)
                {
                    Type parameterType = genericType.GenericTypeArguments.First();
                    MethodInfo methodInfo = genericType.GetTypeInfo().GetDeclaredMethod("LoadState");
                    if (methodInfo != null)
                    {
                        var parameterHandler = ServiceLocator.Current.GetInstance<INavigationParameterHandler>();
                        var parameter = parameterHandler.DeserializeToValidValue(e.NavigationParameter, parameterType);
                        methodInfo.Invoke(stateLocation, new object[] { parameter, e.PageState });
                    }
                }
                else
                {
                    //Without parameter
                    stateLocation.LoadState(e.PageState);
                }
            }
            else if (this.DataContext is ILocation)
            {
                var locationViewModel = this.DataContext as ILocation;
                var genericType = GenericTypeImplemented(locationViewModel.GetType(), typeof(ILocation<>));
                if (genericType != null)
                {
                    Type parameterType = genericType.GenericTypeArguments.First();
                    MethodInfo methodInfo = genericType.GetTypeInfo().GetDeclaredMethod("LoadParameter");
                    if (methodInfo != null)
                    {
                        var parameterHandler = ServiceLocator.Current.GetInstance<INavigationParameterHandler>();
                        var parameter = parameterHandler.DeserializeToValidValue(e.NavigationParameter, parameterType);
                        methodInfo.Invoke(locationViewModel, new object[] { parameter });
                    }
                }
                else
                {
                    //Simple ILocation without parameter or state
                }
            }
        }
        private void navigationHelper_SaveState(object sender, SaveStateEventArgs e)
        {
            if (this.DataContext is ISaveState)
            {
                var navigableViewModel = this.DataContext as ISaveState;
                navigableViewModel.SaveState(e.PageState);
            }
        }

        #endregion
    }
}
