﻿using System;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Data;

namespace LegoBleTestApp.Converters {
	/// <summary>
	/// Converts a boolean value to UI visibility
	/// </summary>
	public class BooleanToVisibilityConverter : IValueConverter {
		/// <summary>
		/// Constructor that inits the visible and collapsed visibilities to the true and false values
		/// </summary>
		public BooleanToVisibilityConverter() {
			TrueValue = Visibility.Visible;
			FalseValue = Visibility.Collapsed;
		}

		public Visibility TrueValue { get; set; }
		public Visibility FalseValue { get; set; }

		/// <summary>
		/// Converts the boolean value to either Visible or Collapsed
		/// </summary>
		/// <param name="value">The value of the boolean</param>
		/// <param name="targetType"></param>
		/// <param name="parameter">Invert inverts the value</param>
		/// <param name="language">The language value</param>
		/// <returns>Visible if true and collapsed when false. Inverted when invert is given as parameter</returns>
		public object Convert(object value, Type targetType, object parameter, string language) {
			bool inverse = parameter as string == "invert";
			if (value is bool) {
				bool val = System.Convert.ToBoolean(value);
				return val ^ inverse ? TrueValue : FalseValue;
			}
			return null;
		}

		/// <summary>
		/// Converts Visible or Collapsed back to a boolean value
		/// </summary>
		/// <param name="value">The value of the visiblility</param>
		/// <param name="targetType"></param>
		/// <param name="parameter">Invert inverts the value</param>
		/// <param name="language">The language value</param>
		/// <returns>True if visible and false when collapsed. Inverted when invert is given as parameter</returns>
		public object ConvertBack(object value, Type targetType, object parameter, string language) {
			bool inverse = parameter as string == "invert";
			return TrueValue.Equals(value) ^ inverse;
		}
	}
}
