﻿using Presenter.Navigation;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LegoBleTestApp.Services
{
    public class ViewService : IViewService
    {
        private readonly INavigationService navigationService;
        private readonly IViewResolver viewResolver;
        private readonly INavigationParameterHandler navigationParameterHandler;

        public ViewService(INavigationService navigationService, IViewResolver viewResolver, INavigationParameterHandler navigationParameterHandler)
        {
            this.navigationService = navigationService;
            this.viewResolver = viewResolver;
            this.navigationParameterHandler = navigationParameterHandler;
        }

        //public event EventHandler Back;

        public bool CanGoBack
        {
            get { return navigationService.CanGoBack; }
        }

        public Type CurrentPage
        {
            get { return viewResolver.ResolveViewModel(navigationService.CurrentSourcePageType); }
        }

        public void NavigateTo<TLocation>() where TLocation : ILocation
        {
            var viewType = viewResolver.ResolveView(typeof(TLocation));
            this.navigationService.NavigateTo(viewType);
        }

        public void NavigateTo<TLocation, TArgument>(TArgument argument) where TLocation : ILocation<TArgument>
        {
            var viewType = viewResolver.ResolveView(typeof(TLocation));
		        this.navigationService.NavigateTo(viewType, navigationParameterHandler.SerializeToValidValue(argument)); //TODO: Serialize
        }

        public void NavigateToAndClearStack<TLocation>() where TLocation : ILocation
        {
            this.navigationService.ClearNavigationStack();
            this.NavigateTo<TLocation>();
        }

        public void NavigateToAndClearStack<TLocation, TArgument>(TArgument argument) where TLocation : ILocation<TArgument>
        {
            this.navigationService.ClearNavigationStack();
            this.NavigateTo<TLocation, TArgument>(argument);
        }

        public void GoBack()
        {
            navigationService.GoBack();
        }
    }
}
