﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GalaSoft.MvvmLight.Command;
using LegoBleTestApp.Services;
using LegoDeviceSDK.Bluetooth;
using LegoDeviceSDK.Generic;
using LegoDeviceSDK.Generic.Data;
using LegoDeviceSDK.Interfaces;

namespace LegoBleTestApp.ViewModel {
	/// <summary>
	/// View that handles all generic services and those without special UI needs
	/// </summary>
	public class GenericServiceViewModel : ExtendedViewModelBase, ILocation<Tuple<int, int>> {
		readonly DeviceManager deviceManager;
		readonly IViewService viewService;
		readonly StandardServiceControlViewModel standardServiceControl;
		readonly ModifyInputFormatControlViewModel modifyInputFormatControl;

		IService service;
		public IService Service {
			get { return service; }
			private set { Set(ref service, value); }
		}

		/// <summary>
		/// Command to go back to the Device view
		/// </summary>
		public RelayCommand GoBackCommand { get { return new RelayCommand(viewService.GoBack); } }

		string serviceName = "Generic";
		/// <summary>
		/// name of the service
		/// </summary>
		public string ServiceName {
			get { return serviceName; }
			private set { Set(ref serviceName, value); }
		}

		/// <summary>
		/// LoadParameter is used in navigating between views. Here the correct device and service is loaded.
		/// </summary>
		/// <param name="parameter">Tuple of integers that represents the index of the device in the device list of the device manager, and the index of the service of that device</param>
		public void LoadParameter(Tuple<int, int> parameter) {
			Service = deviceManager.AllDevices[parameter.Item1].Services[parameter.Item2];
			if (Service == null) {
				viewService.NavigateTo<DeviceViewModel>();
				return;
			}
			ServiceName = Service.ConnectInfo.TypeString;
			standardServiceControl.Service = Service;
			modifyInputFormatControl.Service = Service;
		}

		public GenericServiceViewModel(DeviceManager deviceManager, IViewService viewService, StandardServiceControlViewModel standardServiceControl, ModifyInputFormatControlViewModel modifyInputFormatControl) {
			this.deviceManager = deviceManager;
			this.viewService = viewService;
			this.standardServiceControl = standardServiceControl;
			this.modifyInputFormatControl = modifyInputFormatControl;
		}

	}
}
