using GalaSoft.MvvmLight.Command;
using GalaSoft.MvvmLight.Threading;
using LegoBleTestApp.Services;
using LegoDeviceSDK.Generic;
using LegoDeviceSDK.Helpers;

namespace LegoBleTestApp.ViewModel {
	/// <summary>
	/// This class contains properties that the main View can data bind to.
	/// <para>
	/// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
	/// </para>
	/// <para>
	/// You can also use Blend to data bind with the tool's support.
	/// </para>
	/// <para>
	/// See http://www.galasoft.ch/mvvm
	/// </para>
	/// </summary>
	public class MainViewModel : ExtendedViewModelBase, ILocation {
		readonly IViewService viewService;
		readonly DeviceManager deviceManager;
		/// <summary>
		/// Initializes a new instance of the MainViewModel class.
		/// </summary>
		public MainViewModel(IViewService viewService, DeviceManager deviceManager) {
			this.viewService = viewService;
			this.deviceManager = deviceManager;
			Version = deviceManager.Version;
		}

		public string version;

		public string Version {
			get { return version; }
			private set { Set(ref version, value); }
		}

		public RelayCommand GotoLeListing {
			get { return new RelayCommand(() => viewService.NavigateTo<DeviceListingViewModel>()); }
		}

		void Initialize() {
			
		}
	}
}