﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GalaSoft.MvvmLight.Command;
using GalaSoft.MvvmLight.Threading;
using LegoDeviceSDK.Generic;
using LegoDeviceSDK.Interfaces;

namespace LegoBleTestApp.ViewModel {
	/// <summary>
	/// Control that shows the input format and can manipulate the input format
	/// </summary>
	public class ModifyInputFormatControlViewModel : ExtendedViewModelBase, IServiceDelegate {


		IService service;
		/// <summary>
		/// The service 
		/// </summary>
		public IService Service {
			get { return service; }
			set {
				bool changed = Set(ref service, value);
				if (changed) {
					if(service != null)
						service.RemoveDelegate(this);
					value.AddDelegate(this);
					if (value.InputFormat == null) {
						Mode = 0;
						DeltaInterval = 1;
						Unit = 0;
						NotificationsEnabled = false;
						return;
					}
					Mode = value.InputFormat.Mode;
					DeltaInterval = value.InputFormat.DeltaInterval;
					Unit = (int)value.InputFormat.Unit;
					NotificationsEnabled = value.InputFormat.NotificationsEnabled;
				}
			}
		}

		int mode;
		/// <summary>
		/// Mode of the service
		/// </summary>
		public int Mode {
			get { return mode; }
			set { Set(ref mode, value); }
		}

		uint deltaInterval;
		/// <summary>
		/// The delta interval of the service
		/// </summary>
		public uint DeltaInterval {
			get { return deltaInterval; }
			set { Set(ref deltaInterval, value); }
		}

		int unit;
		/// <summary>
		/// The unit of the service
		/// </summary>
		public int Unit {
			get { return unit; }
			set { Set(ref unit, value); }
		}

		bool notificationsEnabled = false;
		/// <summary>
		/// True if notifiactions are enabled for this service
		/// </summary>
		public bool NotificationsEnabled {
			get { return notificationsEnabled; }
			set { Set(ref notificationsEnabled, value); }
		}

		/// <summary>
		/// Command that updates the input format
		/// </summary>
		public RelayCommand SendUpdateCommand {
			get {
				return new RelayCommand(() => Service.UpdateInputFormat(InputFormat.InputFormatWithConnectId(Service.ConnectInfo.ConnectId, 
					(int)Service.ConnectInfo.TypeEnum,
					Mode,
					DeltaInterval,
					(InputFormatUnit)Unit,
					NotificationsEnabled)));
			}
		}

		/// <summary>
		/// Callback when the input format changes
		/// </summary>
		/// <param name="service">The service with the changed input format</param>
		/// <param name="oldFormat">The old input format</param>
		/// <param name="newFormat">The new input format</param>
		public void DidUpdateInputFormat(IService service, InputFormat oldFormat, InputFormat newFormat) {
			if (Service != null)
				DispatcherHelper.CheckBeginInvokeOnUI(() => RaisePropertyChanged("Service"));
		}

		/// <summary>
		/// Callback when the value changes
		/// </summary>
		/// <param name="service">The service with the changed value</param>
		/// <param name="oldData">The old value</param>
		/// <param name="newData">The new value</param>
		public void DidUpdateValueData(IService service, byte[] oldData, byte[] newData) {
			if (Service != null)
				DispatcherHelper.CheckBeginInvokeOnUI(() => RaisePropertyChanged("Service"));
		}

	}
}
