﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GalaSoft.MvvmLight.Command;
using LegoBleTestApp.Services;
using LegoDeviceSDK.Bluetooth;
using LegoDeviceSDK.Generic;

namespace LegoBleTestApp.ViewModel {
	/// <summary>
	/// The view for a motion sensor service
	/// </summary>
	public class MotionSensorViewModel : ExtendedViewModelBase, ILocation<Tuple<int, int>> {
		readonly DeviceManager deviceManager;
		readonly IViewService viewService;
		readonly StandardServiceControlViewModel standardServiceControl;
		readonly ModifyInputFormatControlViewModel modifyInputFormatControl;

		BluetoothMotionSensor sensor;
		/// <summary>
		/// The motion sensor to display
		/// </summary>
		public BluetoothMotionSensor Sensor {
			get { return sensor; }
			private set { Set(ref sensor, value); }
		}

		/// <summary>
		/// Command to go back to the device view
		/// </summary>
		public RelayCommand GoBackCommand { get { return new RelayCommand(viewService.GoBack); } }

		/// <summary>
		/// LoadParameter is used in navigating between views. Here the correct device and service is loaded.
		/// </summary>
		/// <param name="parameter">Tuple of integers that represents the index of the device in the device list of the device manager, and the index of the service of that device</param>
		public void LoadParameter(Tuple<int, int> parameter) {
			var service = deviceManager.AllDevices[parameter.Item1].Services[parameter.Item2];
			Sensor = service as BluetoothMotionSensor;
			if (Sensor == null) {
				viewService.NavigateTo<DeviceViewModel>();
				return;
			}
			standardServiceControl.Service = Sensor;
			modifyInputFormatControl.Service = Sensor;
		}

		public MotionSensorViewModel(DeviceManager deviceManager, IViewService viewService, StandardServiceControlViewModel standardServiceControl, ModifyInputFormatControlViewModel modifyInputFormatControl) {
			this.deviceManager = deviceManager;
			this.viewService = viewService;
			this.standardServiceControl = standardServiceControl;
			this.modifyInputFormatControl = modifyInputFormatControl;
		
		}

	}
}
