﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GalaSoft.MvvmLight.Command;
using GalaSoft.MvvmLight.Threading;
using LegoDeviceSDK.Bluetooth;
using LegoDeviceSDK.Generic;
using LegoDeviceSDK.Interfaces;

namespace LegoBleTestApp.ViewModel {
	/// <summary>
	/// Shows basic shared information about a service
	/// </summary>
	public class StandardServiceControlViewModel : ExtendedViewModelBase, IServiceDelegate {

		IService service;
		/// <summary>
		/// The service to be shown
		/// </summary>
		public IService Service {
			get { return service; }
			set {
				var changed = Set(ref service, value);
				if(changed)
					value.AddDelegate(this);
			}
		}

		public float Value {
			get {
				if (service == null || Service.InputFormat == null)
					return 0f;

				if (Service.InputFormat.Unit == InputFormatUnit.InputFormatUnitSi)
					return Service.ValueAsFloat;

				return Service.ValueAsInteger;
			}
		}

		/// <summary>
		/// Command that sends a request to read the value of the Service
		/// </summary>
		public RelayCommand SendReadRequestCommand { get { return new RelayCommand(() => Service.SendReadValueRequest()); } }
		/// <summary>
		/// Command that resets the sensor state
		/// </summary>
		public RelayCommand ResetSensorStateCommand { get { return new RelayCommand(() => Service.SendResetStateRequest()); } }


		/// <summary>
		/// Callback when the input format changes
		/// </summary>
		/// <param name="service">The service with the changed input format</param>
		/// <param name="oldFormat">The old input format</param>
		/// <param name="newFormat">The new input format</param>
		public void DidUpdateInputFormat(IService service, InputFormat oldFormat, InputFormat newFormat) {
			if (Service != null)
				DispatcherHelper.CheckBeginInvokeOnUI(() => RaisePropertyChanged("Service"));
		}

		/// <summary>
		/// Callback when the value changes
		/// </summary>
		/// <param name="service">The service with the changed value</param>
		/// <param name="oldData">The old value</param>
		/// <param name="newData">The new value</param>
		public void DidUpdateValueData(IService service, byte[] oldData, byte[] newData) {
			if (Service != null)
				DispatcherHelper.CheckBeginInvokeOnUI(() => RaisePropertyChanged("Value"));
		}

		/// <summary>
		/// When navigating away, remove itself from the service callbacks
		/// </summary>
		public override void OnNavigatedFrom() {
			base.OnNavigatedFrom();
			if (Service != null)
				Service.RemoveDelegate(this);
		}
	}
}
