/*
  In App.xaml:
  <Application.Resources>
      <vm:ViewModelLocator xmlns:vm="clr-namespace:LegoBleTestApp"
                           x:Key="Locator" />
  </Application.Resources>
  
  In the View:
  DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"

  You can also use Blend to do all this with the tool's support.
  See http://www.galasoft.ch/mvvm
*/

using System;
using GalaSoft.MvvmLight.Ioc;
using LegoBleTestApp.Services;
using LegoDeviceSDK.Generic;
using Microsoft.Practices.ServiceLocation;
using Presenter.Navigation;

namespace LegoBleTestApp.ViewModel {
	/// <summary>
	/// This class contains static references to all the view models in the
	/// application and provides an entry point for the bindings.
	/// </summary>
	public class ViewModelLocator {
		/// <summary>
		/// Initializes a new instance of the ViewModelLocator class.
		/// </summary>
		public ViewModelLocator() {
			ServiceLocator.SetLocatorProvider(() => SimpleIoc.Default);

			//Services
			SimpleIoc.Default.Register<INavigationService, NavigationService>();
			SimpleIoc.Default.Register<IViewResolver, ViewResolver>();
			SimpleIoc.Default.Register<IViewService, ViewService>();
			SimpleIoc.Default.Register<INavigationParameterHandler, NavigationParameterHandler>();

			//Other stuff
			SimpleIoc.Default.Register<DeviceManager>();

			//Controls
			SimpleIoc.Default.Register<StandardServiceControlViewModel>();
			SimpleIoc.Default.Register<ModifyInputFormatControlViewModel>();

			//View Models
			SimpleIoc.Default.Register<MainViewModel>();
			SimpleIoc.Default.Register<GenericServiceViewModel>();
			SimpleIoc.Default.Register<DeviceListingViewModel>();
			SimpleIoc.Default.Register<DeviceViewModel>();
			SimpleIoc.Default.Register<MotorViewModel>();
			SimpleIoc.Default.Register<MotionSensorViewModel>();
			SimpleIoc.Default.Register<PiezoTonePlayerViewModel>();
			SimpleIoc.Default.Register<RgbLightViewModel>();
			SimpleIoc.Default.Register<TiltSensorViewModel>();
			SimpleIoc.Default.Register<WindMillTestViewModel>();

		}

		T ViewModel<T>() where T : class {
			if (SimpleIoc.Default.IsRegistered<T>())
				return ServiceLocator.Current.GetInstance<T>();

			return Activator.CreateInstance<T>();
		}

		public MainViewModel Main {
			get { return ViewModel<MainViewModel>(); }
		}

		public GenericServiceViewModel GenericService {
			get { return ViewModel<GenericServiceViewModel>(); }
		}

		public DeviceListingViewModel DeviceListing {
			get { return ViewModel<DeviceListingViewModel>(); }
		}

		public DeviceViewModel Device {
			get { return ViewModel<DeviceViewModel>(); }
		}

		public MotorViewModel Motor {
			get { return ViewModel<MotorViewModel>(); }
		}

		public MotionSensorViewModel MotionSensor {
			get { return ViewModel<MotionSensorViewModel>(); }
		}

		public PiezoTonePlayerViewModel PiezoTonePlayer {
			get { return ViewModel<PiezoTonePlayerViewModel>(); }
		}

		public RgbLightViewModel RgbLight {
			get { return ViewModel<RgbLightViewModel>(); }
		}

		public TiltSensorViewModel TiltSensor {
			get { return ViewModel<TiltSensorViewModel>(); }
		}

		public WindMillTestViewModel WindMillTest {
			get { return ViewModel<WindMillTestViewModel>(); }
		}

		public StandardServiceControlViewModel StandardServiceControl {
			get { return ViewModel<StandardServiceControlViewModel>(); }
		}

		public ModifyInputFormatControlViewModel ModifyInputFormatControl {
			get { return ViewModel<ModifyInputFormatControlViewModel>(); }
		}

		public static void Cleanup() {
		}
	}
}