﻿using System;
using System.Text;
using LegoDeviceSDK.Generic;
using LegoDeviceSDK.Wrapper;

namespace LegoDeviceSDK.Bluetooth.Definitions {
	internal class BluetoothCharacteristicDefinition {
		const string Tag = "BluetoothCharacteristicDefinition";
		internal BluetoothServiceDefinition ServiceDefinition { get; set; }
		internal string Name { get; private set; }
		internal Guid Uuid { get; private set; }
		internal bool Mandatory { get; private set; }
		internal int MandatoryProperties { get; private set; }
		internal int RecommendedProperties { get; private set; }
		internal int Permissions { get; private set; }


		public BluetoothCharacteristicDefinition(String name,
																		BluetoothServiceDefinition serviceDefinition,
																		Guid uuid,
                                    bool mandatory,
                                    int mandatoryProperties,
                                    int recommendedProperties,
                                    int permissions) {
        Name = name;
        ServiceDefinition = serviceDefinition;
        Uuid = uuid;
        Mandatory = mandatory;
        MandatoryProperties = mandatoryProperties;
        RecommendedProperties = recommendedProperties;
        Permissions = permissions;
    }


		internal bool MatchesCharacteristic(BluetoothCharacteristicDefinition characteristic) {
			return (Uuid.Equals(characteristic.Uuid));
		}

		public override string ToString() {
			var builder = new StringBuilder();
			builder.Append(String.Format("<{0}: ", this.GetType()));
			builder.Append(String.Format(", serviceName={0}", this.ServiceDefinition.ServiceName));
			builder.Append(String.Format(", UUID={0}", this.Uuid.ToString()));
			builder.Append(String.Format(", isMandatory={0}", this.Mandatory));
			builder.Append(">");

			return builder.ToString();
		}


		public bool Equals(BluetoothCharacteristicDefinition characteristic) {
			if (this == characteristic) {
				return true;
			}

			if (characteristic == null) {
				return false;
			}

			if (this.Name != characteristic.Name && !(this.Name.Equals(characteristic.Name))) {
				return false;
			}

			if (this.Uuid != characteristic.Uuid && !(this.Uuid.Equals(characteristic.Uuid))) {
				return false;
			}

			if (this.Mandatory != characteristic.Mandatory) {
				return false;
			}

			if (this.MandatoryProperties != characteristic.MandatoryProperties) {
				return false;
			}

			if (this.RecommendedProperties != characteristic.RecommendedProperties) {
				return false;
			}

			return true;
		}

	}
}
