﻿using LegoDeviceSDK.Helpers;
using LegoDeviceSDK.Wrapper;

namespace LegoDeviceSDK.Bluetooth.Definitions {
	internal class DeviceInfoServiceDefinition : BluetoothServiceDefinition {

		internal const ushort DeviceInfoServiceUuid = 0x180A;

		internal const ushort DeviceInfoFirmwareRevisionCharacteristicUuid = 0x2A26;
		internal const ushort DeviceInfoHardwareRevisionCharacteristicUuid = 0x2A27;
		internal const ushort DeviceInfoSoftwareRevisionCharacteristicUuid = 0x2A28;
		internal const ushort DeviceInfoManufacturerNameCharacteristicUuid = 0x2A29;

		internal DeviceInfoServiceDefinition(IGattDeviceService gattDeviceService)
			: base(gattDeviceService) {
			Init();
		}

		void Init() {
			ServiceUuid = Windows.Devices.Bluetooth.GenericAttributeProfile.GattDeviceService.ConvertShortIdToUuid(DeviceInfoServiceUuid);
			ServiceName = "DeviceInfoService";

			var firmwareRevision = new BluetoothCharacteristicDefinition(
							"FirmwareRevision",
							this,
							Windows.Devices.Bluetooth.GenericAttributeProfile.GattDeviceService.ConvertShortIdToUuid(DeviceInfoFirmwareRevisionCharacteristicUuid),
							true,
							((int)BluetoothHelper.Properties.PROPERTY_READ),
							0,
							((int)BluetoothHelper.Permissions.PERMISSION_READ));

			var hardwareRevision = new BluetoothCharacteristicDefinition(
							"HardwareRevision",
							this,
							Windows.Devices.Bluetooth.GenericAttributeProfile.GattDeviceService.ConvertShortIdToUuid(DeviceInfoHardwareRevisionCharacteristicUuid),
							false,
							((int)BluetoothHelper.Properties.PROPERTY_READ),
							0,
							((int)BluetoothHelper.Permissions.PERMISSION_READ));

			var softwareRevision = new BluetoothCharacteristicDefinition(
							"SoftwareRevision",
							this,
							Windows.Devices.Bluetooth.GenericAttributeProfile.GattDeviceService.ConvertShortIdToUuid(DeviceInfoSoftwareRevisionCharacteristicUuid),
							true,
							((int)BluetoothHelper.Properties.PROPERTY_READ),
							0,
							((int)BluetoothHelper.Permissions.PERMISSION_READ));

			var manufacturerName = new BluetoothCharacteristicDefinition(
							"ManufacturerName",
							this,
							Windows.Devices.Bluetooth.GenericAttributeProfile.GattDeviceService.ConvertShortIdToUuid(DeviceInfoManufacturerNameCharacteristicUuid),
							true,
							((int)BluetoothHelper.Properties.PROPERTY_READ),
							0,
							((int)BluetoothHelper.Permissions.PERMISSION_READ));

			CharacteristicDefinitions.Add(firmwareRevision);
			CharacteristicDefinitions.Add(hardwareRevision);
			CharacteristicDefinitions.Add(softwareRevision);
			CharacteristicDefinitions.Add(manufacturerName);
		}

	}
}
