﻿using System;
using LegoDeviceSDK.Helpers;
using LegoDeviceSDK.Wrapper;

namespace LegoDeviceSDK.Bluetooth.Definitions {
	internal class DeviceServiceDefinition : BluetoothServiceDefinition {

		internal const string HubService16BitUuid = "1523";
		internal const string HubCharacteristicNameUuid = "1524";
		internal const string HubCharacteristicButtonState = "1526";
		internal const string HubCharacteristicAttachedIo = "1527";
		internal const string HubCharacteristicDisconnect = "152B";
        internal const string HubCharacteristicLowVoltageAlert = "1528";

		public DeviceServiceDefinition(IGattDeviceService gattDeviceService)
			: base(gattDeviceService) {
			Init();
		}

		void Init() {
			ServiceUuid = ServiceUuid = GuidHelper.GetFullGuid(HubService16BitUuid);
			ServiceName = "DeviceService";

			var deviceName = new BluetoothCharacteristicDefinition(
				"DeviceName",
				this,
				GuidHelper.GetFullGuid(HubCharacteristicNameUuid),
				true,
				((int) BluetoothHelper.Properties.PROPERTY_WRITE | (int) BluetoothHelper.Properties.PROPERTY_WRITE_NO_RESPONSE | (int) BluetoothHelper.Properties.PROPERTY_READ),
				0,
				((int) BluetoothHelper.Permissions.PERMISSION_WRITE | (int) BluetoothHelper.Permissions.PERMISSION_READ));

			var attachedIO = new BluetoothCharacteristicDefinition(
				"AttachedIO",
				this,
				GuidHelper.GetFullGuid(HubCharacteristicAttachedIo),
				true,
				((int)BluetoothHelper.Properties.PROPERTY_NOTIFY),
				0,
				((int)BluetoothHelper.Permissions.PERMISSION_READ));

			var deviceButton = new BluetoothCharacteristicDefinition(
				"DeviceButton",
				this,
				GuidHelper.GetFullGuid(HubCharacteristicButtonState),
				true,
				((int)BluetoothHelper.Properties.PROPERTY_READ | (int)BluetoothHelper.Properties.PROPERTY_NOTIFY),
				0,
				((int)BluetoothHelper.Permissions.PERMISSION_READ));

			var disconnect = new BluetoothCharacteristicDefinition(
				"Disconnect",
				this,
				GuidHelper.GetFullGuid(HubCharacteristicDisconnect),
				true,
				((int)BluetoothHelper.Properties.PROPERTY_WRITE_NO_RESPONSE),
				0,
				((int)BluetoothHelper.Permissions.PERMISSION_WRITE));

           var lowVoltageAlert = new BluetoothCharacteristicDefinition(
				"LowVoltageAlert",
				this,
				GuidHelper.GetFullGuid(HubCharacteristicLowVoltageAlert),
				false,
                ((int)BluetoothHelper.Properties.PROPERTY_READ | (int)BluetoothHelper.Properties.PROPERTY_NOTIFY),
                0,
                ((int)BluetoothHelper.Permissions.PERMISSION_READ));

			CharacteristicDefinitions.Add(deviceName);
			CharacteristicDefinitions.Add(attachedIO);
			CharacteristicDefinitions.Add(deviceButton);
			CharacteristicDefinitions.Add(disconnect);
            CharacteristicDefinitions.Add(lowVoltageAlert);
		}

	}
}
