﻿using LegoDeviceSDK.Helpers;
using LegoDeviceSDK.Wrapper;

namespace LegoDeviceSDK.Bluetooth.Definitions {
	internal class IoServiceDefinition : BluetoothServiceDefinition {

		internal const string InputServiceUuid = "4F0E";

		internal const string CharacteristicInputValueUuid = "1560";
		internal const string CharacteristicInputFormatUuid = "1561";
		internal const string CharacteristicInputCommandUuid = "1563";
		internal const string CharacteristicOutputCommandUuid = "1565";

		internal BluetoothCharacteristicDefinition InputValue;
		internal BluetoothCharacteristicDefinition InputFormat;
		internal BluetoothCharacteristicDefinition InputCommand;
		internal BluetoothCharacteristicDefinition OutputCommand;

		internal IoServiceDefinition(IGattDeviceService gattDeviceService) : base(gattDeviceService) {
			Init();
		}

		void Init() {
			ServiceUuid = GuidHelper.GetFullGuid(InputServiceUuid);
			ServiceName = "IOService";

			InputValue = new BluetoothCharacteristicDefinition(
				"InputValue",
				this,
				GuidHelper.GetFullGuid(CharacteristicInputValueUuid),
				true,
				((int) BluetoothHelper.Properties.PROPERTY_NOTIFY),
				0,
				((int) BluetoothHelper.Permissions.PERMISSION_READ));

			InputFormat = new BluetoothCharacteristicDefinition(
				"InputFormat",
				this,
				GuidHelper.GetFullGuid(CharacteristicInputFormatUuid),
				true,
				((int) BluetoothHelper.Properties.PROPERTY_NOTIFY),
				0,
				((int) BluetoothHelper.Permissions.PERMISSION_READ));

			InputCommand = new BluetoothCharacteristicDefinition(
				"InputCommand",
				this,
				GuidHelper.GetFullGuid(CharacteristicInputCommandUuid),
				true,
				((int) BluetoothHelper.Properties.PROPERTY_WRITE_NO_RESPONSE | (int) BluetoothHelper.Properties.PROPERTY_WRITE),
				0,
				((int) BluetoothHelper.Permissions.PERMISSION_READ));

			OutputCommand = new BluetoothCharacteristicDefinition(
				"OutputCommand",
				this,
				GuidHelper.GetFullGuid(CharacteristicOutputCommandUuid),
				true,
				((int) BluetoothHelper.Properties.PROPERTY_WRITE_NO_RESPONSE | (int) BluetoothHelper.Properties.PROPERTY_WRITE),
				0,
				((int) BluetoothHelper.Permissions.PERMISSION_READ));

			CharacteristicDefinitions.Add(InputValue);
			CharacteristicDefinitions.Add(InputFormat);
			CharacteristicDefinitions.Add(InputCommand);
			CharacteristicDefinitions.Add(OutputCommand);

		}
	}
}
