﻿namespace LegoDeviceSDK.Generic {
	/// <summary>
	/// The connect state of the Device
	/// </summary>
	public enum DeviceState {
		/// <summary>
		/// The Device is disconnected and no connection attempt is in progress
		/// </summary>
        DeviceStateDisconnected,
        /// <summary>
        /// This state is provided only for compatibility with the Unity SDK wrapper. It must not be used directly.
        /// While other native SDKs differentiate between a disconnected device advertising its presence or not,
        /// on WinRT a discovered device is always considered to be advertising.
        /// </summary>
        // TODO: Can we treat disconnected devices known by OS as DeviceStateDisconnectedNotAdvertising,
        // and connected ones (not by the SDK, but the OS itself) as DeviceStateDisconnectedAdvertising?
        __DeviceStateDisconnectedNotAdvertising,
		/// <summary>
		/// A connection attempt is in progress
		/// </summary>
		DeviceStateConnecting,
		/// <summary>
		/// Connected and interrogating Device for required services
		/// </summary>
		DeviceStateInterrogating,
		/// <summary>
		/// Connected and interrogation complete - device is ready for use
		/// </summary>
		DeviceStateInterrogationFinished,
	}
}
