﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LegoDeviceSDK.Wrapper {
	/// <summary>
	/// Wraps a Windows GattDeviceService
	/// </summary>
	internal class GattDeviceService : IGattDeviceService {
		/// <summary>
		/// The GattDeviceService to wrap
		/// </summary>
		internal readonly Windows.Devices.Bluetooth.GenericAttributeProfile.GattDeviceService gattDeviceService;

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="gattDeviceService">The GattDeviceService to wrap</param>
		internal GattDeviceService(Windows.Devices.Bluetooth.GenericAttributeProfile.GattDeviceService gattDeviceService) {
			this.gattDeviceService = gattDeviceService;
		}

		/// <summary>
		/// Gets a characteristic from the Service
		/// </summary>
		/// <param name="characteristicUuid">The Uuid of the characteristic</param>
		/// <returns>The characteristic with the specified Uuid</returns>
		public IReadOnlyList<GattCharacteristic> GetCharacteristics(Guid characteristicUuid) {
			var characteristics = gattDeviceService.GetCharacteristics(characteristicUuid);
			return new List<GattCharacteristic>(characteristics.Select(characteristic => new GattCharacteristic(characteristic)));
		}

		/// <summary>
		/// Disposes of the Service
		/// </summary>
		public void Dispose() {
			gattDeviceService.Dispose();
		}

		/// <summary>
		/// Getter for the Service Uuid
		/// </summary>
		public Guid Uuid { get { return gattDeviceService.Uuid; } }
	}
}
