﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using LegoDeviceSDK.Bluetooth;
using LegoDeviceSDK.Generic;
using NUnit.Framework;

namespace LegoSDKNunitTest.SDK.Bluetooth {
	[TestFixture]
	public class BluetoothInputCommandTest {

		[Test]
		public void TestWriteInputFormat() {
			var connectInfo = new ConnectInfo() {ConnectId = 1, HubIndex = 2, TypeEnum = IoType.IoTypeMotor};
			var inputFormat = InputFormat.InputFormatWithConnectId(connectInfo.ConnectId, (int) connectInfo.TypeEnum, 9, 10, InputFormatUnit.InputFormatUnitRaw, true);
			var deltaBytes = BitConverter.GetBytes(inputFormat.DeltaInterval);
			var expectedDataWritten = new byte[] {
				(byte) 1,
				(byte) 2,
				(byte) connectInfo.ConnectId,
				(byte) connectInfo.TypeEnum,
				(byte) inputFormat.Mode,
				(byte) deltaBytes[0],
				(byte) deltaBytes[1],
				(byte) deltaBytes[2],
				(byte) deltaBytes[3],
				(byte) inputFormat.Unit,
				(byte) (inputFormat.NotificationsEnabled ? 1 : 0),
			};

			var inputCommand = BluetoothInputCommand.WriteInputFormat(inputFormat, connectInfo.ConnectId);

			Assert.That(inputCommand.Data.Length == expectedDataWritten.Length);
			for (int i = 0; i < expectedDataWritten.Length; i++) {
				Assert.That(inputCommand.Data[i] == expectedDataWritten[i]);
			}

		}

		[Test]
		public void TestReadInputValues() {
			var expectedDataWritten = new byte[] {
				(byte) 0,
				(byte) 1,
				(byte) 1,
			};

			var inputCommand = BluetoothInputCommand.ReadValueForConnectId(1);

			Assert.That(inputCommand.Data.Length == expectedDataWritten.Length);
			for (int i = 0; i < expectedDataWritten.Length; i++) {
				Assert.That(inputCommand.Data[i] == expectedDataWritten[i]);
			}

		}



	}
}
