﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using LegoDeviceSDK.Bluetooth;
using NUnit.Framework;

namespace LegoSDKNunitTest.SDK.Bluetooth {
	[TestFixture]
	public class BluetoothOutputCommandTest {
		const int connectId = 1;

		[Test]
		public void TestWriteMotorPowerCommand() {
			int writtenPower = 100;
			var outputCommand = BluetoothOutputCommand.WriteMotorPowerCommand(writtenPower, connectId);

			var data = outputCommand.Data;
			Assert.That(data.Length == 4);

			Assert.That(data[0] == connectId);
			var expectedCommandId = 1;
			Assert.That(data[1] == expectedCommandId);

			var expectedNumberOfBytes = 1;
			Assert.That(data[2] == expectedNumberOfBytes);

			Assert.That(data[3] == writtenPower);
		}

		[Test]
		public void TestWriteMotorPowerCommandNegativeSpeed() {
			int writtenPower = -100;
			var outputCommand = BluetoothOutputCommand.WriteMotorPowerCommand(writtenPower, connectId);

			var data = outputCommand.Data;
			Assert.That(data.Length == 4);

			Assert.That(data[0] == connectId);
			var expectedCommandId = 1;
			Assert.That(data[1] == expectedCommandId);

			var expectedNumberOfBytes = 1;
			Assert.That(data[2] == expectedNumberOfBytes);

			Assert.That((data[3] - 256) == writtenPower);
		}

		[Test]
		public void TestWritePiezoToneFrequencyCommand() {
			var frequency = 440;
			var duration = 1000;
			var outputCommand = BluetoothOutputCommand.WritePiezoToneFrequency(frequency, duration, connectId);

			var data = outputCommand.Data;
			Assert.That(data.Length == 7);

			Assert.That(data[0] == connectId);
			var expectedCommandId = 2;
			Assert.That(data[1] == expectedCommandId);

			var expectedNumberOfBytes = 4;
			Assert.That(data[2] == expectedNumberOfBytes);

			var writtenFrequency = BitConverter.ToUInt16(data, 3);
			Assert.That(writtenFrequency == frequency);

			var writtenDuration = BitConverter.ToUInt16(data, 5);
			Assert.That(writtenDuration == duration);
		}

		[Test]
		public void TestWritePiezoToneStopCommand() {
			var outputCommand = BluetoothOutputCommand.WritePiezoToneStop(connectId);

			var data = outputCommand.Data;
			Assert.That(data.Length == 3);

			Assert.That(data[0] == connectId);
			var expectedCommandId = 3;
			Assert.That(data[1] == expectedCommandId);

			var expectedNumberOfBytes = 0;
			Assert.That(data[2] == expectedNumberOfBytes);
		}

		[Test]
		public void TestWriteRGBCommand() {
			int red = 0;
			int green = 255;
			int blue = 128;
			var outputCommand = BluetoothOutputCommand.WriteRgbLight(red, green, blue, connectId);

			var data = outputCommand.Data;
			Assert.That(data.Length == 6);

			Assert.That(data[0] == connectId);
			var expectedCommandId = 4;
			Assert.That(data[1] == expectedCommandId);

			var expectedNumberOfBytes = 3;
			Assert.That(data[2] == expectedNumberOfBytes);

			Assert.That(data[3] == red);
			Assert.That(data[4] == green);
			Assert.That(data[5] == blue);
		}

	}
}
