﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using LegoDeviceSDK.Bluetooth;
using LegoDeviceSDK.Generic;
using LegoDeviceSDK.Interfaces;
using Windows.UI;

namespace LegoSDKNunitTest.SDK.Bluetooth.Mocks {
	class BluetoothServiceIoMock : IIo {

		List<BluetoothService> ioDelegates = new List<BluetoothService>();
		InputFormat sentInputFormat;
		int sentConnectId;
		public int LastWrittenMotorPower;
		public int LastFrequencyWritten;
		public int LastDurationWritten;
        public Color LastColorWritten;
        public int LastColorIndexWritten;

		public void ReadValueForConnectId(int connectId) {
		}

		public void ResetStateForConnectId(int connectId) {
		}

		public void WriteInputFormat(InputFormat inputFormat, int connectId) {
			sentInputFormat = inputFormat;
			sentConnectId = connectId;
			foreach (var ioDelegate in ioDelegates) {
				ioDelegate.DidReceiveInputFormat(this, inputFormat);
			}
		}

		public void ReadInputFormatForConnectId(int connectId) {
		}

		public void WriteMotorPower(int power, int connectId) {
            WriteMotorPower(power, 0, connectId);
		}

        public void WriteMotorPower(int power, int offset, int connectId) {
            LastWrittenMotorPower = power;
        }

		public void WritePiezoToneFrequency(int frequency, int milliseconds, int connectId) {
			LastFrequencyWritten = frequency;
			LastDurationWritten = milliseconds;
		}

		public void WritePiezoToneStop(int connectId) {
		}

		public void WriteColor(int red, int green, int blue, int connectId) {
            LastColorWritten = Color.FromArgb(0xFF, (byte)red, (byte)green, (byte)blue);
			SdkError error;
			foreach (var ioDelegate in ioDelegates) {
				ioDelegate.HandleUpdatedValueData(new byte[] { (byte)red, (byte)green, (byte)blue }, out error);
			}
		}

        public void WriteColorIndex(int index, int connectId) {
            LastColorIndexWritten = index;
            SdkError error;
            foreach (var ioDelegate in ioDelegates) {
                ioDelegate.HandleUpdatedValueData(new byte[] { (byte)index }, out error);
            }
        }

		public void WriteData(byte[] data, int connectId) {
		}

		public void AddDelegate(BluetoothService ioDelegate) {
			ioDelegates.Add(ioDelegate);
		}

		public void RemoveDelegate(BluetoothService ioDelegate) {
			ioDelegates.Remove(ioDelegate);
		}

    }
}
