﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using LegoDeviceSDK.Bluetooth;
using LegoDeviceSDK.Generic;
using LegoDeviceSDK.Interfaces;
using LegoSDKNunitTest.SDK.Bluetooth.Mocks;
using NUnit.Framework;
using LegoDeviceSDK.Interfaces.Services;

namespace LegoSDKNunitTest.SDK.Bluetooth.Services {
	[TestFixture]
	public class BluetoothCurrentTest : ICurrentSensorDelegate {
		AutoResetEvent testTrigger;
		float receivedMilliAmps;

		BluetoothCurrentSensor currentSensor;
		ConnectInfo connectInfo = new ConnectInfo() { ConnectId = 1, HubIndex = 1, TypeEnum = IoType.IoTypeCurrent };
		IIo ioStub = new BluetoothServiceIoMock();

		[SetUp]
		public void Setup() {
			testTrigger = new AutoResetEvent(false);
			ioStub = new BluetoothServiceIoMock();
			var service = BluetoothService.GetServiceFromConnectInfo(new BluetoothDeviceStub(), connectInfo, ioStub);
			currentSensor = service as BluetoothCurrentSensor;
			if (currentSensor == null)
				Assert.Fail();
			currentSensor.AddDelegate(this);
		}

		[TearDown]
		public void TearDown() {
			currentSensor.RemoveDelegate(this);
			currentSensor = null;
		}

		void UpdateInputFormatMode(InputFormatUnit unit, int numberOfBytes) {
			var data = BluetoothTestStupFactory.InputFormatWriteDataWithRevision(1, currentSensor.ConnectInfo.ConnectId, currentSensor.ConnectInfo.TypeEnum, 0, 1, (int)unit, true, numberOfBytes);

			var inputFormat = InputFormat.InputFormatFromByteArray(data);
			currentSensor.HandleUpdatedInputFormat(inputFormat);
		}

		void UpdateValueWithData(byte[] data) {
			SdkError error;
			currentSensor.HandleUpdatedValueData(data, out error);
			Assert.Null(error);
		}

		[Test]
		public void TestHandleUpdatedValueSi() {
			UpdateInputFormatMode(InputFormatUnit.InputFormatUnitSi, 4);

			var value = 0.10f;
			UpdateValueWithData(BitConverter.GetBytes(value));

			testTrigger.WaitOne(1000);
			Assert.That(Math.Abs(value - currentSensor.MilliAmp) < float.Epsilon);
			Assert.That(Math.Abs(value - receivedMilliAmps) < float.Epsilon);

		}

		[Test]
		public void TestHandleUpdatedValueRaw() {
			UpdateInputFormatMode(InputFormatUnit.InputFormatUnitRaw, 1);

			var value = 3000.10f;
			UpdateValueWithData(BitConverter.GetBytes(value));

			testTrigger.WaitOne(1000);
			Assert.That(Math.Abs(0 - currentSensor.MilliAmp) < float.Epsilon);
			Assert.That(Math.Abs(0 - receivedMilliAmps) < float.Epsilon);
			Assert.That(Math.Abs(value - currentSensor.ValueAsFloat) < float.Epsilon);

		}

		[Test]
		public void TestHandleUpdatedValuePercent() {
			UpdateInputFormatMode(InputFormatUnit.InputFormatUnitRaw, 1);

			var value = 99;
			UpdateValueWithData(BitConverter.GetBytes(value));

			testTrigger.WaitOne(1000);
			Assert.That(Math.Abs(0 - currentSensor.MilliAmp) < float.Epsilon);
			Assert.That(Math.Abs(0 - receivedMilliAmps) < float.Epsilon);
			Assert.That(Math.Abs(value - currentSensor.ValueAsInteger) < float.Epsilon);

		}


		public void DidUpdateMilliAmp(ICurrentSensor service, float milliAmps) {
			receivedMilliAmps = milliAmps;
			testTrigger.Set();
		}

		public void DidUpdateInputFormat(IService service, InputFormat oldFormat, InputFormat newFormat) {
			
		}

		public void DidUpdateValueData(IService service, byte[] oldData, byte[] newData) {
		}
	}
}
