﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using LegoDeviceSDK.Bluetooth;
using LegoDeviceSDK.Generic;
using LegoDeviceSDK.Generic.Data;
using LegoDeviceSDK.Interfaces;
using LegoSDKNunitTest.SDK.Bluetooth.Mocks;
using NUnit.Framework;

namespace LegoSDKNunitTest.SDK.Bluetooth.Services {
	[TestFixture]
	public class BluetoothMotorTest {

		BluetoothMotor motor;
		BluetoothServiceIoMock ioStub;

		[SetUp]
		public void SetUp() {
			ioStub = new BluetoothServiceIoMock();
			var connectInfo = new ConnectInfo() {ConnectId = 1, HubIndex = 1, TypeEnum = IoType.IoTypeMotor};
			var service = BluetoothService.GetServiceFromConnectInfo(new BluetoothDeviceStub(), connectInfo, ioStub);
			motor = service as BluetoothMotor;
		}

		[Test]
		public void TestRunMotorRight() {
			var motorPower = 100;
			motor.RunInDirection(MotorDirection.MotorDirectionRight, motorPower);
			Assert.That(motorPower == motor.Power);
			Assert.That(motorPower == ioStub.LastWrittenMotorPower);
			Assert.That(MotorDirection.MotorDirectionRight == motor.Direction);
			Assert.False(motor.isBraking);
			Assert.False(motor.isDrifting);

			motorPower = 1;
			motor.RunInDirection(MotorDirection.MotorDirectionRight, motorPower);
			Assert.That(motorPower == motor.Power);
			Assert.That(motorPower == ioStub.LastWrittenMotorPower);
			Assert.That(MotorDirection.MotorDirectionRight == motor.Direction);
			Assert.False(motor.isBraking);
			Assert.False(motor.isDrifting);

			motorPower = 0;
			motor.RunInDirection(MotorDirection.MotorDirectionRight, motorPower);
			Assert.That(motorPower == motor.Power);
			Assert.That(motorPower == ioStub.LastWrittenMotorPower);
			Assert.That(MotorDirection.MotorDirectionDrifting == motor.Direction);
			Assert.False(motor.isBraking);
			Assert.True(motor.isDrifting);
		
		}

		[Test]
		public void TestRunMotorleft() {
			var motorPower = 100;
			motor.RunInDirection(MotorDirection.MotorDirectionLeft, motorPower);
			Assert.That(motorPower == motor.Power);
			Assert.That(-motorPower == ioStub.LastWrittenMotorPower);
			Assert.That(MotorDirection.MotorDirectionLeft == motor.Direction);
			Assert.False(motor.isBraking);
			Assert.False(motor.isDrifting);

			motorPower = 1;
			motor.RunInDirection(MotorDirection.MotorDirectionLeft, motorPower);
			Assert.That(motorPower == motor.Power);
			Assert.That(-motorPower == ioStub.LastWrittenMotorPower);
			Assert.That(MotorDirection.MotorDirectionLeft == motor.Direction);
			Assert.False(motor.isBraking);
			Assert.False(motor.isDrifting);

			motorPower = 0;
			motor.RunInDirection(MotorDirection.MotorDirectionRight, motorPower);
			Assert.That(motorPower == motor.Power);
			Assert.That(motorPower == ioStub.LastWrittenMotorPower);
			Assert.That(MotorDirection.MotorDirectionDrifting == motor.Direction);
			Assert.False(motor.isBraking);
			Assert.True(motor.isDrifting);

		}


		[Test]
		public void TestMotorDrift() {
			motor.Drift();
			Assert.That(0 == motor.Power);
			Assert.That(0 == ioStub.LastWrittenMotorPower);
			Assert.That(MotorDirection.MotorDirectionDrifting == motor.Direction);
			Assert.False(motor.isBraking);
			Assert.True(motor.isDrifting);
		}

		[Test]
		public void TestMotorBrake() {
			motor.Brake();
			Assert.That(0 == motor.Power);
			Assert.That(127 == ioStub.LastWrittenMotorPower);
			Assert.That(MotorDirection.MotorDirectionBraking == motor.Direction);
			Assert.True(motor.isBraking);
			Assert.False(motor.isDrifting);
		}



	}
}
