﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using LegoDeviceSDK.Bluetooth;
using LegoDeviceSDK.Generic;
using LegoDeviceSDK.Generic.Data;
using LegoDeviceSDK.Interfaces;
using LegoSDKNunitTest.SDK.Bluetooth.Mocks;
using NUnit.Framework;

namespace LegoSDKNunitTest.SDK.Bluetooth.Services {
	[TestFixture]
	public class BluetoothPiezoTonePlayerTest {
		AutoResetEvent testTrigger;

		BluetoothPiezoTonePlayer piezoTonePlayer;
		readonly ConnectInfo connectInfo = new ConnectInfo() { ConnectId = 1, HubIndex = 1, TypeEnum = IoType.IoTypePiezoTone };
		BluetoothServiceIoMock ioStub = new BluetoothServiceIoMock();

		[SetUp]
		public void Setup() {
			testTrigger = new AutoResetEvent(false);
			ioStub = new BluetoothServiceIoMock();
			var service = BluetoothService.GetServiceFromConnectInfo(new BluetoothDeviceStub(), connectInfo, ioStub);
			piezoTonePlayer = service as BluetoothPiezoTonePlayer;
		}

		[TearDown]
		public void TearDown() {
			piezoTonePlayer = null;
		}

		[Test]
		public void TestPlayPiezoTone() {
			var frequency = 440;
			var duration = 1000;

			piezoTonePlayer.PlayFrequency(frequency, duration);

			Assert.That(frequency == ioStub.LastFrequencyWritten);
			Assert.That(duration == ioStub.LastDurationWritten);
		}

		[Test]
		public void TestPlayPiezoNote() {
			VerifyNote(PiezoTonePlayerNote.PiezoNoteA, 4, 440);
			VerifyNote(PiezoTonePlayerNote.PiezoNoteB, 4, 494);
			VerifyNote(PiezoTonePlayerNote.PiezoNoteA, 5, 880);

		}

		[Test]
		public void TestPlayPiezoNoteHighestSupportedNote() {
			VerifyNote(PiezoTonePlayerNote.PiezoNoteFis, 6, 1480);
		}

		[Test]
		public void TestPlayPiezoNoteHigherThanSupportedRoundedDown() {
			VerifyNote(PiezoTonePlayerNote.PiezoNoteG, 6, 1500);
		}

		[Test]
		public void TestPlayPiezoNoteHigherOctaveThanSupportedRoundedDown() {
			VerifyNote(PiezoTonePlayerNote.PiezoNoteG, 7, 1500);
		}

		[Test]
		public void TestPlayPiezoNoteLowestSupportedNode() {
			VerifyNote(PiezoTonePlayerNote.PiezoNoteC, 0, 16);
		}

		private void VerifyNote(PiezoTonePlayerNote note, int octave, int translatingFrequency) {
      int duration = 0;

      // Execute MUT
      piezoTonePlayer.PlayNote(note, octave, duration);

      // Verify
      Assert.That(translatingFrequency == ioStub.LastFrequencyWritten);
			Assert.That(duration == ioStub.LastDurationWritten);
    }

	}
}
