﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using LegoDeviceSDK.Bluetooth;
using LegoDeviceSDK.Generic;
using LegoDeviceSDK.Interfaces;
using LegoSDKNunitTest.SDK.Bluetooth.Mocks;
using NUnit.Framework;
using LegoDeviceSDK.Interfaces.Services;

namespace LegoSDKNunitTest.SDK.Bluetooth.Services {
	[TestFixture]
	public class BluetoothVoltageSensorTest : IVoltageSensorDelegate {
		AutoResetEvent testTrigger;
		float receivedMilliVolts;

		BluetoothVoltageSensor voltageSensor;
		ConnectInfo connectInfo = new ConnectInfo() { ConnectId = 1, HubIndex = 1, TypeEnum = IoType.IoTypeVoltage };
		IIo ioStub = new BluetoothServiceIoMock();

		[SetUp]
		public void Setup() {
			testTrigger = new AutoResetEvent(false);
			ioStub = new BluetoothServiceIoMock();
			var service = BluetoothService.GetServiceFromConnectInfo(new BluetoothDeviceStub(), connectInfo, ioStub);
			voltageSensor = service as BluetoothVoltageSensor;
			if (voltageSensor == null)
				Assert.Fail();
			voltageSensor.AddDelegate(this);
		}

		[TearDown]
		public void TearDown() {
			voltageSensor.RemoveDelegate(this);
			voltageSensor = null;
		}

		void UpdateInputFormatMode(InputFormatUnit unit, int numberOfBytes) {
			var data = BluetoothTestStupFactory.InputFormatWriteDataWithRevision(1, voltageSensor.ConnectInfo.ConnectId, voltageSensor.ConnectInfo.TypeEnum, 0, 1, (int)unit, true, numberOfBytes);

			var inputFormat = InputFormat.InputFormatFromByteArray(data);
			voltageSensor.HandleUpdatedInputFormat(inputFormat);
		}

		void UpdateValueWithData(byte[] data) {
			SdkError error;
			voltageSensor.HandleUpdatedValueData(data, out error);
			Assert.Null(error);
		}

		[Test]
		public void TestHandleUpdatedValueSi() {
			UpdateInputFormatMode(InputFormatUnit.InputFormatUnitSi, 4);

			var value = 0.10f;
			UpdateValueWithData(BitConverter.GetBytes(value));

			testTrigger.WaitOne(1000);
			Assert.That(Math.Abs(value - voltageSensor.MilliVolts) < float.Epsilon);
			Assert.That(Math.Abs(value - receivedMilliVolts) < float.Epsilon);

		}

		[Test]
		public void TestHandleUpdatedValueRaw() {
			UpdateInputFormatMode(InputFormatUnit.InputFormatUnitRaw, 1);

			var value = 3000.10f;
			UpdateValueWithData(BitConverter.GetBytes(value));

			testTrigger.WaitOne(1000);
			Assert.That(Math.Abs(0 - voltageSensor.MilliVolts) < float.Epsilon);
			Assert.That(Math.Abs(0 - receivedMilliVolts) < float.Epsilon);
			Assert.That(Math.Abs(value - voltageSensor.ValueAsFloat) < float.Epsilon);

		}

		[Test]
		public void TestHandleUpdatedValuePercent() {
			UpdateInputFormatMode(InputFormatUnit.InputFormatUnitRaw, 1);

			var value = 99;
			UpdateValueWithData(BitConverter.GetBytes(value));

			testTrigger.WaitOne(1000);
			Assert.That(Math.Abs(0 - voltageSensor.MilliVolts) < float.Epsilon);
			Assert.That(Math.Abs(0 - receivedMilliVolts) < float.Epsilon);
			Assert.That(Math.Abs(value - voltageSensor.ValueAsInteger) < float.Epsilon);

		}

		public void DidUpdateInputFormat(IService service, InputFormat oldFormat, InputFormat newFormat) {
		}

		public void DidUpdateValueData(IService service, byte[] oldData, byte[] newData) {
		}

		public void DidUpdateMilliVolts(IVoltageSensor newVoltageSensor, float milliVolts) {
			receivedMilliVolts = milliVolts;
			testTrigger.Set();
		}
	}
}
