﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using LegoDeviceSDK.Generic;
using NUnit.Framework;

namespace LegoSDKNunitTest.SDK.Generic {
	[TestFixture]
	public class ConnectInfoTest {

		[Test]
		public void TestConnectInfoFromArrayConnectedCorrect() {
			var data = new byte[12] { 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0 };
			var connectInfo = ConnectInfo.FromByteArray(data);
			Assert.AreEqual(IoType.IoTypeMotor, connectInfo.TypeEnum);

			data[3] = 20;
			connectInfo = ConnectInfo.FromByteArray(data);
			Assert.AreEqual(IoType.IoTypeVoltage, connectInfo.TypeEnum);

			data[3] = 21;
			connectInfo = ConnectInfo.FromByteArray(data);
			Assert.AreEqual(IoType.IoTypeCurrent, connectInfo.TypeEnum);

			data[3] = 22;
			connectInfo = ConnectInfo.FromByteArray(data);
			Assert.AreEqual(IoType.IoTypePiezoTone, connectInfo.TypeEnum);

			data[3] = 23;
			connectInfo = ConnectInfo.FromByteArray(data);
			Assert.AreEqual(IoType.IoTypeRgbLight, connectInfo.TypeEnum);

			data[3] = 34;
			connectInfo = ConnectInfo.FromByteArray(data);
			Assert.AreEqual(IoType.IoTypeTiltSensor, connectInfo.TypeEnum);

			data[3] = 35;
			connectInfo = ConnectInfo.FromByteArray(data);
			Assert.AreEqual(IoType.IoTypeMotionSensor, connectInfo.TypeEnum);
		}

		[Test]
		public void TestConnectInfoFromArrayConnectedCorrectGeneric() {
			var data = new byte[12] { 0, 1, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0 };
			var connectInfo = ConnectInfo.FromByteArray(data);
			Assert.AreEqual(IoType.IoTypeGeneric, connectInfo.TypeEnum);

			data[3] = 19;
			connectInfo = ConnectInfo.FromByteArray(data);
			Assert.AreEqual(IoType.IoTypeGeneric, connectInfo.TypeEnum);

			data[3] = 33;
			connectInfo = ConnectInfo.FromByteArray(data);
			Assert.AreEqual(IoType.IoTypeGeneric, connectInfo.TypeEnum);

			data[3] = 36;
			connectInfo = ConnectInfo.FromByteArray(data);
			Assert.AreEqual(IoType.IoTypeGeneric, connectInfo.TypeEnum);
		}

		[Test]
		public void TestConnectInfoFromArrayWrongLength() {
			var data = new byte[0];
			var connectInfo = ConnectInfo.FromByteArray(data);
			Assert.Null(connectInfo);

			data = new byte[1] { 0 };
			connectInfo = ConnectInfo.FromByteArray(data);
			Assert.Null(connectInfo);

			data = new byte[11] { 0, 1, 0, 2, 0, 0, 0, 0, 0, 0, 0 };
			connectInfo = ConnectInfo.FromByteArray(data);
			Assert.Null(connectInfo);

			data = new byte[13] { 0, 1, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
			connectInfo = ConnectInfo.FromByteArray(data);
			Assert.Null(connectInfo);


		}

	}
}
