﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using LegoDeviceSDK.Generic;
using NUnit.Framework;

namespace LegoSDKNunitTest.SDK.Generic {
	[TestFixture]
	public class InputFormatTest {

		[Test]
		public void FromByteArrayTest() {
			var formatVersion = 2;

			var deltaIntervalBytes = BitConverter.GetBytes(12345);
			var formatWriteData = new byte[] {
				(byte)formatVersion,
				1,
				(byte)IoType.IoTypeMotionSensor,
				0,
				deltaIntervalBytes[0],
				deltaIntervalBytes[1],
				deltaIntervalBytes[2],
				deltaIntervalBytes[3],
				1,
				1,
				2
			};

			var format1 = InputFormat.InputFormatFromByteArray(formatWriteData);
			Assert.NotNull(format1);
			Assert.That(format1.Revision == 2);
			Assert.That(format1.ConnectId == 1);
			Assert.That(format1.TypeId == (int)IoType.IoTypeMotionSensor);
			Assert.That(format1.Mode == 0);
			Assert.That(format1.DeltaInterval == 12345);
			Assert.That(format1.Unit == InputFormatUnit.InputFormatUnitPercentage);
			Assert.True(format1.NotificationsEnabled);
			Assert.That(format1.NumberOfBytes == 2);

		}

		[Test]
		public void CreateInputFormatWriteDataTest() {
			var deltaIntervalBytes = BitConverter.GetBytes(12345);
			var expectedResult = new byte[] {
				(byte)IoType.IoTypeMotionSensor,
				0,
				deltaIntervalBytes[0],
				deltaIntervalBytes[1],
				deltaIntervalBytes[2],
				deltaIntervalBytes[3],
				1,
				1,
			};

			var format1 = InputFormat.InputFormatWithConnectId(1, (int)IoType.IoTypeMotionSensor, 0, 12345, InputFormatUnit.InputFormatUnitPercentage, true);
			var actualResult = InputFormat.InputFormatToByteArray(format1);
			Assert.AreEqual(expectedResult, actualResult);
		}

		[Test]
		public void CompleteSerializeAndDeserializeTest() {
			var expectedInputFormat = InputFormat.InputFormatWithConnectId(1, (int)IoType.IoTypeMotionSensor, 0, 12345, InputFormatUnit.InputFormatUnitPercentage, true);
			var byteArray = InputFormat.InputFormatToByteArray(expectedInputFormat);
			var deltaIntervalBytes = BitConverter.GetBytes(12345);
			var formatWriteData = new byte[] {
				0,
				1,
				(byte)IoType.IoTypeMotionSensor,
				0,
				deltaIntervalBytes[0],
				deltaIntervalBytes[1],
				deltaIntervalBytes[2],
				deltaIntervalBytes[3],
				1,
				1,
				0
			};

			var actualInputFormat = InputFormat.InputFormatFromByteArray(formatWriteData);

			Assert.IsTrue(expectedInputFormat.Equals(actualInputFormat));
			Assert.True(expectedInputFormat.Equals(actualInputFormat));
			Assert.AreEqual(expectedInputFormat.ToString(), actualInputFormat.ToString());
			Assert.AreEqual(expectedInputFormat.ShortDescription, actualInputFormat.ShortDescription);

		}


		[Test]
		public void ChangeConstructorTest() {
			var newDeltainterval = 54321;
			var newMode1 = 1;
			var newMode2 = 2;
			var newUnit = InputFormatUnit.InputFormatUnitSi;
			var newNotifcationsEnabled = false;
			var expectedInputFormat = InputFormat.InputFormatWithConnectId(1, (int)IoType.IoTypeMotionSensor, 0, 12345, InputFormatUnit.InputFormatUnitPercentage, true);

			var actualInputFormat = expectedInputFormat.InputFormatBySettingDeltaInterval(54321);
			Assert.AreEqual(actualInputFormat.DeltaInterval, newDeltainterval);

			actualInputFormat = expectedInputFormat.InputFormatBySettingMode(newMode1);
			Assert.AreEqual(actualInputFormat.Mode, newMode1);

			actualInputFormat = expectedInputFormat.InputFormatBySettingMode(newMode2, newUnit);
			Assert.AreEqual(actualInputFormat.Mode, newMode2);
			Assert.AreEqual(actualInputFormat.Unit, newUnit);

			actualInputFormat = expectedInputFormat.InputFormatBySettingNotificationsEnabled(newNotifcationsEnabled);
			Assert.AreEqual(actualInputFormat.NotificationsEnabled, newNotifcationsEnabled);

		}
	}
}
