The LEGO Device SDK is used communicate with various LEGO manufactured hardware such as sensors and motors.

Currently the library supports only communication with hardware connected through Bluetooth LE (BLE). The SDK will help
you scan for and connect to LEGO BLE devices. Once connected to a device the SDK will provide access to read and be notified about changes to sensor values,
and to send commands to various outputs (motors, piezo-tone, etc.)


Scanning for and Connecting to Advertising Devices
--------------------------------------------------

To scan for nearby advertising devices you use the LegoDeviceManagerImpl as illustrated in the example below.

	void startScanningForDevices
	{
	    LegoDeviceManagerImpl.getInstance().registerCallbackListener(this);

	    try {
            LegoDeviceManagerImpl.getInstance().scan(getActivity());
        } catch (IllegalStateException e) {
            // Bluetooth not enabled
        }
	}

    @Override
    public void onDeviceAppeared(final LegoDevice device) {
        LegoBluetoothDevice legoBluetoothDevice = (LegoBluetoothDevice) device;
        if (legoBluetoothDevice != null && legoBluetoothDevice.getAndroidBluetoothDevice() != null) {
            Log.i(TAG, "Device: " + device.getAndroidBluetoothDevice().getName() + " (@" + device.getAndroidBluetoothDevice().getAddress() + ") appeared");
        }
    }

The DeviceManagerCallbackListener will inform you whenever a new device starts or stop advertising. After a device has been discovered you can connect to it.
Again the DeviceManagerCallbackListener will notify you once the connection is established (or if the connection fails).
Before connecting to a device you should add a DeviceCallbackListener to the device.
Through this listener you will be notified on any services (motors, sensors, etc.) offered by the device.
Remember to unregister the listener again when no longer used!

	void connectToDevice(LegoDevice device)
	{
	    device.registerCallbackListener(this);
	    LegoDeviceManagerImpl.getInstance().connectToDevice(getActivity(), device);
	}

    @Override
    public void onDidFinishInterrogatingDevice(LegoDevice device) {
        Log.i(TAG, "Did successfully connect to device: " + device);
    }

    @Override
    public void didAddService(LegoDevice device, LegoService service) {
        Log.i(TAG, "Device did add service: " + service.serviceName);
    }

Read Values From a Known Input connected to the Device
-----------------------------------------------

Once connected to a LegoDevice, and having discovered one or more services you are ready to start read info from sensors and send commands to motors and other outputs.
For example, to start listening to updates from a tilt sensor:

	void startUsingService(LegoService service)
	{
        if (service instanceof TiltSensor) {
            service.registerCallbackListener(this);
        }
	}

    @Override
    public void didUpdateAngle(TiltSensor sensor, TiltSensor.TiltSensorAngle oldAngle, TiltSensor.TiltSensorAngle newAngle) {
        Log.i(TAG, String.format("Tilt sensor changed angle to %s, %s", newAngle.x, newAngle.y));
    }


Communicate with a Generic Input connected to the Device
---------------------------------------------------------

If a service is an instance of the class GenericService this means that the IO type in unkown to the SDK.
As opposed to the known service types (tilt, motion, etc) a generic service does not have a predefined [LegoService defaultInputFormat]
and therefore an InputFormat is not automatically send to the Device. This means that per default no data is send from the
device to the SDK when the IO reads a new data value. To use a GenericService to receive data updated from a sensor unkown to the SDK you
must create and send an InputFormat to the device. Find inspiration in one of the concrete service classes for this, like the MotionSensor
or VoltageSensor.

Also see the documentation of the GenericService class for an example on how to configure and use an IO unkown to the SDK.


Getting Debug and Error log from the LEGO Device SDK
----------------------------------------------------

You should insert the following code snippet in e.g. the onCreate() method of yout Application extension class:

    @Override
    public void onCreate() {

        ...

        if (BuildConfig.DEBUG) {
            LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_DEBUG);
        } else {
            LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_WARN);
        }

        ...
    }

It is also possible to redirect the LEDevice log statement to another destination, for example an online logging
service or a (debug) text view in your app. You can do this by creating a class that implements the CustomLogger interface.
