package bluetooth.helper;

import android.graphics.Color;

import dk.lego.devicesdk.bluetooth.IoCallbackHelper;
import dk.lego.devicesdk.input_output.IO;
import dk.lego.devicesdk.input_output.InputFormat;

public class IOStub extends IO {
    public Integer lastConnectIdReceived = null;
    public InputFormat lastInputFormatReceived = null;
    public Integer lastWrittenMotorPower = null;
    public Integer lastPiezoFrequencyWritten = null;
    public Integer lastPiezoDurationWritten = null;
    public Boolean isPiezoStopRequested = null;
    public Integer lastColorWritten = null;
    public int lastColorIndexWritten = -1;

    @Override
    public void writeInputFormat(InputFormat inputFormat, int connectId)
    {
        lastInputFormatReceived = inputFormat;
        lastConnectIdReceived = connectId;

        // Assume that the update went well, and return the 'updated' inputFormat right away
        callbackHelper.performDidReceiveInputFormatCallback(this, inputFormat);
    }

    @Override
    public void readInputFormatForConnectId(int connectId)
    {
        //do nothing
    }

    @Override
    public void writeMotorPower(int power, int connectId)
    {
        writeMotorPower(power, 0, connectId);
    }

    @Override
    public void writeMotorPower(int power, int offset, int connectId)
    {
        lastWrittenMotorPower = power;
    }

    @Override
    public void writePiezoToneFrequency(int frequency, int duration, int connectId)
    {
        lastPiezoFrequencyWritten = frequency;
        lastPiezoDurationWritten = duration;
    }

    @Override
    public void writePiezoToneStop(int connectId)
    {
        isPiezoStopRequested = true;
    }

    @Override
    public void writeColor(int red, int green, int blue, int connectId)
    {
        lastColorWritten = Color.rgb(red, green, blue);
    }

    @Override
    public void writeColorIndex(int index, int connectId)
    {
        lastColorIndexWritten = index;
    }

    @Override
    public void writeData(byte[] data, int connectId)
    {
        //do nothing
    }

    @Override
    public void readValueForConnectId(int connectId)
    {
        //do nothing
    }

    @Override
    public void resetStateForConnectId(int connectId)
    {
        //do nothing
    }
}
