package bluetooth.input_and_output;

import junit.framework.TestCase;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;

import dk.lego.devicesdk.device.ConnectInfo;
import dk.lego.devicesdk.input_output.InputCommand;
import dk.lego.devicesdk.input_output.InputFormat;
import dk.lego.devicesdk.utils.ByteUtils;

public class InputCommandTest extends TestCase {

    static final byte COMMAND_ID_INPUT_VALUE = 0;
    static final byte COMMAND_ID_INPUT_FORMAT = 1;

    static final byte COMMAND_TYPE_READ = 1;
    static final byte COMMAND_TYPE_WRITE = 2;

    public void testWriteInputFormat()
    {
        //Setup the format to write
        ConnectInfo connectInfo = new ConnectInfo(1, 2, ConnectInfo.IOType.IO_TYPE_GENERIC);
        InputFormat format = InputFormat.inputFormat(connectInfo.getConnectId(), connectInfo.getType(), 9, 10, InputFormat.InputFormatUnit.INPUT_FORMAT_UNIT_RAW, true);

        // Set up the expected byte-stream in the write command
        int typeId = connectInfo.getType().getValue();
        int mode = format.getMode();
        long deltaInterval = format.getDeltaInterval();
        InputFormat.InputFormatUnit unit = InputFormat.InputFormatUnit.INPUT_FORMAT_UNIT_RAW;
        int notificationEnabled = 1;

        ByteBuffer expectedDataWrittenBuffer = ByteBuffer.allocate(11).order(ByteOrder.LITTLE_ENDIAN);

        appendCommandHeaderToData(expectedDataWrittenBuffer, COMMAND_ID_INPUT_FORMAT, COMMAND_TYPE_WRITE, connectInfo.getConnectId());

        ByteUtils.putUnsignedByte(expectedDataWrittenBuffer, typeId);
        ByteUtils.putUnsignedByte(expectedDataWrittenBuffer, mode);
        ByteUtils.putUnsignedInt(expectedDataWrittenBuffer, deltaInterval);
        ByteUtils.putUnsignedByte(expectedDataWrittenBuffer, unit.getValue());
        ByteUtils.putUnsignedByte(expectedDataWrittenBuffer, notificationEnabled);

        //Execute the method under test (MUT)
        InputCommand command = InputCommand.commandWriteInputFormat(format, connectInfo.getConnectId());

        //Verify the command data byte stream
        assertTrue(Arrays.equals(expectedDataWrittenBuffer.array(), command.getData()));
    }


    public void testReadInputValues()
    {
        //Setup expected result
        byte[] expectedDataWritten = new byte[3];

        appendCommandHeaderToData(expectedDataWritten, COMMAND_ID_INPUT_VALUE, COMMAND_TYPE_READ, 2);

        //Execute the MUT
        InputCommand command = InputCommand.commandReadValueForConnectId(2);

        //Verify
        assertTrue(Arrays.equals(expectedDataWritten, command.getData()));
    }


    private void appendCommandHeaderToData(ByteBuffer buffer, int commandId, int commandType, int connectId)
    {
        ByteUtils.putUnsignedByte(buffer, commandId);
        ByteUtils.putUnsignedByte(buffer, commandType);
        ByteUtils.putUnsignedByte(buffer, connectId);
    }

    private void appendCommandHeaderToData(byte[] data, int commandId, int commandType, int connectId)
    {
        data[0] = (byte) commandId;
        data[1] = (byte) commandType;
        data[2] = (byte) connectId;
    }
}
