package bluetooth.input_and_output;

import junit.framework.TestCase;

import java.util.Arrays;

import bluetooth.mocks_and_stubs.TestStubFactory;
import dk.lego.devicesdk.device.ConnectInfo;
import dk.lego.devicesdk.input_output.InputFormat;
import dk.lego.devicesdk.utils.ByteUtils;

public class InputFormatTest extends TestCase {
    public void testCreateInputFormatFromData() throws Exception {

        byte formatVersion = 2;

        byte[] formatWriteData = TestStubFactory.inputFormatWriteData(formatVersion, 1,
                ConnectInfo.IOType.IO_TYPE_MOTION_SENSOR,
                0,
                12345,
                InputFormat.InputFormatUnit.INPUT_FORMAT_UNIT_PERCENTAGE,
                true,
                2);

        InputFormat format1 = InputFormat.inputFormat(formatWriteData);
        assertNotNull(format1);
        assertEquals(format1.getRevision(), 2);
        assertEquals(format1.getConnectId(), 1);
        assertEquals(format1.getType(), ConnectInfo.IOType.IO_TYPE_MOTION_SENSOR);
        assertEquals(format1.getMode(), 0);
        assertEquals(format1.getDeltaInterval(), 12345);
        assertEquals(format1.getUnit(), InputFormat.InputFormatUnit.INPUT_FORMAT_UNIT_PERCENTAGE);
        assertTrue(format1.isNotificationsEnabled());
        assertEquals(format1.getNumberOfBytes(), 2);
    }


    public void testCreateInputFormatWriteData() throws Exception {
        // Setup expected result
        String expectedResultHex = "22" //Type 34 in hex
                + "00" //Mode
                + "01000000" //DeltaInterval (four bytes)
                + "01" //Unit Percentage
                + "01"; //Notifications enabled
        byte[] expectedResult = ByteUtils.fromHexString(expectedResultHex);

        // Run method under test
        InputFormat format = InputFormat.inputFormat(
                2,
                ConnectInfo.IOType.IO_TYPE_TILT_SENSOR, // 34
                0,
                1,
                InputFormat.InputFormatUnit.INPUT_FORMAT_UNIT_PERCENTAGE,
                true);

        byte[] actualResult = format.writeFormatData();

        // Verify Results
        assertTrue(Arrays.equals(actualResult, expectedResult));
    }
}
