package bluetooth.input_and_output;

import junit.framework.TestCase;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import dk.lego.devicesdk.input_output.OutputCommand;
import dk.lego.devicesdk.utils.ByteUtils;

public class OutputCommandTest extends TestCase {
    OutputCommand command;
    int connectId;

    @Override
    public void setUp() throws Exception {
        super.setUp();
        connectId = 1;
    }

    public void testWriteMotorPowerCommand()
    {
        byte writtenPower = 100;
        command = OutputCommand.commandWriteMotorPower(writtenPower, connectId);

        byte[] data = command.getData();
        assertEquals("Unexpected length of set speed command", 4, data.length);

        assertEquals(connectId, data[0]);

        byte expectedCommandID = 1; //Write motor power commandID
        assertEquals(expectedCommandID, data[1]);

        byte expectedNumberOfBytes = 1;
        assertEquals(expectedNumberOfBytes, data[2]);

        assertEquals(writtenPower, data[3]);
    }


    public void testWriteMotorPowerCommand_NegativeSpeed()
    {
        byte writtenPower = -100;
        command = OutputCommand.commandWriteMotorPower(writtenPower, connectId);

        assertEquals(writtenPower, command.getData()[3]);
    }

    public void testWritePiezoToneFrequency()
    {
        int frequency = 440;
        int duration = 1000;

        command = OutputCommand.commandWritePiezoToneFrequency(frequency, duration, connectId);

        ByteBuffer data = ByteBuffer.wrap(command.getData()).order(ByteOrder.LITTLE_ENDIAN);
        assertEquals("Unexpected length of write piezo tone command", 7, data.capacity());


        assertEquals(connectId, ByteUtils.getUnsignedByte(data, 0));

        byte expectedCommandId = 2; //Play Piezo Tone commandID
        assertEquals(expectedCommandId, ByteUtils.getUnsignedByte(data, 1));

        byte expectedNumberOfBytes = 4; //Two bytes for frequency and two for duration in ms
        assertEquals(expectedNumberOfBytes, ByteUtils.getUnsignedByte(data, 2));

        assertEquals(frequency, ByteUtils.getUnsignedShort(data, 3));

//        int writtenDuration = ByteUtils.from2BytesToInt(Arrays.copyOfRange(data, 5, 7));
        assertEquals(duration, ByteUtils.getUnsignedShort(data, 5));
    }


    public void testWritePiezoToneStop()
    {
        command = OutputCommand.commandWritePiezoToneStopForConnectId(connectId);

        byte[] data = command.getData();
        assertEquals("Unexpected length stop piezo tone command", 3, data.length);

        assertEquals(connectId, data[0]);

        int expectedCommandID = 3; //Stop Piezo Tone commandID
        assertEquals(expectedCommandID, data[1]);

        int expectedNumberOfBytes = 0; //No payload
        assertEquals(expectedNumberOfBytes, data[2]);
    }

    public void testWriteRGB()
    {
        int red = 0;
        int green = 255;
        int blue = 128;

        command = OutputCommand.commandWriteRGBLight(red, green, blue, connectId);

        ByteBuffer data = ByteBuffer.wrap(command.getData()).order(ByteOrder.LITTLE_ENDIAN);

        assertEquals("Unexpected length of write RGB command", 6, data.capacity());

        assertEquals(connectId, ByteUtils.getUnsignedByte(data, 0));

        int expectedCommandID = 4; //Write RGB commandID
        assertEquals(expectedCommandID, ByteUtils.getUnsignedByte(data, 1));

        int expectedNumberOfBytes = 3; //One byte for each color component
        assertEquals(expectedNumberOfBytes, ByteUtils.getUnsignedByte(data, 2));

        assertEquals(red, ByteUtils.getUnsignedByte(data, 3));
        assertEquals(green, ByteUtils.getUnsignedByte(data, 4));
        assertEquals(blue, ByteUtils.getUnsignedByte(data, 5));
    }
}