package bluetooth.mocks_and_stubs;

import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattService;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import dk.lego.devicesdk.bluetooth.service_definitions.BluetoothServiceDefinition;
import dk.lego.devicesdk.bluetooth.service_definitions.CharacteristicDefinition;
import dk.lego.devicesdk.bluetooth.wrappers.AndroidBluetoothDeviceWrapper;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class CoreBluetoothMockFactory {

    public static AndroidBluetoothDeviceWrapper deviceWithServices() {

        return new AndroidBluetoothDeviceWrapperMock("Test", "1");
    }


    public static BluetoothGattService deviceService() {
        BluetoothGattCharacteristic nameCharacteristic = CoreBluetoothMockFactory.characteristicWithData(null, BluetoothServiceDefinition.deviceServiceDefinition().getDeviceName());
        BluetoothGattCharacteristic ioCharacteristic = CoreBluetoothMockFactory.characteristicWithData(null, BluetoothServiceDefinition.deviceServiceDefinition().getAttachedIO());
        BluetoothGattCharacteristic buttonStateCharacteristic = CoreBluetoothMockFactory.characteristicWithData(null, BluetoothServiceDefinition.deviceServiceDefinition().getDeviceButton());

        BluetoothGattService mockService = mock(BluetoothGattService.class);
        List<BluetoothGattCharacteristic> characteristics = new ArrayList<>();
        characteristics.add(nameCharacteristic);
        characteristics.add(ioCharacteristic);
        characteristics.add(buttonStateCharacteristic);

        when(mockService.getCharacteristics()).thenReturn(characteristics);
        when(mockService.getUuid()).thenReturn(BluetoothServiceDefinition.deviceServiceDefinition().getServiceUUID());

        return mockService;
    }

    public static BluetoothGattService deviceInfoService() {
        BluetoothGattCharacteristic firmwareRevision = CoreBluetoothMockFactory.characteristicWithData(null, BluetoothServiceDefinition.deviceInfoServiceDefinition().getFirmwareRevision());
        BluetoothGattCharacteristic hardwareRevision = CoreBluetoothMockFactory.characteristicWithData(null, BluetoothServiceDefinition.deviceInfoServiceDefinition().getHardwareRevision());
        BluetoothGattCharacteristic softwareRevision = CoreBluetoothMockFactory.characteristicWithData(null, BluetoothServiceDefinition.deviceInfoServiceDefinition().getSoftwareRevision());
        BluetoothGattCharacteristic manufacturerName = CoreBluetoothMockFactory.characteristicWithData(null, BluetoothServiceDefinition.deviceInfoServiceDefinition().getManufacturerName());

        BluetoothGattService mockService = mock(BluetoothGattService.class);
        List<BluetoothGattCharacteristic> characteristics = new ArrayList<>();
        characteristics.add(firmwareRevision);
        characteristics.add(hardwareRevision);
        characteristics.add(softwareRevision);
        characteristics.add(manufacturerName);

        when(mockService.getCharacteristics()).thenReturn(characteristics);
        when(mockService.getUuid()).thenReturn(BluetoothServiceDefinition.deviceInfoServiceDefinition().getServiceUUID());

        return mockService;
    }


    public static BluetoothGattService inputService() {
        return inputServiceWithPeripheral();
    }

    public static BluetoothGattService inputServiceWithPeripheral() {
        BluetoothGattCharacteristic valueCharacteristic = CoreBluetoothMockFactory.inputValueCharacteristicWithData(null);
        BluetoothGattCharacteristic inputFormatCharacteristic = CoreBluetoothMockFactory.inputFormatCharacteristicWithData(null);
        BluetoothGattCharacteristic inputCommandCharacteristic = CoreBluetoothMockFactory.inputCommandCharacteristicWithData(null);
        BluetoothGattCharacteristic outputCommandCharacteristic = CoreBluetoothMockFactory.outputCommandCharacteristicWithData(null);

        BluetoothGattService mockService = mock(BluetoothGattService.class);
        List<BluetoothGattCharacteristic> characteristics = new ArrayList<>(4);
        characteristics.add(valueCharacteristic);
        characteristics.add(inputFormatCharacteristic);
        characteristics.add(inputCommandCharacteristic);
        characteristics.add(outputCommandCharacteristic);

        when(mockService.getCharacteristics()).thenReturn(characteristics);
        when(mockService.getUuid()).thenReturn(BluetoothServiceDefinition.ioServiceDefinition().getServiceUUID());

        return mockService;
    }

    public static BluetoothGattCharacteristic inputValueCharacteristicWithData(byte[] valueData) {
        return characteristicWithData(valueData, BluetoothServiceDefinition.ioServiceDefinition().getInputValue());
    }

    public static BluetoothGattCharacteristic inputFormatCharacteristicWithData(byte[] valueData) {
        return characteristicWithData(valueData, BluetoothServiceDefinition.ioServiceDefinition().getInputFormat());
    }

    public static BluetoothGattCharacteristic inputCommandCharacteristicWithData(byte[] valueData) {
        return characteristicWithData(valueData, BluetoothServiceDefinition.ioServiceDefinition().getInputCommand());
    }

    public static BluetoothGattCharacteristic outputCommandCharacteristicWithData(byte[] valueData) {
        return characteristicWithData(valueData, BluetoothServiceDefinition.ioServiceDefinition().getOutputCommand());
    }

    public static BluetoothGattCharacteristic deviceButtonCharacteristic(byte[] data) {
        return characteristicWithDataAndService(data, BluetoothServiceDefinition.deviceServiceDefinition().getDeviceButton());
    }

    public static BluetoothGattCharacteristic deviceLowVoltageAlertCharacteristic(byte[] data) {
        return characteristicWithDataAndService(data, BluetoothServiceDefinition.deviceServiceDefinition().getLowVoltageAlert());
    }

    private static BluetoothGattCharacteristic characteristicWithData(byte[] data, CharacteristicDefinition characteristicDefinition) {

        return characteristicWithUUID(characteristicDefinition.getUuid(),
                data,
                characteristicDefinition.getMandatoryProperties() | characteristicDefinition.getRecommendedProperties());
    }

    private static BluetoothGattCharacteristic characteristicWithDataAndService(byte[] data, CharacteristicDefinition characteristicDefinition) {

        BluetoothGattService mockedService = null;
        if (characteristicDefinition.getServiceDefinition().getServiceUUID().equals(BluetoothServiceDefinition.deviceInfoServiceDefinition().getServiceUUID())) {
            mockedService = deviceInfoService();
        } else if (characteristicDefinition.getServiceDefinition().getServiceUUID().equals(BluetoothServiceDefinition.ioServiceDefinition().getServiceUUID())) {
            mockedService = inputService();
        } else if (characteristicDefinition.getServiceDefinition().getServiceUUID().equals(BluetoothServiceDefinition.deviceServiceDefinition().getServiceUUID())) {
            mockedService = deviceService();
        }

        return characteristicWithUUIDAndService(characteristicDefinition.getUuid(),
                data,
                mockedService,
                characteristicDefinition.getMandatoryProperties() | characteristicDefinition.getRecommendedProperties());
    }

    private static BluetoothGattCharacteristic characteristicWithUUID(UUID characteristicUUID, byte[] valueData) {
        return characteristicWithUUID(characteristicUUID, valueData, BluetoothGattCharacteristic.PERMISSION_READ);
    }

    private static BluetoothGattCharacteristic characteristicWithUUID(UUID characteristicUUID, byte[] valueData, int properties) {
        BluetoothGattCharacteristic mockCharacteristic = mock(BluetoothGattCharacteristic.class);

        when(mockCharacteristic.getUuid()).thenReturn(characteristicUUID);
        when(mockCharacteristic.getValue()).thenReturn(valueData);
        if (valueData != null && valueData.length >= 1) {
            when(mockCharacteristic.getIntValue(BluetoothGattCharacteristic.FORMAT_UINT8, 0)).thenReturn((int) valueData[0]);
        }
        when(mockCharacteristic.getProperties()).thenReturn(properties);


        return mockCharacteristic;
    }

    private static BluetoothGattCharacteristic characteristicWithUUIDAndService(UUID characteristicUUID, byte[] valueData, BluetoothGattService mockService, int properties) {
        BluetoothGattCharacteristic mockCharacteristic = mock(BluetoothGattCharacteristic.class);

        when(mockCharacteristic.getUuid()).thenReturn(characteristicUUID);
        when(mockCharacteristic.getValue()).thenReturn(valueData);
        if (valueData != null && valueData.length >= 1) {
            when(mockCharacteristic.getIntValue(BluetoothGattCharacteristic.FORMAT_UINT8, 0)).thenReturn((int) valueData[0]);
        }
        when(mockCharacteristic.getProperties()).thenReturn(properties);
        when(mockCharacteristic.getService()).thenReturn(mockService);

        return mockCharacteristic;
    }
}
