package bluetooth.services;

import junit.framework.TestCase;

import bluetooth.helper.IOStub;
import dk.lego.devicesdk.device.ConnectInfo;
import dk.lego.devicesdk.services.Motor;

public class MotorTest extends TestCase {
    Motor motor;
    IOStub ioStub;

    @Override
    public void setUp() throws Exception {
        super.setUp();

        ioStub = new IOStub();
        ConnectInfo connectInfo = new ConnectInfo(1, 1, ConnectInfo.IOType.IO_TYPE_MOTOR);
        motor = Motor.createService(connectInfo, ioStub);
    }

    public void testRunMotorRight() {

        motor.run(Motor.MotorDirection.MOTOR_DIRECTION_RIGHT, 100);
        assertEquals(100, motor.getPower());
        assertEquals(100, ioStub.lastWrittenMotorPower.intValue());
        assertEquals(Motor.MotorDirection.MOTOR_DIRECTION_RIGHT, motor.getDirection());
        assertFalse(motor.isBraking());
        assertFalse(motor.isDrifting());

        motor.run(Motor.MotorDirection.MOTOR_DIRECTION_RIGHT, 1);
        assertEquals(1, motor.getPower());
        assertEquals(1, ioStub.lastWrittenMotorPower.intValue());
        assertEquals(Motor.MotorDirection.MOTOR_DIRECTION_RIGHT, motor.getDirection());
        assertFalse(motor.isBraking());
        assertFalse(motor.isDrifting());

        motor.run(Motor.MotorDirection.MOTOR_DIRECTION_RIGHT, 0);
        assertEquals(0, motor.getPower());
        assertEquals(0, ioStub.lastWrittenMotorPower.intValue());
        assertEquals(Motor.MotorDirection.MOTOR_DIRECTION_DRIFTING, motor.getDirection());
        assertTrue(motor.isDrifting());
        assertFalse(motor.isBraking());
    }

    public void testRunMotorLeft() {
        motor.run(Motor.MotorDirection.MOTOR_DIRECTION_LEFT, 100);
        assertEquals(100, motor.getPower());
        assertEquals(-100, ioStub.lastWrittenMotorPower.intValue());
        assertEquals(Motor.MotorDirection.MOTOR_DIRECTION_LEFT, motor.getDirection());
        assertFalse(motor.isBraking());
        assertFalse(motor.isDrifting());

        motor.run(Motor.MotorDirection.MOTOR_DIRECTION_LEFT, 1);
        assertEquals(1, motor.getPower());
        assertEquals(-1, ioStub.lastWrittenMotorPower.intValue());
        assertEquals(Motor.MotorDirection.MOTOR_DIRECTION_LEFT, motor.getDirection());
        assertFalse(motor.isBraking());
        assertFalse(motor.isDrifting());

        motor.run(Motor.MotorDirection.MOTOR_DIRECTION_LEFT, 0);
        assertEquals(0, motor.getPower());
        assertTrue(motor.isDrifting());
        assertEquals(0, ioStub.lastWrittenMotorPower.intValue());
        assertEquals(Motor.MotorDirection.MOTOR_DIRECTION_DRIFTING, motor.getDirection());
        assertTrue(motor.isDrifting());
        assertFalse(motor.isBraking());

    }


    public void testMotorDrift() {
        motor.drift();
        assertEquals(0, motor.getPower());
        assertEquals(0, ioStub.lastWrittenMotorPower.intValue());
        assertEquals(Motor.MotorDirection.MOTOR_DIRECTION_DRIFTING, motor.getDirection());
        assertTrue(motor.isDrifting());
        assertFalse(motor.isBraking());

    }

    public void testMotorBrake() {
        motor.brake();
        assertEquals(0, motor.getPower());
        assertEquals(127, ioStub.lastWrittenMotorPower.intValue());
        assertEquals(Motor.MotorDirection.MOTOR_DIRECTION_BRAKING, motor.getDirection());
        assertFalse(motor.isDrifting());
        assertTrue(motor.isBraking());
    }
}
