package bluetooth.services;

import android.test.suitebuilder.annotation.SmallTest;

import junit.framework.TestCase;

import bluetooth.helper.IOStub;
import dk.lego.devicesdk.device.ConnectInfo;
import dk.lego.devicesdk.services.PiezoTonePlayer;

public class PiezoTonePlayerTest extends TestCase {

    private IOStub ioStub;
    private ConnectInfo connectInfo;
    private PiezoTonePlayer player;

    @Override
    public void setUp() throws Exception {
        super.setUp();

        ioStub = new IOStub();
        connectInfo = new ConnectInfo(1, 1, ConnectInfo.IOType.IO_TYPE_PIEZO_TONE_PLAYER);
        player = PiezoTonePlayer.createService(connectInfo, ioStub);
    }

    @SmallTest
    public void testPlayPiezoTone()
    {
        int frequency = 440;
        int duration = 1000;

        // Execute MUT
        player.playFrequency(frequency, duration);

        // Verify
        assertEquals(frequency, ioStub.lastPiezoFrequencyWritten.intValue());
        assertEquals(duration, ioStub.lastPiezoDurationWritten.intValue());
    }

    @SmallTest
    public void testPlayPiezoNode()
    {
        verifyNode(PiezoTonePlayer.PiezoTonePlayerNote.PIEZO_NOTE_A, 4, 440);
        verifyNode(PiezoTonePlayer.PiezoTonePlayerNote.PIEZO_NOTE_B, 4, 494);
        verifyNode(PiezoTonePlayer.PiezoTonePlayerNote.PIEZO_NOTE_A, 5, 880);
    }

    @SmallTest
    public void testPlayPiezoNode_highest_supported_node() {
        verifyNode(PiezoTonePlayer.PiezoTonePlayerNote.PIEZO_NOTE_FIS, 6, 1480);
    }

    @SmallTest
    public void testPlayPiezoNode_higher_than_supported_is_rounded_down_to_max() {
        verifyNode(PiezoTonePlayer.PiezoTonePlayerNote.PIEZO_NOTE_G, 6, 1500);
    }

    @SmallTest
    public void testPlayPiezoNode_higher_octave_than_supported_is_rounded_down_to_max() {
        verifyNode(PiezoTonePlayer.PiezoTonePlayerNote.PIEZO_NOTE_G, 7, 1500);
    }

    @SmallTest
    public void testPlayPiezoNode_lowest_supported_node() {
        verifyNode(PiezoTonePlayer.PiezoTonePlayerNote.PIEZO_NOTE_C, 0, 16);
    }

    @SmallTest
    public void testStopPiezoTone()
    {
        assertNull(ioStub.isPiezoStopRequested);

        //Execute MUT
        player.stopPlaying();

        //Verify
        assertTrue(ioStub.isPiezoStopRequested);
    }

    private void verifyNode(PiezoTonePlayer.PiezoTonePlayerNote note, int octave, int translatingFrequency) {
        int duration = 0;

        // Execute MUT
        player.playNote(note, octave, duration);

        // Verify
        assertEquals(translatingFrequency, ioStub.lastPiezoFrequencyWritten.intValue());
        assertEquals(duration, ioStub.lastPiezoDurationWritten.intValue());

    }
}
