package device;

import android.test.suitebuilder.annotation.SmallTest;

import junit.framework.TestCase;

import dk.lego.devicesdk.device.DeviceInfo;

public class DeviceInfoTest extends TestCase {

    DeviceInfo deviceInfo;

    @Override
    protected void setUp() throws Exception {
        super.setUp();
        deviceInfo = new DeviceInfo();
    }

    @SmallTest
    public void testEquals() throws Exception
    {
        DeviceInfo deviceInfoAlt = new DeviceInfo();

        assertEquals(deviceInfo, deviceInfoAlt);
        assertEquals(deviceInfoAlt, deviceInfo);

        deviceInfo.setHardwareRevisionString("0.0.1");

        assertFalse(deviceInfo.equals(deviceInfoAlt));
        assertFalse(deviceInfoAlt.equals(deviceInfo));

        deviceInfoAlt.setHardwareRevisionString("0.0.1");
        assertEquals(deviceInfo, deviceInfoAlt);
        assertEquals(deviceInfoAlt, deviceInfo);

        deviceInfo.setFirmwareRevisionString("1");
        deviceInfo.setFirmwareRevisionString("2");
        deviceInfo.setManufacturerName("LEGO Systems");
        deviceInfoAlt.setFirmwareRevisionString("1");
        deviceInfoAlt.setFirmwareRevisionString("2");
        deviceInfoAlt.setManufacturerName("LEGO Systems");
        assertEquals(deviceInfo, deviceInfoAlt);
        assertEquals(deviceInfoAlt, deviceInfo);
    }
    
    @SmallTest
    public void testIsComplete() throws Exception
    {
        deviceInfo.setHardwareRevisionString("0.0.1");
        assertFalse(deviceInfo.isComplete());

        deviceInfo.setFirmwareRevisionString("1");
        deviceInfo.setSoftwareRevisionString("2");
        deviceInfo.setManufacturerName("LEGO Systems");
        assertTrue(deviceInfo.isComplete());
    }

    @SmallTest
    public void testRevisionFromStringParsing() throws Exception
    {
        deviceInfo.setHardwareRevisionString("1.2.03.004");

        assertEquals(1, deviceInfo.getHardwareRevision().getMajorVersion());
        assertEquals(2, deviceInfo.getHardwareRevision().getMinorVersion());
        assertEquals(3, deviceInfo.getHardwareRevision().getBugFixVersion());
        assertEquals(4, deviceInfo.getHardwareRevision().getBuildNumber());
    }

    @SmallTest
    public void testRevisionComponents_success_scenario() throws Exception
    {
        assertNull(deviceInfo.getFirmwareRevision());
        assertNull(deviceInfo.getSoftwareRevision());
        assertNull(deviceInfo.getHardwareRevision());

        deviceInfo.setFirmwareRevisionString("1");
        assertEquals(deviceInfo.getFirmwareRevision().getMajorVersion(), 1);
        assertEquals(deviceInfo.getFirmwareRevision().getMinorVersion(), 0);
        assertEquals(deviceInfo.getFirmwareRevision().getBugFixVersion(), 0);

        deviceInfo.setFirmwareRevisionString("0.2");
        assertEquals(deviceInfo.getFirmwareRevision().getMajorVersion(), 0);
        assertEquals(deviceInfo.getFirmwareRevision().getMinorVersion(), 2);
        assertEquals(deviceInfo.getFirmwareRevision().getBugFixVersion(), 0);

        deviceInfo.setFirmwareRevisionString("1.2.99.200");
        assertEquals(deviceInfo.getFirmwareRevision().getMajorVersion(), 1);
        assertEquals(deviceInfo.getFirmwareRevision().getMinorVersion(), 2);
        assertEquals(deviceInfo.getFirmwareRevision().getBugFixVersion(), 99);
        assertEquals(deviceInfo.getFirmwareRevision().getBuildNumber(), 200);
    }


    public void testRevisionComponents_not_number_gives_zero_version_numbers() throws Exception
    {
        deviceInfo.setFirmwareRevisionString("A.B.C");
        assertNotNull(deviceInfo.getFirmwareRevision());

        assertEquals(deviceInfo.getFirmwareRevision().getMajorVersion(), 0);
        assertEquals(deviceInfo.getFirmwareRevision().getMinorVersion(), 0);
        assertEquals(deviceInfo.getFirmwareRevision().getBugFixVersion(), 0);
    }
}
