package logging;

import android.test.suitebuilder.annotation.SmallTest;

import junit.framework.TestCase;

import dk.lego.devicesdk.logging.CustomLogger;
import dk.lego.devicesdk.logging.DefaultSilentCustomLogger;
import dk.lego.devicesdk.logging.LDSDKLogger;

public class LDSDKLoggerTest extends TestCase {

    public static final String EXPECTED_MESSAGE = "ExpectedLogEntry";
    public static final Exception EXPECTED_EXCEPTION = new Exception(EXPECTED_MESSAGE);

    private CustomLogger testLogger;
    private String lastMessage;
    public Exception lastException;

    @Override
    protected void setUp() throws Exception {
        super.setUp();

        lastMessage = null;
        lastException = null;
        testLogger = new CustomLogger() {

            @Override
            public void verbose(String s) {
                lastMessage = s;
            }

            @Override
            public void debug(String s) {
                lastMessage = s;
            }

            @Override
            public void info(String s) {
                lastMessage = s;
            }

            @Override
            public void warn(String s) {
                lastMessage = s;
            }

            @Override
            public void error(String s) {
                lastMessage = s;
            }

            @Override
            public void error(Exception e) {
                lastException = e;
            }
        };

        LDSDKLogger.getInstance().setCustomLogger(testLogger);
    }

    @SmallTest
    public void testVerbose() throws Exception {
        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_DEBUG);
        LDSDKLogger.v(EXPECTED_MESSAGE);

        assertNull(lastMessage);

        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_VERBOSE);
        LDSDKLogger.v(EXPECTED_MESSAGE);

        assertEquals(EXPECTED_MESSAGE, lastMessage);
    }

    @SmallTest
    public void testDebug() throws Exception {
        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_INFO);
        LDSDKLogger.d(EXPECTED_MESSAGE);

        assertNull(lastMessage);

        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_VERBOSE);
        LDSDKLogger.d(EXPECTED_MESSAGE);

        assertEquals(EXPECTED_MESSAGE, lastMessage);

        lastMessage = null;

        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_DEBUG);
        LDSDKLogger.d(EXPECTED_MESSAGE);

        assertEquals(EXPECTED_MESSAGE, lastMessage);
    }

    @SmallTest
    public void testInfo() throws Exception {
        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_WARN);
        LDSDKLogger.i(EXPECTED_MESSAGE);

        assertNull(lastMessage);

        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_DEBUG);
        LDSDKLogger.i(EXPECTED_MESSAGE);

        assertEquals(EXPECTED_MESSAGE, lastMessage);

        lastMessage = null;

        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_INFO);
        LDSDKLogger.i(EXPECTED_MESSAGE);

        assertEquals(EXPECTED_MESSAGE, lastMessage);
    }

    @SmallTest
    public void testWarn() throws Exception {
        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_ERROR);
        LDSDKLogger.w(EXPECTED_MESSAGE);

        assertNull(lastMessage);

        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_INFO);
        LDSDKLogger.w(EXPECTED_MESSAGE);

        assertEquals(EXPECTED_MESSAGE, lastMessage);

        lastMessage = null;

        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_WARN);
        LDSDKLogger.w(EXPECTED_MESSAGE);

        assertEquals(EXPECTED_MESSAGE, lastMessage);
    }

    @SmallTest
    public void testErrorMsg() throws Exception {

        assertNull(lastMessage);

        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_WARN);
        LDSDKLogger.e(EXPECTED_MESSAGE);

        assertEquals(EXPECTED_MESSAGE, lastMessage);

        lastMessage = null;

        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_ERROR);
        LDSDKLogger.e(EXPECTED_MESSAGE);

        assertEquals(EXPECTED_MESSAGE, lastMessage);
    }

    @SmallTest
    public void testErrorException() throws Exception {

        assertNull(lastException);

        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_WARN);
        LDSDKLogger.e(EXPECTED_EXCEPTION);

        assertEquals(EXPECTED_EXCEPTION, lastException);
        assertEquals(EXPECTED_MESSAGE, lastException.getLocalizedMessage());

        lastMessage = null;

        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_ERROR);
        LDSDKLogger.e(EXPECTED_EXCEPTION);

        assertEquals(EXPECTED_EXCEPTION, lastException);
        assertEquals(EXPECTED_MESSAGE, lastException.getLocalizedMessage());
    }

    @SmallTest
    public void testSilentLogging() throws Exception {

        LDSDKLogger.getInstance().setCustomLogger(new DefaultSilentCustomLogger());

        assertNull(lastMessage);

        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_WARN);
        LDSDKLogger.e(EXPECTED_MESSAGE);

        assertNull(lastMessage);

        LDSDKLogger.getInstance().setLogLevel(LDSDKLogger.LoggerLevel.LOGGER_LEVEL_ERROR);
        LDSDKLogger.e(EXPECTED_MESSAGE);

        assertNull(lastMessage);
    }
}
