package dk.lego.devicesdk.bluetooth;

import java.util.Collections;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import dk.lego.devicesdk.LDSDKError;
import dk.lego.devicesdk.utils.HandlerHelper;

class BluetoothDeviceManagerCallbackHelper {

    private final Set<LegoBluetoothDeviceManagerCallbackListener> callbackListeners = Collections.newSetFromMap(new ConcurrentHashMap<LegoBluetoothDeviceManagerCallbackListener, Boolean>());

    protected void registerCallbackListener(LegoBluetoothDeviceManagerCallbackListener callback) {
        callbackListeners.add(callback);
    }

    protected void unregisterCallbackListener(LegoBluetoothDeviceManagerCallbackListener callback) {
        callbackListeners.remove(callback);
    }

    protected void performDeviceDidAppearCallback(final LegoBluetoothDevice legoBluetoothDevice) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (final LegoBluetoothDeviceManagerCallbackListener listener : callbackListeners) {
                    listener.onDeviceDidAppear(legoBluetoothDevice);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDeviceDidDisappearCallback(final LegoBluetoothDevice legoBluetoothDevice) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (LegoBluetoothDeviceManagerCallbackListener callbackListener : callbackListeners) {
                    callbackListener.onDeviceDidDisappear(legoBluetoothDevice);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }


    protected void performWillStartConnectingToDeviceCallback(final LegoBluetoothDevice legoBluetoothDevice) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (LegoBluetoothDeviceManagerCallbackListener callbackListener : callbackListeners) {
                    callbackListener.onWillStartConnectingToDevice(legoBluetoothDevice);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidStartInterrogatingDeviceCallback(final LegoBluetoothDevice legoBluetoothDevice) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (LegoBluetoothDeviceManagerCallbackListener callbackListener : callbackListeners) {
                    callbackListener.onDidStartInterrogatingDevice(legoBluetoothDevice);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidFinishInterrogatingDeviceCallback(final LegoBluetoothDevice legoBluetoothDevice) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (LegoBluetoothDeviceManagerCallbackListener callbackListener : callbackListeners) {
                    callbackListener.onDidFinishInterrogatingDevice(legoBluetoothDevice);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidDisconnectFromDeviceCallback(final LegoBluetoothDevice legoBluetoothDevice, final boolean autoReconnect, final LDSDKError error) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (LegoBluetoothDeviceManagerCallbackListener callbackListener : callbackListeners) {
                    callbackListener.onDidDisconnectFromDevice(legoBluetoothDevice, autoReconnect, error);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidFailToConnectToDeviceCallback(final LegoBluetoothDevice legoBluetoothDevice, final boolean autoReconnect, final LDSDKError error) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (LegoBluetoothDeviceManagerCallbackListener callbackListener : callbackListeners) {
                    callbackListener.onDidFailToConnectToDevice(legoBluetoothDevice, autoReconnect, error);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    private void postRunnableFromSuitedHandler(Runnable deviceDisappeared) {
        HandlerHelper.getInstance().post(deviceDisappeared);
    }
}
