package dk.lego.devicesdk.bluetooth;

import java.util.Collections;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import dk.lego.devicesdk.LDSDKError;
import dk.lego.devicesdk.device.DeviceCallbackListener;
import dk.lego.devicesdk.device.DeviceInfo;
import dk.lego.devicesdk.device.LegoDevice;
import dk.lego.devicesdk.services.LegoService;
import dk.lego.devicesdk.utils.HandlerHelper;

public class DeviceCallbackHelper {

    private final Set<DeviceCallbackListener> callbackListeners = Collections.newSetFromMap(new ConcurrentHashMap<DeviceCallbackListener, Boolean>());

    public void registerCallbackListener(DeviceCallbackListener callback) {
        callbackListeners.add(callback);
    }

    public void unregisterCallbackListener(DeviceCallbackListener callback) {
        callbackListeners.remove(callback);
    }


    protected void performDidUpdateDeviceInfoCallack(final LegoDevice device, final DeviceInfo deviceInfo, final LDSDKError error) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceCallbackListener callbackListener : callbackListeners) {
                    callbackListener.didUpdateDeviceInfo(device, deviceInfo, error);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidChangeButtonStateCallback(final LegoDevice device, final boolean buttonPressed) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceCallbackListener callbackListener : callbackListeners)
                {
                    callbackListener.didChangeButtonState(device, buttonPressed);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidUpdateLowVoltageStateCallback(final LegoDevice device, final boolean lowVoltage) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceCallbackListener callbackListener : callbackListeners)
                {
                    callbackListener.didUpdateLowVoltageState(device, lowVoltage);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidUpdateBatteryLevelCallback(final LegoDevice device, final int batteryLevel) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceCallbackListener callbackListener : callbackListeners)
                {
                    callbackListener.didUpdateBatteryLevel(device, batteryLevel);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidChangeNameCallback(final LegoDevice device, final String oldName, final String newName) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceCallbackListener callbackListener : callbackListeners) {
                    callbackListener.didChangeNameFrom(device, oldName, newName);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidFailToAddServiceWithErrorCallback(final LegoDevice device, final LDSDKError error) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceCallbackListener callbackListener : callbackListeners) {
                    callbackListener.didFailToAddServiceWithError(device, error);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidRemoveServiceCallback(final LegoDevice device, final LegoService service) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceCallbackListener callbackListener : callbackListeners) {
                    callbackListener.didRemoveService(device, service);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidAddServiceCallback(final LegoDevice device, final LegoService service) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceCallbackListener callbackListener : callbackListeners) {
                    callbackListener.didAddService(device, service);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    private void postRunnableFromSuitedHandler(Runnable deviceDisappeared) {
        HandlerHelper.getInstance().post(deviceDisappeared);
    }
}
