package dk.lego.devicesdk.bluetooth;

import android.util.SparseArray;

import java.util.Collections;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import dk.lego.devicesdk.device.ConnectInfo;
import dk.lego.devicesdk.input_output.IO;
import dk.lego.devicesdk.input_output.IOCallbackListener;
import dk.lego.devicesdk.input_output.InputFormat;
import dk.lego.devicesdk.utils.HandlerHelper;

public class IoCallbackHelper {

    private final Set<IOCallbackListener> callbackListeners = Collections.newSetFromMap(new ConcurrentHashMap<IOCallbackListener, Boolean>());

    public void registerCallbackListener(IOCallbackListener callback) {
        callbackListeners.add(callback);
    }

    public void unregisterCallbackListener(IOCallbackListener callback) {
        callbackListeners.remove(callback);
    }

    public void performDidRequestConnectInfo(final IO io) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (IOCallbackListener callbackListener : callbackListeners) {
                    callbackListener.didRequestConnectInfo(io);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    public void performDidReceiveValueDataCallback(final IO io, final SparseArray<byte[]> idToValue) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (IOCallbackListener callbackListener : callbackListeners) {
                    ConnectInfo connectInfo = callbackListener.didRequestConnectInfo(io);
                    byte[] value = idToValue.get(connectInfo.getConnectId());
                    if (value != null) {
                        callbackListener.didReceiveValueData(io, value);
                    }
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    public void performDidReceiveInputFormatCallback(final IO io, final InputFormat format) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (IOCallbackListener callbackListener : callbackListeners) {
                    ConnectInfo connectInfo = callbackListener.didRequestConnectInfo(io);
                    if (connectInfo.getConnectId() == format.getConnectId()) {
                        callbackListener.didReceiveInputFormat(io, format);
                    }
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    private void postRunnableFromSuitedHandler(Runnable deviceDisappeared) {
        HandlerHelper.getInstance().post(deviceDisappeared);
    }
}
