package dk.lego.devicesdk.bluetooth;

import android.bluetooth.BluetoothGattCharacteristic;

import dk.lego.devicesdk.android.bluetooth.le.ScanRecord;
import dk.lego.devicesdk.device.LegoDevice;

/**
 * This class represents a LEGO Bluetooth device
 */
public interface LegoBluetoothDevice extends LegoDevice {
    /**
     * @return RSSI Value of the Bluetooth device
     */
    public int getRSSIValue();

    /**
     * @return Advertisement data/scan record of the Bluetooth device
     */
    public ScanRecord getScanRecord();

    /**
     * @return The Android abstraction of the Bluetooth device
     */
    public android.bluetooth.BluetoothDevice getAndroidBluetoothDevice();

    /**
     * Wrapper for {@link android.bluetooth.BluetoothGatt#setCharacteristicNotification(android.bluetooth.BluetoothGattCharacteristic, boolean)}
     *
     * @param characteristic The characteristic for which to enable notifications
     * @param enable Set to true to enable notifications/indications
     */
    void setCharacteristicNotification(BluetoothGattCharacteristic characteristic, boolean enable);

    /**
     * Wrapper for {@link android.bluetooth.BluetoothGatt#writeCharacteristic(android.bluetooth.BluetoothGattCharacteristic)}
     * On Android we need to maintain a queue of read/write requests to ensure proper functionning.
     * This call adds write requests to the queue.
     *
     * @param characteristic Characteristic to write on the remote device
     * @param data The value data to write. See {@link android.bluetooth.BluetoothGattCharacteristic#setValue(String)}
     * @param writeType The type of write to perform. See {@link android.bluetooth.BluetoothGattCharacteristic#setWriteType(int)}
     */
    void addWriteOfCharacteristicToQueue(BluetoothGattCharacteristic characteristic, byte[] data, int writeType);
}
