package dk.lego.devicesdk.bluetooth.service_definitions;

import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattService;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import dk.lego.devicesdk.logging.LDSDKLogger;

public class BluetoothServiceDefinition {

    protected String serviceName;
    protected UUID serviceUUID;
    protected List<CharacteristicDefinition> characteristicDefinitions = new ArrayList<>();
    private List<UUID> characteristicUUIDs;

    public BluetoothServiceDefinition() {
    }

    public static IOServiceDefinition ioServiceDefinition() {
        return IOServiceDefinition.getInstance();
    }

    public static DeviceServiceDefinition deviceServiceDefinition() {
        return DeviceServiceDefinition.getInstance();
    }

    public static DeviceInfoServiceDefinition deviceInfoServiceDefinition() {
        return DeviceInfoServiceDefinition.getInstance();
    }

    public static BatteryServiceDefinition batteryServiceDefinition() {
        return BatteryServiceDefinition.getInstance();
    }

    public static BluetoothServiceDefinition serviceDefinitionWithUUID(UUID serviceUUID) {
        if (ioServiceDefinition().getServiceUUID().equals(serviceUUID)) {
            return ioServiceDefinition();
        } else if (deviceServiceDefinition().getServiceUUID().equals(serviceUUID)) {
            return deviceServiceDefinition();
        } else if (deviceInfoServiceDefinition().getServiceUUID().equals(serviceUUID)) {
            return deviceInfoServiceDefinition();
        } else if (batteryServiceDefinition().getServiceUUID().equals(serviceUUID)) {
            return batteryServiceDefinition();
        }
        return null;
    }

    public String getServiceName() {
        return serviceName;
    }

    public UUID getServiceUUID() {
        return serviceUUID;
    }

    public boolean matchesService(BluetoothGattService service) {
        return (serviceUUID.equals(service.getUuid()));
    }

    BluetoothGattCharacteristic characteristicForDefinition(CharacteristicDefinition definition, BluetoothGattService service)
    {
        for (BluetoothGattCharacteristic characteristic : service.getCharacteristics()) {
            if (characteristic.getUuid().equals(definition.getUuid())) {
                return characteristic;
            }
        }
        return null;
    }

    public List<UUID> characteristicUUIDs()
    {
        if (characteristicUUIDs == null) {
            characteristicUUIDs = new ArrayList<>(characteristicDefinitions.size());

            for (CharacteristicDefinition definition : characteristicDefinitions) {
                characteristicUUIDs.add(definition.getUuid());
            }
        }
        return characteristicUUIDs;
    }

    public CharacteristicDefinition characteristicDefinitionWithUUID(UUID uuid)
    {
        for (CharacteristicDefinition definition : characteristicDefinitions) {
            if (definition.getUuid().equals(uuid)) {
                return definition;
            }
        }
        LDSDKLogger.d(String.format("Did not find characteristic with UUID %s in service %s", uuid.toString(), serviceName));
        return null;
    }

    @Override
    public String toString() {
        return String.format("<%s: ", getClass().getSimpleName()) +
                String.format("< %s", serviceName) +
                String.format("(%s)", serviceUUID) +
                String.format(", self.characteristicDefinitions=%s", characteristicDefinitions) + ">";
    }

    private String toStringShort() {
        return String.format("%s(%s)", serviceName, serviceUUID.toString());
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        return isEqualToDefinition((BluetoothServiceDefinition) o);
    }

    public boolean isEqualToDefinition(BluetoothServiceDefinition o) {
        if (characteristicDefinitions != null ? !characteristicDefinitions.equals(o.characteristicDefinitions) : o.characteristicDefinitions != null)
            return false;
        if (characteristicUUIDs != null ? !characteristicUUIDs.equals(o.characteristicUUIDs) : o.characteristicUUIDs != null)
            return false;
        if (serviceName != null ? !serviceName.equals(o.serviceName) : o.serviceName != null)
            return false;
        if (serviceUUID != null ? !serviceUUID.equals(o.serviceUUID) : o.serviceUUID != null)
            return false;
        return true;
    }

    @Override
    public int hashCode() {
        int result = serviceName != null ? serviceName.hashCode() : 0;
        result = 31 * result + (serviceUUID != null ? serviceUUID.hashCode() : 0);
        result = 31 * result + (characteristicDefinitions != null ? characteristicDefinitions.hashCode() : 0);
        result = 31 * result + (characteristicUUIDs != null ? characteristicUUIDs.hashCode() : 0);
        return result;
    }
}
