package dk.lego.devicesdk.bluetooth.service_definitions;

import java.util.UUID;

import dk.lego.devicesdk.bluetooth.BluetoothHelper;

public class DeviceInfoServiceDefinition extends BluetoothServiceDefinition {

    private static final String DEVICE_INFO_SERVICE_UUID = "0x180A";

    private static final String DEVICE_INFO_FIRMWARE_REVISION_CHARACTERISTIC_UUID = "0x2A26";
    private static final String DEVICE_INFO_HARDWARE_REVISION_CHARACTERISTIC_UUID = "0x2A27";
    private static final String DEVICE_INFO_SOFTWARE_REVISION_CHARACTERISTIC_UUID = "0x2A28";
    private static final String DEVICE_INFO_MANUFACTURER_NAME_CHARACTERISTIC_UUID = "0x2A29";

    private static DeviceInfoServiceDefinition instance = null;
    private CharacteristicDefinition firmwareRevision;
    private CharacteristicDefinition hardwareRevision;
    private CharacteristicDefinition softwareRevision;
    private CharacteristicDefinition manufacturerName;

    private DeviceInfoServiceDefinition() {
    }

    public static DeviceInfoServiceDefinition getInstance() {
        if (instance == null) {
            instance = new DeviceInfoServiceDefinition();
            instance.init();
        }
        return instance;
    }


    void init() {
        serviceUUID = UUID.fromString(BluetoothHelper.UUIDWithPrefixStandardBase(DEVICE_INFO_SERVICE_UUID));
        serviceName = "DeviceInfoService";

        firmwareRevision = new CharacteristicDefinition(
                "FirmwareRevision",
                this,
                UUID.fromString(BluetoothHelper.UUIDWithPrefixStandardBase(DEVICE_INFO_FIRMWARE_REVISION_CHARACTERISTIC_UUID)),
                true,
                (BluetoothHelper.Properties.PROPERTY_READ.ordinal()),
                0,
                (BluetoothHelper.Permissions.PERMISSION_READ.ordinal()));

        hardwareRevision = new CharacteristicDefinition(
                "HardwareRevision",
                this,
                UUID.fromString(BluetoothHelper.UUIDWithPrefixStandardBase(DEVICE_INFO_HARDWARE_REVISION_CHARACTERISTIC_UUID)),
                false,
                (BluetoothHelper.Properties.PROPERTY_READ.ordinal()),
                0,
                (BluetoothHelper.Permissions.PERMISSION_READ.ordinal()));

        softwareRevision = new CharacteristicDefinition(
                "SoftwareRevision",
                this,
                UUID.fromString(BluetoothHelper.UUIDWithPrefixStandardBase(DEVICE_INFO_SOFTWARE_REVISION_CHARACTERISTIC_UUID)),
                true,
                (BluetoothHelper.Properties.PROPERTY_READ.ordinal()),
                0,
                (BluetoothHelper.Permissions.PERMISSION_READ.ordinal()));

        manufacturerName = new CharacteristicDefinition(
                "ManufacturerName",
                this,
                UUID.fromString(BluetoothHelper.UUIDWithPrefixStandardBase(DEVICE_INFO_MANUFACTURER_NAME_CHARACTERISTIC_UUID)),
                true,
                (BluetoothHelper.Properties.PROPERTY_READ.ordinal()),
                0,
                (BluetoothHelper.Permissions.PERMISSION_READ.ordinal()));

        characteristicDefinitions.add(firmwareRevision);
        characteristicDefinitions.add(hardwareRevision);
        characteristicDefinitions.add(softwareRevision);
        characteristicDefinitions.add(manufacturerName);
    }

    public CharacteristicDefinition getFirmwareRevision() {
        return firmwareRevision;
    }

    public CharacteristicDefinition getHardwareRevision() {
        return hardwareRevision;
    }

    public CharacteristicDefinition getSoftwareRevision() {
        return softwareRevision;
    }

    public CharacteristicDefinition getManufacturerName() {
        return manufacturerName;
    }

    @Override
    public boolean equals(Object o) {
        return super.equals(o);
    }

    @Override
    public int hashCode() {
        return super.hashCode();
    }
}
