package dk.lego.devicesdk.bluetooth.service_definitions;

import java.util.UUID;

import dk.lego.devicesdk.bluetooth.BluetoothHelper;

public class DeviceServiceDefinition extends BluetoothServiceDefinition {

    private static final String HUB_SERVICE_16_BIT_UUID = "0x1523";
    private static final String HUB_CHARACTERISTIC_NAME_UUID = "0x1524";
    private static final String HUB_CHARACTERISTIC_COLOR = "0x1525";
    private static final String HUB_CHARACTERISTIC_BUTTON_STATE = "0x1526";
    private static final String HUB_CHARACTERISTIC_ATTACHED_IO = "0x1527";
    private static final String HUB_CHARACTERISTIC_LOW_VOLTAGE_ALERT = "0x1528";

    private static DeviceServiceDefinition instance = null;
    private CharacteristicDefinition deviceName;
    private CharacteristicDefinition attachedIO;
    private CharacteristicDefinition deviceButton;
    private CharacteristicDefinition lowVoltageAlert;

    private DeviceServiceDefinition() {
    }

    public static DeviceServiceDefinition getInstance() {
        if (instance == null) {
            instance = new DeviceServiceDefinition();
            instance.init();
        }
        return instance;
    }


    void init() {
        serviceUUID = UUID.fromString(BluetoothHelper.UUIDWithPrefixCustomBase(HUB_SERVICE_16_BIT_UUID));
        serviceName = "DeviceService";

        deviceName = new CharacteristicDefinition(
                "DeviceName",
                this,
                UUID.fromString(BluetoothHelper.UUIDWithPrefixCustomBase(HUB_CHARACTERISTIC_NAME_UUID)),
                true,
                (BluetoothHelper.Properties.PROPERTY_WRITE.ordinal() | BluetoothHelper.Properties.PROPERTY_WRITE_NO_RESPONSE.ordinal() | BluetoothHelper.Properties.PROPERTY_READ.ordinal()),
                0,
                (BluetoothHelper.Permissions.PERMISSION_WRITE.ordinal() | BluetoothHelper.Permissions.PERMISSION_READ.ordinal()));

        attachedIO = new CharacteristicDefinition(
                "AttachedIO",
                this,
                UUID.fromString(BluetoothHelper.UUIDWithPrefixCustomBase(HUB_CHARACTERISTIC_ATTACHED_IO)),
                true,
                (BluetoothHelper.Properties.PROPERTY_NOTIFY.ordinal()),
                0,
                (BluetoothHelper.Permissions.PERMISSION_READ.ordinal()));

        deviceButton = new CharacteristicDefinition(
                "DeviceButton",
                this,
                UUID.fromString(BluetoothHelper.UUIDWithPrefixCustomBase(HUB_CHARACTERISTIC_BUTTON_STATE)),
                true,
                (BluetoothHelper.Properties.PROPERTY_READ.ordinal() | BluetoothHelper.Properties.PROPERTY_NOTIFY.ordinal()),
                0,
                (BluetoothHelper.Permissions.PERMISSION_READ.ordinal()));

        lowVoltageAlert = new CharacteristicDefinition(
                "LowVoltageAlert",
                this,
                UUID.fromString(BluetoothHelper.UUIDWithPrefixCustomBase(HUB_CHARACTERISTIC_LOW_VOLTAGE_ALERT)),
                false,
                (BluetoothHelper.Properties.PROPERTY_READ.ordinal() | BluetoothHelper.Properties.PROPERTY_NOTIFY.ordinal()),
                0,
                (BluetoothHelper.Permissions.PERMISSION_READ.ordinal()));

        characteristicDefinitions.add(deviceName);
        characteristicDefinitions.add(attachedIO);
        characteristicDefinitions.add(deviceButton);
        characteristicDefinitions.add(lowVoltageAlert);
    }

    public CharacteristicDefinition getDeviceName() {
        return deviceName;
    }

    public CharacteristicDefinition getAttachedIO() {
        return attachedIO;
    }

    public CharacteristicDefinition getDeviceButton() {
        return deviceButton;
    }

    public CharacteristicDefinition getLowVoltageAlert() {
        return lowVoltageAlert;
    }

    public UUID getShortServiceUUID() {
        return UUID.fromString(HUB_SERVICE_16_BIT_UUID);
    }

    @Override
    public boolean equals(Object o) {
        return super.equals(o);
    }

    @Override
    public int hashCode() {
        return super.hashCode();
    }
}