package dk.lego.devicesdk.bluetooth.service_definitions;

import java.util.UUID;

import dk.lego.devicesdk.bluetooth.BluetoothHelper;

public class IOServiceDefinition extends BluetoothServiceDefinition {

    private static final String INPUT_SERVICE_UUID = "0x4F0E";

    private static final String CHARACTERISTIC_INPUT_VALUE_UUID    = "0x1560";
    private static final String CHARACTERISTIC_INPUT_FORMAT_UUID   = "0x1561";
    private static final String CHARACTERISTIC_INPUT_COMMAND_UUID  = "0x1563";
    private static final String CHARACTERISTIC_OUTPUT_COMMAND_UUID = "0x1565";

    private static IOServiceDefinition instance = null;
    private CharacteristicDefinition inputValue;
    private CharacteristicDefinition inputFormat;
    private CharacteristicDefinition inputCommand;
    private CharacteristicDefinition outputCommand;

    private IOServiceDefinition() {
    }

    public static IOServiceDefinition getInstance() {
        if (instance == null) {
            instance = new IOServiceDefinition();
            instance.init();
        }
        return instance;
    }

    void init() {
        serviceUUID = UUID.fromString(BluetoothHelper.UUIDWithPrefixCustomBase(INPUT_SERVICE_UUID));
        serviceName = "IOService";

        inputValue = new CharacteristicDefinition(
                "InputValue",
                this,
                UUID.fromString(BluetoothHelper.UUIDWithPrefixCustomBase(CHARACTERISTIC_INPUT_VALUE_UUID)),
                true,
                (BluetoothHelper.Properties.PROPERTY_NOTIFY.ordinal()),
                0,
                (BluetoothHelper.Permissions.PERMISSION_READ.ordinal()));

        inputFormat = new CharacteristicDefinition(
                "InputFormat",
                this,
                UUID.fromString(BluetoothHelper.UUIDWithPrefixCustomBase(CHARACTERISTIC_INPUT_FORMAT_UUID)),
                true,
                (BluetoothHelper.Properties.PROPERTY_NOTIFY.ordinal()),
                0,
                (BluetoothHelper.Permissions.PERMISSION_READ.ordinal()));

        inputCommand = new CharacteristicDefinition(
                "InputCommand",
                this,
                UUID.fromString(BluetoothHelper.UUIDWithPrefixCustomBase(CHARACTERISTIC_INPUT_COMMAND_UUID)),
                true,
                (BluetoothHelper.Properties.PROPERTY_WRITE_NO_RESPONSE.ordinal() | BluetoothHelper.Properties.PROPERTY_WRITE.ordinal()),
                0,
                (BluetoothHelper.Permissions.PERMISSION_READ.ordinal()));

        outputCommand = new CharacteristicDefinition(
                "OutputCommand",
                this,
                UUID.fromString(BluetoothHelper.UUIDWithPrefixCustomBase(CHARACTERISTIC_OUTPUT_COMMAND_UUID)),
                true,
                (BluetoothHelper.Properties.PROPERTY_WRITE_NO_RESPONSE.ordinal() | BluetoothHelper.Properties.PROPERTY_WRITE.ordinal()),
                0,
                (BluetoothHelper.Permissions.PERMISSION_READ.ordinal()));

        characteristicDefinitions.add(inputValue);
        characteristicDefinitions.add(inputFormat);
        characteristicDefinitions.add(inputCommand);
        characteristicDefinitions.add(outputCommand);
    }

    public CharacteristicDefinition getInputValue() {
        return inputValue;
    }

    public CharacteristicDefinition getInputFormat() {
        return inputFormat;
    }

    public CharacteristicDefinition getInputCommand() {
        return inputCommand;
    }

    public CharacteristicDefinition getOutputCommand() {
        return outputCommand;
    }

    @Override
    public boolean equals(Object o) {
        return super.equals(o);
    }

    @Override
    public int hashCode() {
        return super.hashCode();
    }
}
