package dk.lego.devicesdk.bluetooth.wrappers;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothManager;
import android.content.Context;

public class AndroidBluetoothAdapterWrapperImpl implements AndroidBluetoothAdapterWrapper {
    private final BluetoothAdapter adapter;
    private AndroidBluetoothScanCallbackListener listener = null;

    private final BluetoothAdapter.LeScanCallback leScanCallback = new BluetoothAdapter.LeScanCallback() {
        @Override
        public synchronized void onLeScan(BluetoothDevice bluetoothDevice, int rssi, byte[] scanRecord) {
            AndroidBluetoothDeviceWrapper androidBluetoothDeviceWrapper = new AndroidBluetoothDeviceWrapperImpl(bluetoothDevice);

            assert listener != null;
            listener.onLeScan(androidBluetoothDeviceWrapper, rssi, scanRecord);
        }
    };

    public AndroidBluetoothAdapterWrapperImpl(Context context) throws IllegalStateException {
        BluetoothManager bluetoothManager = (BluetoothManager) context.getSystemService(Context.BLUETOOTH_SERVICE);
        if (bluetoothManager != null) {
            adapter = bluetoothManager.getAdapter();
        } else {
            throw new IllegalStateException("getSystemService() retuned null");
        }
    }

    @Override
    public void startLeScan() {
        adapter.startLeScan(leScanCallback);
    }

    @Override
    public void stopLeScan() {
        adapter.stopLeScan(leScanCallback);
    }

    @Override
    public boolean isBluetoothAvailable() {
        return (adapter != null);
    }

    @Override
    public boolean isBluetoothEnabledOnDevice() {
        return adapter.isEnabled();
    }

    @Override
    public void connectCallbackListener(AndroidBluetoothScanCallbackListener listener) {
        this.listener = listener;
    }
}
