package dk.lego.devicesdk.device;

/**
 Represent info about a connected {@link dk.lego.devicesdk.device.LegoDevice} such as firmware-, hardware- and software revision.
 */
public class DeviceInfo {

    /**
     * The firmware revision of a {@link dk.lego.devicesdk.device.LegoDevice}
     */
    private Revision firmwareRevision = null;

    /**
     * The hardware revision of a {@link dk.lego.devicesdk.device.LegoDevice}
     */
    private Revision hardwareRevision = null;

    /**
     * The software revision of a {@link dk.lego.devicesdk.device.LegoDevice}
     */
    private Revision softwareRevision = null;

    /**
     * The manufacturer name of the {@link dk.lego.devicesdk.device.LegoDevice}
     */
    private String manufacturerName = null;

    public DeviceInfo() {
    }

    public Revision getFirmwareRevision() {
        return firmwareRevision;
    }

    public Revision getHardwareRevision() {
        return hardwareRevision;
    }

    public Revision getSoftwareRevision() {
        return softwareRevision;
    }

    public String getManufacturerName() {
        return manufacturerName;
    }

    public void setFirmwareRevisionString(String firmwareRevision) {
        this.firmwareRevision = Revision.createFromString(firmwareRevision);
    }

    public void setHardwareRevisionString(String hardwareRevision) {
        this.hardwareRevision = Revision.createFromString(hardwareRevision);
    }

    public void setSoftwareRevisionString(String softwareRevision) {
        this.softwareRevision = Revision.createFromString(softwareRevision);
    }

    public void setManufacturerName(String manufacturerName) {
        this.manufacturerName = manufacturerName;
    }

    public boolean isComplete() {
        // Hardware revision is not mandatory
        return (firmwareRevision != null && /* hardwareRevision != null && */ softwareRevision != null && manufacturerName != null);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        DeviceInfo that = (DeviceInfo) o;

        if (firmwareRevision != null ? !firmwareRevision.equals(that.firmwareRevision) : that.firmwareRevision != null)
            return false;
        if (hardwareRevision != null ? !hardwareRevision.equals(that.hardwareRevision) : that.hardwareRevision != null)
            return false;
        if (manufacturerName != null ? !manufacturerName.equals(that.manufacturerName) : that.manufacturerName != null)
            return false;
        if (softwareRevision != null ? !softwareRevision.equals(that.softwareRevision) : that.softwareRevision != null)
            return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = firmwareRevision != null ? firmwareRevision.hashCode() : 0;
        result = 31 * result + (hardwareRevision != null ? hardwareRevision.hashCode() : 0);
        result = 31 * result + (softwareRevision != null ? softwareRevision.hashCode() : 0);
        result = 31 * result + (manufacturerName != null ? manufacturerName.hashCode() : 0);
        return result;
    }
}
