package dk.lego.devicesdk.device;

import java.util.Collections;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import dk.lego.devicesdk.LDSDKError;
import dk.lego.devicesdk.utils.HandlerHelper;

class DeviceManagerCallbackHelper {

    private final Set<DeviceManagerCallbackListener> callbackListeners = Collections.newSetFromMap(new ConcurrentHashMap<DeviceManagerCallbackListener, Boolean>());

    protected void registerCallbackListener(DeviceManagerCallbackListener callback) {
        callbackListeners.add(callback);
    }

    protected void unregisterCallbackListener(DeviceManagerCallbackListener callback) {
        callbackListeners.remove(callback);
    }

    protected void performDeviceAppearedCallback(final LegoDevice legoDevice) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceManagerCallbackListener callbackListener : callbackListeners) {
                    callbackListener.onDeviceAppeared(legoDevice);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDeviceDisappearedCallback(final LegoDevice legoDevice) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceManagerCallbackListener callbackListener : callbackListeners) {
                    callbackListener.onDeviceDisappeared(legoDevice);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performWillStartConnectingToDeviceCallback(final LegoDevice legoDevice) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceManagerCallbackListener callbackListener : callbackListeners) {
                    callbackListener.onWillStartConnectingToDevice(legoDevice);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidStartInterrogatingDevice(final LegoDevice legoDevice) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceManagerCallbackListener callbackListener : callbackListeners) {
                    callbackListener.onDidStartInterrogatingDevice(legoDevice);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidFinishInterrogratingDevice(final LegoDevice legoDevice) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceManagerCallbackListener callbackListener : callbackListeners) {
                    callbackListener.onDidFinishInterrogatingDevice(legoDevice);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidDisconnectFromDevice(final LegoDevice legoDevice, final boolean autoReconnect, final LDSDKError error) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceManagerCallbackListener callbackListener : callbackListeners) {
                    callbackListener.onDidDisconnectFromDevice(legoDevice, autoReconnect, error);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    protected void performDidFailToConnectToDevice(final LegoDevice legoDevice, final boolean autoReconnect, final LDSDKError error) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                for (DeviceManagerCallbackListener callbackListener : callbackListeners) {
                    callbackListener.onDidFailToConnectToDevice(legoDevice, autoReconnect, error);
                }
            }
        };

        postRunnableFromSuitedHandler(runnable);
    }

    private void postRunnableFromSuitedHandler(Runnable deviceDisappeared) {
        HandlerHelper.getInstance().post(deviceDisappeared);
    }
}
