package dk.lego.devicesdk.device;

import java.util.ArrayList;
import java.util.List;

import dk.lego.devicesdk.bluetooth.DeviceCallbackHelper;
import dk.lego.devicesdk.logging.LDSDKLogger;
import dk.lego.devicesdk.services.LegoService;

public abstract class LegoDeviceImpl implements LegoDevice {

    public static final String UNDEFINED_DEVICE_ID = "Undefined";

    protected DeviceState connectState = DeviceState.DEVICE_CONNECTION_STATE_DISCONNECTED_NOT_ADVERTISING;
    protected String deviceId = UNDEFINED_DEVICE_ID;
    protected String name;
    protected boolean buttonPressed;
    protected Integer batteryLevel;
    protected DeviceInfo deviceInfo = new DeviceInfo();
    protected List<LegoService> services = new ArrayList<>();
    protected boolean lowVoltage = false;
    protected DeviceCategory category = DeviceCategory.DEVICE_CATEGORY_UNKNOWN;
    protected DeviceFunction supportedFunctions = DeviceFunction.DEVICE_FUNCTION_UNKNOWN;
    protected int lastConnectedNetworkId = -1;

    protected DeviceCallbackHelper callbackHelper = new DeviceCallbackHelper();

    public LegoDeviceImpl() {
    }

    @Override
    public List<LegoService> getServices() {
        return services;
    }

    @Override
    public List<LegoService> getInternalServices() {
        return getFilteredServices(true);
    }

    @Override
    public List<LegoService> getExternalServices() {
        return getFilteredServices(false);
    }

    @Override
    public DeviceState getConnectState() {
        return connectState;
    }

    @Override
    public String getDeviceId() {
        return deviceId;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public boolean isButtonPressed() {
        return buttonPressed;
    }

    @Override
    public Integer getBatteryLevel() {
        return batteryLevel;
    }

    @Override
    public DeviceInfo getDeviceInfo() {
        return deviceInfo;
    }

    @Override
    public void registerCallbackListener(DeviceCallbackListener listener) {
        if (listener != null) {
            callbackHelper.registerCallbackListener(listener);
        } else {
            LDSDKLogger.w("Ignoring attempt to add null callback listener to LegoDeviceImpl");
        }
    }

    @Override
    public void unregisterCallbackListener(DeviceCallbackListener listener) {
        if (listener != null) {
            callbackHelper.unregisterCallbackListener(listener);
        } else {
            LDSDKLogger.w("Ignoring attempt to remove null callback listener to LegoDeviceImpl");
        }
    }

    private List<LegoService> getFilteredServices(boolean internalOnly) {
        List<LegoService> filteredServices = new ArrayList<>();

        for (LegoService service : services) {
            if (service.isInternalService()) {
                if (internalOnly) {
                    filteredServices.add(service);
                }
            } else {
                if (!internalOnly) {
                    filteredServices.add(service);
                }
            }
        }
        return filteredServices;
    }
}
