package dk.lego.devicesdk.input_output;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import dk.lego.devicesdk.utils.ByteUtils;

public class InputCommand {

    private static final int HEADER_SIZE = 3;

    private static final int COMMAND_ID_INPUT_VALUE = 0;
    private static final int COMMAND_ID_INPUT_FORMAT = 1;

    private static final int COMMAND_TYPE_READ = 1;
    private static final int COMMAND_TYPE_WRITE = 2;

    private final byte[] data;

    private InputCommand(int commandId, int commandType, int connectId, byte[] payloadData) {
        ByteBuffer dataBuffer = ByteBuffer.allocate(HEADER_SIZE + payloadData.length).order(ByteOrder.LITTLE_ENDIAN);
        ByteUtils.putUnsignedByte(dataBuffer, commandId);
        ByteUtils.putUnsignedByte(dataBuffer, commandType);
        ByteUtils.putUnsignedByte(dataBuffer, connectId);
        dataBuffer.put(payloadData);

        this.data = dataBuffer.array();
    }

    public static InputCommand commandWriteInputFormat(InputFormat format, int connectId) {
        return new InputCommand(COMMAND_ID_INPUT_FORMAT, COMMAND_TYPE_WRITE, connectId, format.writeFormatData());
    }

    public static InputCommand commandReadInputFormatForConnectId(int connectId) {
        return new InputCommand(COMMAND_ID_INPUT_FORMAT, COMMAND_TYPE_READ, connectId, new byte[0]);
    }

    public static InputCommand commandReadValueForConnectId(int connectId) {
        return new InputCommand(COMMAND_ID_INPUT_VALUE, COMMAND_TYPE_READ, connectId, new byte[0]);
    }

    public byte[] getData() {
        return (data == null) ? null : data.clone();
    }
}
