package dk.lego.devicesdk.input_output;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import dk.lego.devicesdk.utils.ByteUtils;

public class OutputCommand {
    private static final byte HEADER_SIZE = 3;

    private static final byte WRITE_MOTOR_POWER_COMMAND_ID = 0x01;

    private static final byte PLAY_PIEZO_TONE_COMMAND_ID = 0x02;
    private static final byte STOP_PIEZO_TONE_COMMAND_ID = 0x03;

    private static final byte WRITE_RGB_COMMAND_ID = 0x04;

    private static final byte WRITE_DIRECT_ID = 0x05;

    public static final int COMMAND_PAYLOAD_SIZE_PIEZO = 4;
    public static final int COMMAND_PAYLOAD_SIZE_RGB_LIGHT = 3;

    private final byte[] data;

    private OutputCommand(int connectId, int commandId, byte[] payloadData) {
        byte[] tempData = new byte[HEADER_SIZE + payloadData.length];

        tempData[0] = (byte) connectId;
        tempData[1] = (byte) commandId;
        tempData[2] = (byte) payloadData.length;

        for (int i = HEADER_SIZE, j = 0; i < HEADER_SIZE + payloadData.length; i++, j++) {
            tempData[i] = payloadData[j];
        }

        this.data = tempData;
    }

    public static OutputCommand commandWriteMotorPower(int power, int connectId)
    {
        return new OutputCommand(connectId, WRITE_MOTOR_POWER_COMMAND_ID, ByteUtils.byteArrayFromByte(power));
    }

    public static OutputCommand commandWritePiezoToneFrequency(int frequency, int duration, int connectId)
    {
        ByteBuffer payloadBuffer = ByteBuffer.allocate(4).order(ByteOrder.LITTLE_ENDIAN);
        ByteUtils.putUnsignedShort(payloadBuffer, frequency);
        ByteUtils.putUnsignedShort(payloadBuffer, duration);

        return new OutputCommand(connectId, PLAY_PIEZO_TONE_COMMAND_ID, payloadBuffer.array());
    }

    public static OutputCommand commandWriteRGBLight(int red, int green, int blue, int connectId)
    {
        ByteBuffer payloadBuffer = ByteBuffer.allocate(3).order(ByteOrder.LITTLE_ENDIAN);
        ByteUtils.putUnsignedByte(payloadBuffer, red);
        ByteUtils.putUnsignedByte(payloadBuffer, green);
        ByteUtils.putUnsignedByte(payloadBuffer, blue);

        return new OutputCommand(connectId, WRITE_RGB_COMMAND_ID, payloadBuffer.array());
    }

    public static OutputCommand commandWriteRGBLightIndex(int index, int connectId)
    {
        ByteBuffer payloadBuffer = ByteBuffer.allocate(1).order(ByteOrder.LITTLE_ENDIAN);
        ByteUtils.putUnsignedByte(payloadBuffer, index);

        return new OutputCommand(connectId, WRITE_RGB_COMMAND_ID, payloadBuffer.array());
    }

    public static OutputCommand commandWithDirectWriteThroughData(byte[] data, int connectId)
    {
        return new OutputCommand(connectId, WRITE_DIRECT_ID, data);
    }

    public static OutputCommand commandWritePiezoToneStopForConnectId(int connectId)
    {
        return new OutputCommand(connectId, STOP_PIEZO_TONE_COMMAND_ID, new byte[]{});
    }

    public byte[] getData() {
        return data.clone();
    }
}
