package dk.lego.devicesdk.logging;

/**
 * You may provide an implementation of this interface to the {@link dk.lego.devicesdk.logging.LDSDKLogger} to have all
 * log from the LEGO Device SDK written to a custom destination (for instance a remote logging server).
 */
public interface CustomLogger {
    /**
     * Used to log detailed information. This information will probably only be useful during development and debugging.
     *
     * @param msg the message to log
     */
    public void verbose(String msg);

    /**
     * Used to log detailed information. This information will probably only be useful during development and debugging.
     *
     * @param msg the message to log
     */
    public void debug(String msg);

    /**
     * Used to log information on the flow through the system and other interesting events.
     * @param msg the message to log
     */
    public void info(String msg);

    /**
     * Used in situations like use of deprecated APIs, poor use of API, near errors, other runtime situations that are undesirable or unexpected, but not necessarily "wrong".
     * @param msg the message to log
     */
    public void warn(String msg);

    /**
     * Used to log runtime errors or unexpected conditions.
     * @param msg the message to log
     */
    public void error(String msg);


    /**
     * Used to log runtime errors or unexpected conditions.
     * @param e the exception that was thrown that caused the error.
     */
    public void error(Exception e);
}