package dk.lego.devicesdk.services;

import dk.lego.devicesdk.LDSDKError;
import dk.lego.devicesdk.device.ConnectInfo;
import dk.lego.devicesdk.input_output.IO;
import dk.lego.devicesdk.input_output.InputFormat;
import dk.lego.devicesdk.logging.LDSDKLogger;

public class CurrentSensor extends LegoService {

    private static final String SERVICE_CURRENT_SENSOR_NAME = "Current Sensor";

    private CurrentSensor(ConnectInfo connectInfo, IO io) {
        super(connectInfo, io);
    }

    @Override
    public String getServiceName() {
        return SERVICE_CURRENT_SENSOR_NAME;
    }

    @Override
    public InputFormat getDefaultInputFormat() {
        // Only one available mode for current sensor
        return InputFormat.inputFormat(getConnectInfo().getConnectId(), getConnectInfo().getType(), 0, 30, InputFormat.InputFormatUnit.INPUT_FORMAT_UNIT_SI, true);
    }

    public static CurrentSensor createService(ConnectInfo connectInfo, IO io) {
        return new CurrentSensor(connectInfo, io);
    }

    public float getValueAsMilliAmps()
    {
        if (this.inputFormat.getMode() == 0 && this.inputFormat.getUnit() == InputFormat.InputFormatUnit.INPUT_FORMAT_UNIT_SI) {
            return this.getValueAsFloat();
        } else {
            LDSDKLogger.w("Can only retrieve milli amps from Current Sensor when sensor is in mode 0 and uses SI units");
            return 0;
        }
    }

    @Override
    public void handleUpdatedValueData(byte[] valueData) throws LDSDKError {
        super.handleUpdatedValueData(valueData);
        callbackHelper.performDidUpdateMilliAmps(this, this.getValueAsMilliAmps());
    }
}
