package dk.lego.devicesdk.services;

import dk.lego.devicesdk.device.ConnectInfo;
import dk.lego.devicesdk.device.LegoDevice;
import dk.lego.devicesdk.input_output.IO;
import dk.lego.devicesdk.logging.LDSDKLogger;

public class LegoServiceFactory {

    public static LegoService create(ConnectInfo connectInfo, IO io, LegoDevice device) {
        if (io == null || connectInfo == null) {
            LDSDKLogger.e("Cannot instantiate service with null IO object or ConnectInfo");
            return null;
        }

        LegoService result;
        switch (connectInfo.getTypeEnum()) {
            case IO_TYPE_GENERIC:
                result = GenericService.createService(connectInfo, io);
                break;
            case IO_TYPE_MOTION_SENSOR:
                result = MotionSensor.createService(connectInfo, io);
                break;
            case IO_TYPE_TILT_SENSOR:
                result = TiltSensor.createService(connectInfo, io);
                break;
            case IO_TYPE_PIEZO_TONE_PLAYER:
                result = PiezoTonePlayer.createService(connectInfo, io);
                break;
            case IO_TYPE_MOTOR:
                result = Motor.createService(connectInfo, io);
                break;
            case IO_TYPE_CURRENT:
                result = CurrentSensor.createService(connectInfo, io);
                break;
            case IO_TYPE_VOLTAGE:
                result = VoltageSensor.createService(connectInfo, io);
                break;
            case IO_TYPE_RGB_LIGHT:
                result = RGBLight.createService(connectInfo, io);
                break;
            default:
                result = GenericService.createService(connectInfo, io);
                break;
        }

        result.setDevice(device);

        return result;
    }
}
