package dk.lego.devicesdk.services;

/**
 * Implement this listener to be notified when the {@link dk.lego.devicesdk.services.RGBLight} updates its value
 */
public interface RGBLightCallbackListener extends ServiceCallbackListener {

    /**
     * Invoked when the first (initial) color of the RGB Light is read
     *
     * @param rgbLight The RGB light
     */
    public void didReadInitialColor(RGBLight rgbLight);

    /**
     * Invoked when the RGBLight service receives an updated value
     * Will only be invoked when the RGB light is in the absolute mode
     *
     * @param rgbLight The RGB light
     * @param oldRGBColor The previous color
     * @param newRGBColor The new color
     */
    public void didUpdateRGBValue(RGBLight rgbLight, Integer oldRGBColor, Integer newRGBColor);

    /**
     * Invoked when the RGBLight service receives an updated color index
     * Will only be invoked when the RGB light is in the discrete mode
     *
     * @param rgbLight The RGB light
     * @param oldColorIndex The previous color index
     * @param newColorIndex The new color index
     */
    public void didUpdateRGBColorIndex(RGBLight rgbLight, int oldColorIndex, int newColorIndex);
}
