package dk.lego.devicesdk.services;

import dk.lego.devicesdk.input_output.InputFormat;

/**
 * Implement this listener to be notified when a service (typically a sensor of some kind) sends an updated value.
 */
public interface ServiceCallbackListener {
    /**
     * Invoked when a service receives an updated value.
     * You can use of the convenience methods {@link dk.lego.devicesdk.services.LegoService#getNumberFromValueData},
     * {@link dk.lego.devicesdk.services.LegoService#getValueAsInteger} or {@link dk.lego.devicesdk.services.LegoService#getValueAsFloat}
     * to retrieve the value as a number.
     *
     * @param service  The service that received an updated value
     * @param oldValue The previous value
     * @param newValue The new value
     */
    void didUpdateValueData(LegoService service, byte[] oldValue, byte[] newValue);

    /**
     * Invoked when a service receives an updated {@link dk.lego.devicesdk.input_output.InputFormat}
     *
     * @param service   The service that received an updated value
     * @param oldFormat The previous input format
     * @param newFormat The new input format
     */
    void didUpdateInputFormat(LegoService service, InputFormat oldFormat, InputFormat newFormat);
}
