package dk.lego.devicesdk.services;

import dk.lego.devicesdk.LDSDKError;
import dk.lego.devicesdk.device.ConnectInfo;
import dk.lego.devicesdk.input_output.IO;
import dk.lego.devicesdk.input_output.InputFormat;
import dk.lego.devicesdk.logging.LDSDKLogger;

public class VoltageSensor extends LegoService {

    private static final String SERVICE_VOLTAGE_SENSOR_NAME = "Voltage Sensor";

    private VoltageSensor(ConnectInfo connectInfo, IO io) {
        super(connectInfo, io);
    }

    @Override
    public String getServiceName() {
        return SERVICE_VOLTAGE_SENSOR_NAME;
    }

    @Override
    public InputFormat getDefaultInputFormat() {
        // Only one available mode for voltage sensor
        return InputFormat.inputFormat(getConnectInfo().getConnectId(), getConnectInfo().getType(), 0, 30, InputFormat.InputFormatUnit.INPUT_FORMAT_UNIT_SI, true);
    }

    public static VoltageSensor createService(ConnectInfo connectInfo, IO io) {
        return new VoltageSensor(connectInfo, io);
    }

    /**
     * @return The battery voltage in milli volts
     */
    public float getValueAsMilliVolts() {
        if (this.inputFormat.getMode() == 0 && this.inputFormat.getUnit() == InputFormat.InputFormatUnit.INPUT_FORMAT_UNIT_SI) {
            return this.getValueAsFloat();
        } else {
            LDSDKLogger.w("Can only retrieve milli volts from Voltage Sensor when sensor is in mode 0 and uses SI units");
            return 0;
        }
    }

    @Override
    public void handleUpdatedValueData(byte[] valueData) throws LDSDKError {
        super.handleUpdatedValueData(valueData);
        callbackHelper.performDidUpdateMilliVolts(this, this.getValueAsMilliVolts());
    }
}
