package dk.lego.devicesdk.unity;

import dk.lego.devicesdk.LDSDKError;
import dk.lego.devicesdk.device.DeviceCallbackListener;
import dk.lego.devicesdk.device.DeviceInfo;
import dk.lego.devicesdk.device.LegoDevice;
import dk.lego.devicesdk.services.LegoService;

public class LegoDeviceWrapperImpl implements LegoDeviceWrapper, DeviceCallbackListener  {

    private static LegoDeviceWrapperImpl instance = null;

    private LegoDeviceManagerWrapperImpl deviceManagerWrapper;
    private LegoServiceWrapperImpl serviceWrapper;
    private WrapperSerialization wrapperSerialization;
    private UnityInvoker unityInvoker;

    public static LegoDeviceWrapperImpl getInstance() {
        if (instance == null) {
            instance = new LegoDeviceWrapperImpl();
        }
        return instance;
    }

    private LegoDeviceWrapperImpl() {
        this(LegoDeviceManagerWrapperImpl.getInstance(), LegoServiceWrapperImpl.getInstance(), new WrapperSerialization(), new UnityInvoker());
    }

    LegoDeviceWrapperImpl(LegoDeviceManagerWrapperImpl deviceManagerWrapper, LegoServiceWrapperImpl serviceWrapper, WrapperSerialization wrapperSerialization, UnityInvoker unityInvoker) {
        this.deviceManagerWrapper = deviceManagerWrapper;
        this.serviceWrapper = serviceWrapper;
        this.wrapperSerialization = wrapperSerialization;
        this.unityInvoker = unityInvoker;
    }

    @Override
    public void changeDeviceName(String legoDeviceID, String deviceName) {
        LegoDevice device = deviceManagerWrapper.findDevice(legoDeviceID);
        if (device != null) {
            device.setName(deviceName);
        }
    }

    @Override
    public void didUpdateDeviceInfo(LegoDevice device, DeviceInfo deviceInfo, LDSDKError error) {
        unityInvoker.invokeMethod(UnitySDKCallbacks.LEDeviceDidUpdateDeviceInfo, wrapperSerialization.serializeDevice(device));
    }

    @Override
    public void didChangeNameFrom(LegoDevice device, String oldName, String newName) {
        unityInvoker.invokeMethod(UnitySDKCallbacks.LEDeviceDidChangeName, wrapperSerialization.serializeDeviceNameChange(device, oldName, newName));
    }

    @Override
    public void didChangeButtonState(LegoDevice device, boolean pressed) {
        unityInvoker.invokeMethod(UnitySDKCallbacks.LEDeviceDidChangeButtonState, wrapperSerialization.serializeDeviceButtonStateChange(device, pressed));
    }

    @Override
    public void didUpdateBatteryLevel(LegoDevice device, int newLevel) {
        unityInvoker.invokeMethod(UnitySDKCallbacks.LEDeviceDidUpdateBatteryLevel, wrapperSerialization.serializeDeviceBatteryLevel(device, newLevel));
    }

    @Override
    public void didUpdateLowVoltageState(LegoDevice device, boolean lowVoltage) {
        unityInvoker.invokeMethod(UnitySDKCallbacks.LEDeviceDidUpdateLowVoltageState, wrapperSerialization.serializeDeviceLowVoltage(device, lowVoltage));
    }

    @Override
    public void didAddService(LegoDevice device, LegoService service) {
        service.registerCallbackListener(serviceWrapper);
        unityInvoker.invokeMethod(UnitySDKCallbacks.LEDeviceDidAddService, wrapperSerialization.serializeService(service));
    }

    @Override
    public void didRemoveService(LegoDevice device, LegoService service) {
        service.unregisterCallbackListener(serviceWrapper);
        unityInvoker.invokeMethod(UnitySDKCallbacks.LEDeviceDidRemoveService, wrapperSerialization.serializeService(service, true));
    }

    @Override
    public void didFailToAddServiceWithError(LegoDevice device, LDSDKError error) {
        unityInvoker.invokeMethod(UnitySDKCallbacks.LEDeviceDidFailToAddServiceWithError, wrapperSerialization.serializeDeviceError(device, error));
    }

}
