#!/bin/bash

# This script compiles Android wrapper for the LEGO Device SDK that can be used by Unity 
# and copies them to the Unity plugins directory.
#
# By default the resulting wrappers is copied to Unity plugins folder which is expected
# to be contained by a 'LEGO-Device-SDK-Unity' directory at the same level as this repository.
# You can specify a different destination path as the first argument though.

UNITY_PLUGIN_DIRECTORY=${1:-`pwd`/../LEGO-Device-SDK-Unity/Unity/Assets/Plugins}
ANDROID_PLUGIN_DIR="${UNITY_PLUGIN_DIRECTORY}/Android"
UNITY_NATIVE_SDK_DIRECTORY=${UNITY_PLUGIN_DIRECTORY}/../Scripts/LEGODeviceUnitySDK/NativeSDK/Android

# Clean up existing plugin directory
rm -rf "${ANDROID_PLUGIN_DIR}"
if [ $? -ne 0 ];
then
	echo "Failed to clean up plugin directory '${ANDROID_PLUGIN_DIR}'"
	exit 1
fi

# Create Unity plugins folder
mkdir -p "${ANDROID_PLUGIN_DIR}"
if [ $? -ne 0 ];
then
	echo "Failed to create plugins directory '${ANDROID_PLUGIN_DIR}'"
	exit 1
fi

# Build Android wrapper
echo "Building Android wrapper..."
./gradlew :LegoDeviceSDK:clean :LegoDeviceSDK:compileReleaseSources :LegoDeviceSDK:deploy
if [ $? -ne 0 ];
then
	echo "Failed to build Android wrapper"
	exit 1
fi

# Copy Android wrapper
cp build/deploy/libs/LegoDeviceSDK.jar "${ANDROID_PLUGIN_DIR}"
if [ $? -ne 0 ];
then
	echo "Failed to copy Android wrapper"
	exit 1
fi

# Copy wrapper's dependecies
cp LegoDeviceSDK/libs/*.jar "${ANDROID_PLUGIN_DIR}"
if [ $? -ne 0 ];
then
	echo "Failed to copy wrapper's dependencies"
	exit 1
fi

# Copy interface files containing native Android API listing
UNITY_WRAPPERS_DIR=LegoDeviceSDK/src/main/java/dk/lego/devicesdk/unity
cp "${UNITY_WRAPPERS_DIR}/LegoDeviceManagerWrapper.java" "${UNITY_NATIVE_SDK_DIRECTORY}"
cp "${UNITY_WRAPPERS_DIR}/LegoDeviceWrapper.java" "${UNITY_NATIVE_SDK_DIRECTORY}"
cp "${UNITY_WRAPPERS_DIR}/LegoServiceWrapper.java" "${UNITY_NATIVE_SDK_DIRECTORY}"
cp "${UNITY_WRAPPERS_DIR}/LegoLoggerWrapper.java" "${UNITY_NATIVE_SDK_DIRECTORY}"

# Cleanup
echo "Cleaning up..."
rm -rf build

# Write success in bold
echo
echo -e "\033[1mSUCCESS\033[0m"
echo -e "\033[1mBuilt Android wrapper\033[0m"
echo
